/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;
import java.io.*;

public class CaptureFile
{
	private String m_name = "";
	private String m_originalName = "";
	private String commandTemplate = "";
	private Vector<String> fullPathTemplates = new Vector<String>();
	private String m_directory = "";
	private String m_originalDirectory = "";
	private int m_encoding = Formatter.TYPE_UNFORMATTED;
	private String m_changeChannelTemplate = "";
	private String m_changeSettingTemplate = "";
	private String m_clientGetAddressTemplate = "";
	private boolean m_bSeekable = false;
	private int m_streamPort = -2;

	public CaptureFile()
	{

	}

	public void setEncoding(int encoding)
	{
		m_encoding = encoding;
	}

	public String getName()
	{
		return Formatter.decoded(m_name, m_encoding);
	}

	public String getUnformattedName()
	{
		return m_name;
	}

	public String getOriginalName()
	{
		return Formatter.decoded(m_originalName, m_encoding);
	}

	public String getEncodedName()
	{
		return Formatter.encode(m_name, m_encoding);
	}

	public String getDirectory()
	{
		return Formatter.decoded(m_directory, m_encoding);
	}

	public String getUnformattedOriginalDirectory()
	{
		return m_originalDirectory;
	}

	public String getUnformattedDirectory()
	{
		return m_directory;
	}

	public String getEncodedDirectory()
	{
		return Formatter.encode(m_directory, m_encoding);
	}

	public String getDirectoryAndName()
	{
		return getDirectory() + File.separator + getName();
	}

	public String getDirectoryAndOriginalName()
	{
		return getDirectory() + File.separator + getOriginalName();
	}

	public String getEncodedDirectoryAndName()
	{
		return Formatter.encode(getDirectoryAndName(), m_encoding);
	}

	public int getFilenameCount()
	{
		return fullPathTemplates.size();
	}

	public String getFullPath(int index)
	{
		return fullPathTemplates.get(index).replace("$name", getDirectoryAndName());
	}

	public String getFullPath()
	{
		if (fullPathTemplates.isEmpty())
			return getDirectoryAndName();
		else
			return getFullPath(0);
	}

	public String getEncodedFullPath(int index)
	{
		return Formatter.encode(getFullPath(index), m_encoding);
	}

	public String getEncodedFullPath()
	{
		if (fullPathTemplates.isEmpty())
			return getEncodedDirectoryAndName();
		else
			return getEncodedFullPath(0);
	}

	public String getFilename(int index)
	{
		return fullPathTemplates.get(index).replace("$name", getName());
	}

	public String getFilename()
	{
		if (fullPathTemplates.isEmpty())
			return getName();
		else
			return getFilename(0);
	}

	public String getEncodedFilename(int index)
	{
		return Formatter.encode(getFilename(index), m_encoding);
	}

	public String getEncodedFilename()
	{
		if (fullPathTemplates.isEmpty())
			return getEncodedName();
		else
			return getEncodedFilename(0);
	}

	public String getCommandLine(CaptureEngine captureEngine)
	{
		//return programPID + "," + videoPID + "," + audioPID + "," + audioTYPE + ">" + fileName;
		//return commandTemplate.replace("$name", fileName.replace("'", "''"));
		return commandTemplate.replace("$name", captureEngine.formatFileName(getEncodedDirectoryAndName()));
	}

	public String getChangeChannelCommand(CaptureEngine captureEngine)
	{
		return m_changeChannelTemplate.replace("$name", captureEngine.formatFileName(getEncodedDirectoryAndName()));
	}

	public String getChangeSettingCommand(CaptureEngine captureEngine)
	{
		return m_changeSettingTemplate.replace("$name", captureEngine.formatFileName(getEncodedDirectoryAndName()));
	}

	public String getClientGetAddress(CaptureEngine captureEngine)
	{
		return m_clientGetAddressTemplate.replace("$name", captureEngine.formatFileName(getEncodedDirectoryAndName()));
	}

	public void setName(String name)
	{
		m_name = Formatter.formatDirectory(name);
		//m_name = Formatter.formatAscii2(m_name);
	}

	public void setOriginalName(String name)
	{
		m_originalName = Formatter.formatDirectory(name);
		//m_originalName = Formatter.formatAscii2(m_originalName);
	}

	public void setDirectory(String directory)
	{
		m_directory = Formatter.formatDriveDirectory(directory);

		while ( m_directory.endsWith(File.separator) )
			m_directory = m_directory.substring(0, m_directory.length()-1);

		//m_directory = Formatter.formatAscii2(m_directory);
	}

	public void setOriginalDirectory(String directory)
	{
		m_originalDirectory = directory;
	}

	public void setCommandTemplate(String commandTemp)
	{
		commandTemplate = commandTemp;
	}

	public void setChangeChannelTemplate(String changeChannelTemplate)
	{
		m_changeChannelTemplate = changeChannelTemplate;
	}

	public void setChangeSettingTemplate(String changeSettingTemplate)
	{
		m_changeSettingTemplate = changeSettingTemplate;
	}

	public void setClientGetAddressTemplate(String clientGetAddressTemplate)
	{
		m_clientGetAddressTemplate = clientGetAddressTemplate;
	}

	public void setSeekable(boolean bSeekable)
	{
		m_bSeekable = bSeekable;
	}

	public boolean isSeekable()
	{
		return m_bSeekable;
	}

	public void addFullPathTemplate(String fullPathTemp)
	{
		fullPathTemplates.add(fullPathTemp);
	}

	public void setFullPathTemplates(Vector<String> fullPathTemps)
	{
		fullPathTemplates = fullPathTemps;
	}

	public void setStreamPort(int streamPort)
	{
		m_streamPort = streamPort;
	}

	public int getStreamPort()
	{
		return m_streamPort;
	}

	public String replaceKeywords(String text)
	{
		text = text.replace("$streamport", m_streamPort < 0 ? "8006" : Integer.toString(m_streamPort));
		text = text.replace("$name", getDirectoryAndName());
		text = text.replace("$originalname", getDirectoryAndOriginalName());
		text = text.replace("$filenameonly", getFilename());
		text = text.replace("$encfilenameonly", getEncodedFilename());
		text = text.replace("$filename", getFullPath());
		text = text.replace("$encfilename", getEncodedFullPath());
		text = text.replace("$enccapturepath", getEncodedDirectory());
		return text.replace("$capturepath", getDirectory());
	}
}
