/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

class ChannelDataRes extends HTTPResponse
{

	public ChannelDataRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if("01".equals(action))
		{
			m_outStream.write(showCountryList());
			return;
		}
		else if("02".equals(action))
		{
			m_outStream.write(showRegionList());
			return;
		}
		else if("03".equals(action))
		{
			m_outStream.write(showStationList());
			return;
		}
		else if("04".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(showScanResult());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}
		else if("05".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(addSelectedPrograms());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}
		else if("06".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(deleteChannel());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}			
			return;
		}
		else if("07".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				scanAll(true);
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}
		/*
		else if("08".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				rescanAll(true);
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}
		 */
		else if("09".equals(action))
		{
			m_outStream.write(showAddEditForm());
			return;
		}
		else if("10".equals(action))
		{
			m_outStream.write(exportChannelList());
			return;
		}
		else if("11".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(updateChannel());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}      
		else if("12".equals(action))
		{
			m_outStream.write(showAddCustomForm());
			return;
		}      
		else if("13".equals(action))
		{
			String scan = m_urlData.getParameter("scan");

			ThreadLock.getInstance().getLock();
			try
			{
				if (scan != null && scan.equals("Scan"))
					m_outStream.write(showScanResult());
				else
					m_outStream.write(updateChannel());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}

			return;
		}
		else if("14".equals(action))
		{
			m_outStream.write(showChannelOrder(m_store.m_channelOrder));
			return;
		}
		else if("15".equals(action))
		{
			m_outStream.write(resetCustomOrder());
			return;
		}
		else if("16".equals(action))
		{
			m_outStream.write(sortCustomOrder());
			return;
		}
		else if("17".equals(action))
		{
			m_outStream.write(updateCustomOrder());
			return;
		}
		else if("18".equals(action))
		{
			m_outStream.write(addMissingToCustomOrder());
			return;
		}
		else if("19".equals(action))
		{
			m_outStream.write(showChannelGroup());
			return;
		}
		else if("20".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(addUpdateChannelGroup());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}
		else if("21".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				scanAll(false);
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
			return;
		}
		else if("22".equals(action))
		{
			ThreadLock.getInstance().getLock();
			try
			{
				m_outStream.write(deleteAllChannels());
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}			
			return;
		}
		else if("23".equals(action))
		{
			m_outStream.write(parseStationList());
			return;
		}
		else if("24".equals(action))
		{
			m_outStream.write(showChannels(true));
			return;
		}
		else if("25".equals(action))
		{
			m_outStream.write(showChannels(false));
			return;
		}
		else if("26".equals(action))
		{
			m_outStream.write(updateDefaultData());
			return;
		}
		else
		{
			m_outStream.write(showChannels());
			return;
		}
	}

	private byte[] exportChannelList() throws Exception
	{
		StringBuffer buff = new StringBuffer();

		buff.append("HTTP/1.0 200 OK\nContent-Type: text/xml\n");
		buff.append("Content-Disposition: attachment; filename=\"channels.xml\"\n");
		buff.append("Pragma: no-cache\n");
		buff.append("Cache-Control: no-cache\n");
		buff.append("\n");

		m_store.saveChannels(buff);

		return buff.toString().getBytes("UTF-8");
	}

	private byte[] xmlChannelSetting(Channel channel) throws Exception
	{
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "channel", null);
		Element root = doc.getDocumentElement();
		if (channel == null)
		{
			channel = new Channel();
			channel.setDemodulator(m_store.getCurrentChannel());	// Only want demodulator data.
			root.setAttribute("update", "0");
		}
		else
			root.setAttribute("update", "1");

		channel.getXML(doc, root);

		Element elm = doc.createElement("allNetworkTypes");
		m_store.getNetworkTypeXml(doc, elm, false);
		root.appendChild(elm);

		elm = doc.createElement("allCaptureTypes");
		m_store.getCaptureTypesXml(doc, elm);
		root.appendChild(elm);

		elm = doc.createElement("allPreviewTypes");
		m_store.getPreviewTypesXml(doc, elm);
		root.appendChild(elm);

		elm = doc.createElement("allPreviewRemoteTypes");
		m_store.getPreviewRemoteTypesXml(doc, elm);
		root.appendChild(elm);

		elm = doc.createElement("allSurfTypes");
		m_store.getSurfTypesXml(doc, elm);
		root.appendChild(elm);

		elm = doc.createElement("allSurfRemoteTypes");
		m_store.getSurfRemoteTypesXml(doc, elm);
		root.appendChild(elm);

		elm = doc.createElement("allCapturePaths");
		m_store.getCapturePathsXml(doc, elm);
		root.appendChild(elm);

		elm = doc.createElement("allVideoTypes");
		channel.getVideoTypesXml(doc, elm);
		root.appendChild(elm);

		elm = doc.createElement("allAudioTypes");
		channel.getAudioTypesXml(doc, elm);
		root.appendChild(elm);

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "channel-details.xsl", m_headers, m_urlData);

		return transformer.transform();
	}

	private byte[] showAddEditForm() throws Exception
	{
		String name = m_urlData.getParameter("ID");

		HashMap<String, Channel> channels = m_store.getChannels();
		Channel channel = null;

		if (name != null)
		{
			channel = (Channel) channels.get(name);
		}

		return xmlChannelSetting(channel);
	}

	private byte[] showAddCustomForm() throws Exception
	{
		return xmlChannelSetting(null);
	}

	protected String GetScanCommand(Channel ch, CaptureDevice captureDevice, CaptureEngine captureEngine)
	{
		String fullPath = m_store.getProperty("Capture.engine.path", "util") + File.separator + "scan.ce";
		if ( !captureEngine.SetEngine("Generic", fullPath) )
		{
			DVScheduler.log("Error reading scan engine file (" + fullPath + ")");

			return null;
		}

		String command = captureEngine.getInitialScanCommand();
		command = captureEngine.replaceKeywords(command);	// Has to be first.
		command = captureDevice.replaceKeywords(command);
		command = m_store.replaceKeywords(command);
		command = ch.replaceKeywords(command);

		return Formatter.formatCommand(command);
	}

	/*
	private void rescanAll()
	throws Exception
	{
	}
	 */

	private void scanAll(boolean bScanAll)
			throws Exception
			{
		int country = 0;
		int region = 0;
		try
		{
			country = Integer.parseInt(m_urlData.getParameter("country"));
			region = Integer.parseInt(m_urlData.getParameter("region"));

		}
		catch (Exception e)
		{
			throw new Exception("country or region code not valid: "
					+ e.toString());
		}

		String networkType = m_urlData.getParameter("networkType");
		if ( (networkType == null) || (networkType.isEmpty()) )
			networkType = "DVB-T";

		PageTemplate template = new PageTemplate(m_headers, m_urlData, "channelscan.html");
		m_outStream.write(template.getPageBytes(0));

		CaptureDeviceList devList = CaptureDeviceList.getInstance();

		if(devList.getRunningDeviceCount() > 0)
		{
			m_outStream.write("Can not scan channels, Captures Running!".getBytes("UTF-8"));
			return;
		}

		if (devList.getDeviceCount(networkType) == 0)
		{
			m_outStream.write("No Suitable Devices Available!".getBytes("UTF-8"));
			return;
		}

		String out = "<form action=\"/servlet/" + m_urlData.getServletClass()
				+ "\" method=\"POST\" accept-charset=\"UTF-8\">\n"
				+ "<input type=\"hidden\" name=\"action\" value='05'>\n";

		m_outStream.write(out.getBytes("UTF-8"));
		m_outStream.flush();

		ChannelList list = new ChannelList(null, networkType);
		Channel[] channelList = list.getStations(country, region);
		list.close();

		try
		{
			int channelCount = 0;
			Channel ch = null;
			String logText = "<table><tr><td style=\"font-size:80%\"><b>Scan Log:</b><br>";
			for (int x = 0; x < channelList.length; x++)
			{
				if ( !bScanAll && (m_urlData.getParameter("add" + x) == null) )
					continue;

				int channelsFound = 0;
				ch = channelList[x];
				if(ch != null)
				{
					StringBuffer buff = new StringBuffer(2048);

					buff.append("<table class='channelScanResult'>\n");

					buff.append("<tr class=\"scanChannelHeading\">\n");
					buff.append("<td class=\"scanChannelHeadingDataName\">"
							+ HTMLEncoder.encode(ch.getName()) + "</td>\n");
					buff.append("<td class='scanChannelHeadingData'>Access</td>");
					buff.append("<td class=\"scanChannelHeadingData\">Program</td>\n");
					buff.append("<td class=\"scanChannelHeadingData\">Video</td>\n");
					buff.append("<td class=\"scanChannelHeadingData\">Audio</td>\n");
					buff.append("<td class=\"scanChannelHeadingData\">Add</td>\n");
					buff.append("</tr>\n");

					// do not delay on first scan
					if(x > 0)
						Thread.sleep(1000);

					CaptureDevice captureDevice = (CaptureDevice)devList.GetFirstDevice(networkType);
					if (captureDevice == null)
						throw new Exception("No suitable devices found");
					CaptureEngine captureEngine = new CaptureEngine();
					String scanCommand = GetScanCommand(ch, captureDevice, captureEngine);
					if (scanCommand == null)
						throw new Exception("Scan command not found in capture engine path");
					DVScheduler.log("Running channel scan command: " + scanCommand);

					Runtime runner = Runtime.getRuntime();

					Process scan = runner.exec(Formatter.parseCommand(scanCommand));

					ScanResult result = new ScanResult(ch, captureEngine);
					result.readInput(scan.getInputStream());
					result.parseXML();
					scan.waitFor();
					logText += "<br>" + result.getLog().replaceAll("\n", "<br>");

					Vector<?> scanResult = result.getResult();
					Channel chData = null;

					for (int y = 0; y < scanResult.size(); y++)
					{
						chData = (Channel) scanResult.get(y);

						if(chData != null)
						{
							buff.append("<tr class='scanChannelResult'>");

							buff.append("<td>\n<input type='text' size='20' name='name" + channelCount
									+ "' value='" + HTMLEncoder.encode(checkName(chData.getName())) + "'>\n");

							buff.append("<input type='hidden' name='networkType" + channelCount
									+ "' value='" + ch.getNetworkType() + "'>\n");

							buff.append("<input type='hidden' name='freq" + channelCount
									+ "' value='" + chData.getFrequency() + "'>\n");

							buff.append("<input type='hidden' name='band" + channelCount
									+ "' value='" + chData.getBandWidth() + "'>\n");

							buff.append("<input type='hidden' name='demodulatorName" + channelCount
									+ "' value='" + HTMLEncoder.encode(checkAlias(chData.getDemodulatorName())) + "'>\n");

							buff.append("<input type='hidden' name='originalNetworkId" + channelCount
									+ "' value='" + chData.getOriginalNetworkId() + "'>\n");

							buff.append("<input type='hidden' name='transportStreamId" + channelCount
									+ "' value='" + chData.getTransportStreamId() + "'>\n");

							buff.append("<input type='hidden' name='sourceId" + channelCount
									+ "' value='" + chData.getSourceId() + "'>\n");

							buff.append("<input type='hidden' name='pids" + channelCount 
									+ "' value='" + getPids(chData) + "'>\n");

							buff.append("<input type='hidden' name='teletextid" + channelCount
									+ "' value='" + chData.getTeletextPid() + "'>\n");

							buff.append("<input type='hidden' name='teletextpage" + channelCount
									+ "' value='" + chData.getTeletextSubtitlePage() + "'>\n");

							int[] streamData = getFirstOfType(chData, Channel.TYPE_PMT);
							if (streamData == null)
							{
								buff.append("<input type='hidden' name='pmtid" + channelCount
										+ "' value='-1'>\n");
							}
							else
							{
								buff.append("<input type='hidden' name='pmtid" + channelCount
										+ "' value='" + streamData[0] + "'>\n");
							}

							streamData = getFirstOfType(chData, Channel.TYPE_PCR);
							if (streamData == null)
							{
								buff.append("<input type='hidden' name='pcrid" + channelCount
										+ "' value='-1'>\n");
							}
							else
							{
								buff.append("<input type='hidden' name='pcrid" + channelCount
										+ "' value='" + streamData[0] + "'>\n");
							}

							buff.append("</td>\n\n");

							buff.append("<td align='center'>" + chData.getAccessStr());
							buff.append("<input type='hidden' name='access" + channelCount 
									+ "' value='" + chData.getAccess() + "'></td>\n"); 

							buff.append("<td align='center'>" + chData.getProgramID());
							buff.append("<input type='hidden' name='programid"
									+ channelCount + "' value='" + chData.getProgramID()
									+ "'>\n"); 
							buff.append("</td>\n");

							//
							// do the video pid
							//
							buff.append("<td align='center'>");
							int count = getStreamTypeCount(chData, Channel.TYPE_VIDEO_MPG) + getStreamTypeCount(chData, Channel.TYPE_VIDEO_H264);
							if(count == 0)
							{
								buff.append("-");
								buff.append("<input type='hidden' name='videoid" + channelCount
										+ "' value='-1'>\n");                  
							}
							if(count == 1)
							{
								int[] data = getFirstOfType(chData, Channel.TYPE_VIDEO_MPG);
								if(data == null)
									data = getFirstOfType(chData, Channel.TYPE_VIDEO_H264);
								buff.append(data[0]);
								if(data[1] == Channel.TYPE_VIDEO_H264)
									buff.append(" H264");
								else
									buff.append(" MPG");
								buff.append("<input type='hidden' name='videoid" + channelCount
										+ "' value='" + data[0] + ":" + data[1] + "'>\n");
							}
							else if(count > 1)
							{
								buff.append("<SELECT NAME='videoid" + channelCount + "'>\n");
								for(int st = 0; st < chData.getStreams().size(); st++)
								{
									streamData = (int[])chData.getStreams().get(st);
									if(streamData[1] == Channel.TYPE_VIDEO_MPG || streamData[1] == Channel.TYPE_VIDEO_H264)
									{
										buff.append("<OPTION value=\"" + streamData[0] + ":" + streamData[1] + "\">");
										buff.append(streamData[0]);
										if(streamData[1] == Channel.TYPE_VIDEO_H264)
											buff.append(" H264");
										else
											buff.append(" MPG");                        
										buff.append("</OPTION>\n");
									}
								}                  
								buff.append("</SELECT>");
							}
							buff.append("</td>\n");

							//
							// do the audio pid
							//
							buff.append("<td align='center'>");
							count = getStreamTypeCount(chData, Channel.TYPE_AUDIO_AC3);
							count += getStreamTypeCount(chData, Channel.TYPE_AUDIO_MPG);
							count += getStreamTypeCount(chData, Channel.TYPE_AUDIO_AAC);
							if(count == 0)
							{
								buff.append("-");
								buff.append("<input type='hidden' name='audioid" + channelCount
										+ "' value='-1'>\n");                  
							}                      
							if(count == 1)
							{
								int[] data = getFirstOfType(chData, Channel.TYPE_AUDIO_AC3);
								if(data == null)
									data =  getFirstOfType(chData, Channel.TYPE_AUDIO_AAC);
								if(data == null)
									data =  getFirstOfType(chData, Channel.TYPE_AUDIO_MPG);
								buff.append(data[0]);
								if(data[1] == Channel.TYPE_AUDIO_AC3)
									buff.append(" AC3");
								else if(data[1] == Channel.TYPE_AUDIO_AAC)
									buff.append(" AAC");
								else
									buff.append(" MPG");
								buff.append("<input type='hidden' name='audioid" + channelCount
										+ "' value='" + data[0] + ":" + data[1] + "'>\n");                  
							}
							else if(count > 1)
							{
								buff.append("<SELECT NAME='audioid" + channelCount + "'>\n");
								for(int st = 0; st < chData.getStreams().size(); st++)
								{
									streamData = (int[])chData.getStreams().get(st);
									if(streamData[1] == Channel.TYPE_AUDIO_MPG || streamData[1] == Channel.TYPE_AUDIO_AC3 || streamData[1] == Channel.TYPE_AUDIO_AAC)
									{
										buff.append("<OPTION value=\"" + streamData[0] + ":" + streamData[1] + "\">");
										buff.append(streamData[0]);
										if(streamData[1] == Channel.TYPE_AUDIO_AC3)
											buff.append(" AC3");
										else if(streamData[1] == Channel.TYPE_AUDIO_AAC)
											buff.append(" AAC");
										else
											buff.append(" MPG");                        
										buff.append("</OPTION>\n");
									}
								}                  
								buff.append("</SELECT>");
							}                
							buff.append("</td>\n");


							buff.append("<td align='center'><input type='checkbox' name='add"
									+ channelCount + "' value='add' onClick='unTickAddAll()'></td>\n");

							buff.append("</tr>\n\n");

							channelCount++;
							channelsFound++;
						}
						else
						{
							break;
						}
					}

					if(channelsFound == 0)
					{
						buff.append("<tr class='scanChannelResult'>");
						buff.append("<td>No Programs Found</td>\n");
						buff.append("<td align='center'>N/A</td>\n");
						buff.append("<td align='center'>N/A</td>\n");
						buff.append("<td align='center'>N/A</td>\n");
						buff.append("<td align='center'>N/A</td>\n");
						buff.append("</tr>\n\n");
					}

					buff.append("<tr><td colspan='6'>" + result.getLog2().replaceAll("\n", "<br>") + "</td></tr>\n");
					buff.append("</table><br>\n");

					m_outStream.write(buff.toString().getBytes("UTF-8"));
					m_outStream.flush();
					m_outStream.write("\n\n\n\n\n           \n\n\n\n\n\n".getBytes("UTF-8"));
					m_outStream.flush();
				}
			}

			if(channelCount > 0)
			{
				out = "<table class='channelScanResult'><tr class='scanChannelHeading'><td class='scanChannelHeadingDataName'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'>Select FTA</td><td class='scanChannelHeadingData'><input type='checkbox' id='addAllFree' name='addAllFree' value='addAllFree' onClick='tickAllFree()'></td></tr>";
				out += "<tr class='scanChannelHeading'><td class='scanChannelHeadingDataName'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'>Select All</td><td class='scanChannelHeadingData'><input type='checkbox' id='addAll' name='addAll' value='addAll' onClick='tickAll()'></td></tr></table><br>";
				out += "<input type=\"submit\" value=\"Add Selected\"></form></body></html><br><br>\n" 
						+ logText + "</td></tr></table>\n";
			}

			m_outStream.write(out.getBytes("UTF-8"));
			m_outStream.flush();

		}
		catch (Exception e)
		{
			throw e;
		}

			}

	private byte[] deleteChannel() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		String name = m_urlData.getParameter("ID");
		if(name != null)
		{
			m_store.removeChannel(name);
			m_store.saveChannels(null);

			guide.deleteChannelMapCustom(name);
		}

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return out.toString().getBytes("UTF-8");
	}

	private byte[] deleteAllChannels() throws Exception
	{
		m_store.removeAllChannels();
		GuideStore.getInstance().deleteAllChannels();

		return getServletRedir().getBytes("UTF-8");
	}

	protected class Channel2
	{
		Channel m_channel;
		String m_setting, m_sortData;
	}

	private byte[] showChannels(boolean bDynamic) throws Exception
	{
		StringBuffer out = new StringBuffer(4096);
		HashMap<String, Channel> channels = m_store.getChannels();
		List<Channel2> chList = new LinkedList<Channel2>();
		Iterator<Channel> iter = channels.values().iterator();
		while (iter.hasNext())
		{
			Channel2 channel2 = new Channel2();
			channel2.m_channel = iter.next();
			channel2.m_setting = channel2.m_channel.getSetting();
			channel2.m_sortData = (channel2.m_channel.isGroup() ? "2" : "1") + channel2.m_channel.getSetting() + 
					"\n" + channel2.m_channel.getName().toLowerCase();
			chList.add(channel2);
		}

		String sort = m_urlData.getParameter("sort");
		if ( (sort == null) || (sort.isEmpty()) )
		{
			sort = m_urlData.getCookie("ChannelListSort");
			if ( (sort == null) || (sort.isEmpty()) )
				sort = "default";
		}

		if (sort.equals("number"))
		{
			Comparator<Channel2> comparator = new Comparator<Channel2>() {
				public int compare(Channel2 g1, Channel2 g2) {
					return g1.m_channel.getNumber() - g2.m_channel.getNumber();
				}
			};
			Collections.sort(chList, comparator);
		}
		else if (sort.equals("name"))
		{
			Comparator<Channel2> comparator = new Comparator<Channel2>() {
				public int compare(Channel2 g1, Channel2 g2) {
					return g1.m_channel.getName().compareToIgnoreCase(g2.m_channel.getName());
				}
			};
			Collections.sort(chList, comparator);
		}
		else if (sort.equals("setting"))
		{
			Comparator<Channel2> comparator = new Comparator<Channel2>() {
				public int compare(Channel2 g1, Channel2 g2) {
					int result = g1.m_channel.getSetting().compareTo(g2.m_channel.getSetting());
					if (result == 0)
					{
						return g1.m_channel.getName().compareToIgnoreCase(g2.m_channel.getName());
					}
					else
						return result;
				}
			};

			Collections.sort(chList, comparator);
		}
		else if (sort.equals("frequency"))
		{
			Comparator<Channel2> comparator = new Comparator<Channel2>() {
				public int compare(Channel2 g1, Channel2 g2) {
					int result = g1.m_channel.getFrequency() - g2.m_channel.getFrequency();
					if (result == 0)
					{
						return g1.m_channel.getName().compareToIgnoreCase(g2.m_channel.getName());
					}
					else
						return result;
				}
			};
			Collections.sort(chList, comparator);
		}
		else if (sort.equals("diseqc"))
		{
			Comparator<Channel2> comparator = new Comparator<Channel2>() {
				public int compare(Channel2 g1, Channel2 g2) {
					int result = g1.m_channel.getDiseqc().compareTo(g2.m_channel.getDiseqc());
					if (result == 0)
					{
						result = g1.m_channel.getFrequency() - g2.m_channel.getFrequency();
						if (result == 0)
						{
							return g1.m_channel.getName().compareToIgnoreCase(g2.m_channel.getName());
						}
						else
							return result;
					}
					else
						return result;
				}
			};
			Collections.sort(chList, comparator);
		}
		else if (sort.equals("typeDiseqcFrequency"))
		{
			Comparator<Channel2> comparator = new Comparator<Channel2>() {
				public int compare(Channel2 g1, Channel2 g2) {
					int result = g1.m_channel.getNetworkType().compareTo(g2.m_channel.getNetworkType());
					if (result == 0)
					{
						result = g1.m_channel.getDiseqc().compareTo(g2.m_channel.getDiseqc());
						if (result == 0)
						{
							result = g1.m_channel.getFrequency() - g2.m_channel.getFrequency();
							if (result == 0)
							{
								return g1.m_channel.getName().compareToIgnoreCase(g2.m_channel.getName());
							}
							else
								return result;
						}
						else
							return result;
					}
					else
						return result;
				}
			};
			Collections.sort(chList, comparator);
		}
		else
		{
			Comparator<Channel2> comparator = new Comparator<Channel2>() {
				public int compare(Channel2 g1, Channel2 g2) {
					return g1.m_sortData.compareTo(g2.m_sortData);
				}
			};
			Collections.sort(chList, comparator);
		}

		Channel ch = null;
		Iterator<Channel2> iter2 = chList.iterator();
		while (iter2.hasNext())
		{
			ch = iter2.next().m_channel;
			String channelNameUrlEncoded = Formatter.encodeURL_UTF8(ch.getName());

			out.append("<tr><td><span class='channelInfo'>" + ch.getNumber() + "</span></td>");
			out.append("<td><span class='channelName'>" + HTMLEncoder.encode(ch.getName()) + "</span></td>");
			out.append("<td class='channelInfo'>" + (ch.isGroup() ? "Group" : HTMLEncoder.encode(ch.getNetworkType())) + "</td>\n");
			out.append("<td class='channelInfo'>" + (ch.isGroup() ? HTMLEncoder.encode(ch.getChannelsString()) : (ch.getDemodulatorName().isEmpty() ? HTMLEncoder.encode(ch.getSetting()) : HTMLEncoder.encode(ch.getDemodulatorName()))) + "</td>\n");

			/*
         out.append("<td class='channelInfo'>" + ch.getProgramID() + "</td>\n");
         out.append("<td class='channelInfo'>" + ch.getVideoPid() + "</td>\n");
         out.append("<td class='channelInfo'>" + ch.getAudioPid());
         if(ch.getAudioType() == Channel.TYPE_AUDIO_AC3)
            out.append("-AC3");
         else if(ch.getAudioType() == Channel.TYPE_AUDIO_AAC)
            out.append("-AAC");
         else
            out.append("-MPG");
         out.append("</td>\n");
			 */

			out.append("<td>" + (ch.isGroup() ? "" : ch.getSelectionDescription()) + "</td>\n");
			out.append("<td>" + (ch.isGroup() ? "" : ch.getAccessStr()) + "</td>\n");
			out.append("<td>" + (ch.isGroup() ? "" : ch.getCapType()) + "</td>\n");

			// The rest...
			if (bDynamic)
			{
				out.append("<td>" + (ch.isGroup() ? "" : ch.getDemodulatorName()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getFrequency()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getModulation()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getSymbolRate()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getPhysicalChannel()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getBandWidth()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getPilot()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getRollOff()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getPolarization()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getLowOsc()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getHighOsc()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getLNBSwitch()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getLongitude()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getDirLongitude()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getAzimuth()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getElevation()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getInputRange()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getDiseqc()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getDemodulatorData()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getOriginalNetworkId()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getTransportStreamId()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getSourceId()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getCharacterSet()) + "</td>\n");

				out.append("<td>" + (ch.isGroup() ? "" : ch.getPreviewType()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getPreviewRemoteType()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getSurfType()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getSurfRemoteType()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getCapturePath()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getUserData()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getSelectionStr()) + "</td>\n");

				out.append("<td>" + (ch.isGroup() ? "" : ch.getProgramID()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.isBuildPat()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.isIncludeVideoAudioSubtitleOnly()) + "</td>\n");

				out.append("<td>" + (ch.isGroup() ? "" : ch.getPmtPid()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getPcrPid()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getVideoPid()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getVideoTypeStr()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getVideoPmtTypeStr()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.isPatchPmtVideo()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getAudioPid()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getAudioTypeStr()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getAudioPmtTypeStr()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.isPatchPmtAudio()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getTeletextPid()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getTeletextSubtitlePage()) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getOtherPidString(false)) + "</td>\n");
				out.append("<td>" + (ch.isGroup() ? "" : ch.getDisabledPidString(false)) + "</td>\n");
			}

			out.append("<td nowrap='1' class='channelInfo'>");
			out.append(" <a onClick='return confirmAction(\"Delete\");' href='/servlet/"
					+ m_urlData.getServletClass()
					+ "?action=06&ID="
					+ channelNameUrlEncoded
					+ "'>");
			out.append("<img border=0 src='/images/delete.png' alt='Delete Channel' title='Delete Channel' align='absmiddle' width='24' height='24'>");
			out.append("</a>");

			if (ch.isGroup())
			{
				out.append(" <a href='/servlet/" + m_urlData.getServletClass()
						+ "?action=19&channelName=" + channelNameUrlEncoded + "'>");
				out.append("<img border=0 src='/images/edit.png' alt='Edit Channel' title='Edit Channel' align='absmiddle' width='24' height='24'>");
				out.append("</a> ");
			}
			else
			{
				out.append(" <a href='/servlet/" + m_urlData.getServletClass()
						+ "?action=09&ID=" + channelNameUrlEncoded + "'>");
				out.append("<img border=0 src='/images/edit.png' alt='Edit Channel' title='Edit Channel' align='absmiddle' width='24' height='24'>");
				out.append("</a> ");
			}
			out.append("</td>\n");

			out.append("</tr>\n");
		}

		PageTemplate template = new PageTemplate(m_headers, m_urlData, bDynamic ? "channelsDynamic.html" : "channels.html", true);
		template.replace("$channels", out.toString());

		return template.getPageBytes();
	}

	private byte[] showChannels() throws Exception
	{
		String dynamic = m_urlData.getCookie("ChannelListDynamic");
		if ( (dynamic == null) || (dynamic.isEmpty()) || !dynamic.equals("1") )
			return showChannels(false);
		else
			return showChannels(true);
	}

	private String checkName(String name)
	{
		return name.trim();
		//return Formatter.format(name.trim());
		/*
		StringBuffer finalName = null;

		try
		{
			finalName = new StringBuffer(256);
			for (int x = 0; x < name.length(); x++)
			{
				char charAt = name.charAt(x);

				if((charAt >= 'a' && charAt <= 'z')
						|| (charAt >= 'A' && charAt <= 'Z')
						|| (charAt >= '0' && charAt <= '9') || charAt == ' ')
					finalName.append(charAt);
				else
					finalName.append('-');
			}

		}
		catch (Exception e)
		{
			name = "error";
		}

		return finalName.toString();
		 */
	}

	private String checkAlias(String alias)
	{
		return alias.trim();
	}

	private byte[] addSelectedPrograms() throws Exception
	{
		int numAdded = 0;

		for (int x = 0; x < 1000; x++)
		{
			String add = m_urlData.getParameter("add" + x);

			if(add != null)
			{
				String name = checkName(m_urlData.getParameter("name" + x, ""));

				int access = m_urlData.getParameterInt("access" + x, Channel.ACCESS_FTA);
				int progID = m_urlData.getParameterInt("programid" + x, -1);
				int videoid = -1;
				int videoType = -1;
				int audioid = -1;
				int audioType = -1;
				try
				{
					String videoData = m_urlData.getParameter("videoid" + x, "");
					String[] vdSplit = videoData.split(":");
					videoid = Integer.parseInt(vdSplit[0]);
					videoType = Integer.parseInt(vdSplit[1]);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}
				try
				{
					String audioData = m_urlData.getParameter("audioid" + x, "");
					String[] adSplit = audioData.split(":");
					audioid = Integer.parseInt(adSplit[0]);
					audioType = Integer.parseInt(adSplit[1]);
				}
				catch (Exception e)
				{
					e.printStackTrace();
				}

				int teletextPID = m_urlData.getParameterInt("teletextid" + x, -1);
				int teletextSubtitlePage = m_urlData.getParameterInt("teletextpage" + x, -1);
				int pmtPID = m_urlData.getParameterInt("pmtid" + x, -1);
				int pcrPID = m_urlData.getParameterInt("pcrid" + x, -1);

				Vector<Integer> pids2 = new Vector<Integer>();
				pids2.add(0);	// PAT
				//pids2.add(1);	// CAT
				//pids2.add(2);	// TSDT

				/*
				if (networkType.equalsIgnoreCase("ATSC"))
				{
					pids2.add(0x1ffb);	// STT, xVCT, MGT, RRT, DCCT, DCCSCT
				}
				else if (networkType.startsWith("DVB"))
				{
					pids2.add(16);	// NIT
					pids2.add(17);	// SDT
					pids2.add(18);	// EIT
					//pids2.add(19);	// RST, optional
					pids2.add(20);	// TDT
				}
				 */

				String pids = "";
				try
				{
					pids = m_urlData.getParameter("pids" + x, "");
					if(pids != null)
					{
						String[] pids3 = pids.split(",");
						for (int i=0; i<pids3.length; i++)
						{
							int pid2 = Integer.parseInt(pids3[i]);
							if ( (pid2 != videoid) && (pid2 != audioid) && (pid2 != teletextPID) && (pid2 != pmtPID) && (pid2 != pcrPID) )
								pids2.add(pid2);
						}
					}
				}
				catch (Exception e) {}

				if(progID > -1)
				{
					Channel chan = getDemodulatorSetting(x);
					chan.setChannel(name, access, progID, videoid, videoType, 
							audioid, audioType, teletextPID, teletextSubtitlePage, pmtPID, pcrPID, pids2);

					//chan.setAudioType(audioType);
					m_store.addChannel(chan);
					numAdded++;
				}
			}
		}

		if(numAdded > 0)
			m_store.saveChannels(null);

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return out.toString().getBytes("UTF-8");
	}

	protected void updateEpgChannelMapping(String oldChannelName, String newChannelName)
	{
		GuideStore guide = GuideStore.getInstance();
		guide.updateChannelMapCustom(oldChannelName, newChannelName);
	}

	private void updateScheduleItems(String oldChannelName, String newChannelName) throws Exception
	{
		HashMap<String, ScheduleItem> times = m_store.getScheduleList();
		String[] keys = (String[]) times.keySet().toArray(new String[0]);
		Arrays.sort(keys);
		boolean bSave = false;

		for (int x = 0; x < keys.length; x++)
		{
			ScheduleItem item = (ScheduleItem) times.get(keys[x]);

			if(item == null)
			{
				DVScheduler.log("ERROR: for some reason one of your schedule items in the MAP is null : " + keys[x]);
				break;
			}

			if ( (item.getState() != ScheduleItem.RUNNING) && 
					//(item.getState() != ScheduleItem.SKIPPED) && 
					//(item.getState() != ScheduleItem.FINISHED) && 
					(item.getChannel().equals(oldChannelName)) )
			{
				item.setChannel(newChannelName);
				bSave = true;
			}
		}

		if (bSave)
			m_store.saveSchedules(null);
	}

	protected void updateChannels(String oldChannelName, String newChannelName)
	{
		Iterator<Channel> iter = m_store.getChannels().values().iterator();
		while (iter.hasNext())
		{
			iter.next().updateChannelGroup(oldChannelName, newChannelName);
		}
	}

	private byte[] updateChannel() throws Exception
	{
		String update = m_urlData.getParameter("update");
		boolean bUpdate = false;
		int updateDemodulator = 0;
		if (update != null)
		{
			if (update.equals("Update"))
				bUpdate = true;
			else if (update.equals("Update"))
				updateDemodulator = 1;
			else if (update.equals("Update All"))
				updateDemodulator = 2;
		}

		int access = m_urlData.getParameterInt("access", Channel.ACCESS_FTA);

		String oldName = m_urlData.getParameter("oldName", "");

		String name = checkName(m_urlData.getParameter("name", ""));

		int number = m_urlData.getParameterInt("number", -1);
		String capType = m_urlData.getParameter("capturetype", "");
		String previewType = m_urlData.getParameter("previewtype", "");
		String previewRemoteType = m_urlData.getParameter("previewremotetype", "");
		String surfType = m_urlData.getParameter("surftype", "");
		String surfRemoteType = m_urlData.getParameter("surfremotetype", "");
		String capturePath = m_urlData.getParameter("capturePath", "");
		String userData = m_urlData.getParameter("userdata", "");
		int selection = m_urlData.getParameterInt("selection", 1);
		int videoid = m_urlData.getParameterInt("videoid", -1);
		int videoType = m_urlData.getParameterInt("videoType", -1);
		int audioid = m_urlData.getParameterInt("audioid", -1);
		int audioType = m_urlData.getParameterInt("audioType", -1);
		int teletextPID = m_urlData.getParameterInt("teletextid", -1);
		int teletextSubtitlePage = m_urlData.getParameterInt("teletextpage", -1);

		int progID = m_urlData.getParameterInt(selection == 0 ? "programid" : "customProgramid", -1);

		boolean bTeletextRealTime = false;
		try
		{
			String param = null;
			switch(selection)
			{
			case 0:
				param = m_urlData.getParameter("programIdRealtimeTeletext").trim();
				bTeletextRealTime = param.equals("1") || param.equals("true");
				break;
			case 2: case 1:
				param = m_urlData.getParameter("customRealtimeTeletext").trim();
				bTeletextRealTime = param.equals("1") || param.equals("true");
				break;
			default:
				bTeletextRealTime = true;
				break;
			}
		}
		catch (Exception e)	{}

		int pmtPID = m_urlData.getParameterInt("pmtid", -1);
		int pcrPID = m_urlData.getParameterInt("pcrid", -1);

		boolean bBuildPat = false;
		try
		{
			switch(selection)
			{
			case 0:
				bBuildPat = m_urlData.getParameter("programIdRebuildPat").trim().equals("1");
				break;
			case 2: case 1:
				bBuildPat = m_urlData.getParameter("customRebuildPat").trim().equals("1");
				break;
			default:
				bBuildPat = true;
				break;
			}
		}
		catch (Exception e)	{}

		boolean bPatchPmtVideo = false;
		try
		{
			bPatchPmtVideo = m_urlData.getParameter("customPatchPmtVideo").trim().equals("1");
		}
		catch (Exception e)	{}

		boolean bPatchPmtAudio = false;
		try
		{
			bPatchPmtAudio = m_urlData.getParameter("customPatchPmtAudio").trim().equals("1");
		}
		catch (Exception e)	{}

		boolean bIncludeVideoAudioSubtitleOnly = false;
		try
		{
			bIncludeVideoAudioSubtitleOnly = m_urlData.getParameter("programIdVasOnly").trim().equals("1");
		}
		catch (Exception e)	{}

		Vector<Integer> disabledpids = new Vector<Integer>();
		try
		{
			String pids = (selection == 0) ? m_urlData.getParameter("disabledpidProgId").trim() : m_urlData.getParameter("disabledpid").trim();
			int pid = 0;
			if (!pids.isEmpty())
			{
				String[] pids2 = pids.split(",");
				for(int x = 0; x < pids2.length; x++)
				{
					pid = Integer.parseInt(pids2[x].trim());
					if (pid > -1)
						disabledpids.add(pid);
				}
			}
		}
		catch (Exception e)	{}

		Vector<Integer> pids3 = new Vector<Integer>();
		try
		{
			String pids = (selection == 0) ? m_urlData.getParameter("includePidsProgId").trim() : m_urlData.getParameter("pid").trim();
			int pid = 0;
			if (!pids.isEmpty())
			{
				String[] pids2 = pids.split(",");
				for(int x = 0; x < pids2.length; x++)
				{
					pid = Integer.parseInt(pids2[x].trim());
					if (pid > -1 && !disabledpids.contains(pid))
						pids3.add(pid);
				}
			}
		}
		catch (Exception e)	{}

		GuideStore guide = GuideStore.getInstance();

		if ( ((selection == 0) && (progID > -1)) || 
				(selection == 1) || 
				((selection == 2) && ((videoid > -1) || (audioid > -1) || (pmtPID > -1) || (pcrPID > -1) || (teletextPID > -1) || !pids3.isEmpty())) || 
				(updateDemodulator > 0) )
		{
			Channel chan = getDemodulatorSetting(-1);
			chan.setChannel(name, access, progID, videoid, videoType, 
					audioid, audioType, teletextPID, teletextSubtitlePage, pmtPID, pcrPID, pids3);
			chan.setNumber(number);
			chan.setPreviewType(previewType);
			chan.setPreviewRemoteType(previewRemoteType);
			chan.setSurfType(surfType);
			chan.setSurfRemoteType(surfRemoteType);
			chan.setCapturePath(capturePath);
			chan.setSelection(selection);
			chan.setCaptureType(capType);
			chan.setBuildPat(bBuildPat);
			chan.setPatchPmtVideo(bPatchPmtVideo);
			chan.setPatchPmtAudio(bPatchPmtAudio);
			chan.setIncludeVideoAudioSubtitleOnly(bIncludeVideoAudioSubtitleOnly);
			chan.setTeletextRealTime(bTeletextRealTime);
			chan.setDisabledPid(disabledpids);
			chan.setUserData(userData);

			if (updateDemodulator == 1)
			{
				m_store.updateDemodulator(oldName, chan);
				m_store.setCurrentChannel(chan);
			}
			else if (updateDemodulator == 2)
			{
				m_store.updateDemodulatorAll(oldName, chan);
				m_store.setCurrentChannel(chan);
			}
			else
			{
				if (bUpdate)
					m_store.updateChannel(oldName, chan);
				else
					m_store.addChannel(chan);

				m_store.setCurrentChannel(chan);

				if (bUpdate)
				{
					name = chan.getName();
					updateScheduleItems(oldName, name);
					guide.renameEpgExclusionChannel(oldName, name);
					updateEpgChannelMapping(oldName, name);
					updateChannels(oldName, name);
				}
			}
		}

		m_store.saveChannels(null);

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return out.toString().getBytes("UTF-8");
	}   

	private byte[] showScanResult() throws Exception
	{
		Channel ch = getDemodulatorSetting(-1);

		m_store.setCurrentChannel(ch);

		PageTemplate template = new PageTemplate(m_headers, m_urlData, "channel-scanresult.html");

		CaptureDeviceList devList = CaptureDeviceList.getInstance();

		if(devList.getRunningDeviceCount() > 0)
		{
			template.replace("$scanresult", "Can not scan channels while captures are active!");
			template.replace("$scanlog", "");
			return template.getPageBytes();
		}

		String networkType = ch.getNetworkType();
		if (devList.getDeviceCount(networkType) == 0)
		{
			template.replace("$scanresult", "No Suitable Devices Available!");
			template.replace("$scanlog", "");
			return template.getPageBytes();
		}

		//String name = m_urlData.getParameter("name");
		String name = "Name";

		StringBuffer buff = new StringBuffer(1024);

		buff.append("<form action='/servlet/" + m_urlData.getServletClass()
				+ "' method='POST' accept-charset=\"UTF-8\">\n");
		buff.append("<input type='hidden' name='action' value='05'>\n");

		buff.append("<table class='channelScanResult'>\n");

		buff.append("<tr class='scanChannelHeading'>");
		buff.append("<td class='scanChannelHeadingDataName'>" + name + "</td>");
		buff.append("<td class='scanChannelHeadingData'>Access</td>");
		buff.append("<td class='scanChannelHeadingData'>Program</td>");
		buff.append("<td class=\"scanChannelHeadingData\">Video</td>\n");
		buff.append("<td class=\"scanChannelHeadingData\">Audio</td>\n");
		buff.append("<td class='scanChannelHeadingData'>Add</td>");
		buff.append("</tr>\n");

		CaptureDevice captureDevice = (CaptureDevice)devList.GetFirstDevice(networkType);
		if (captureDevice == null)
			throw new Exception("No suitable devices found");
		CaptureEngine captureEngine = new CaptureEngine();
		String scanCommand = GetScanCommand(ch, captureDevice, captureEngine);
		if (scanCommand == null)
			throw new Exception("Scan command not found in capture engine path");		
		DVScheduler.log("Running channel scan command: " + scanCommand);

		Runtime runner = Runtime.getRuntime();

		Process scan = runner.exec(Formatter.parseCommand(scanCommand));

		ScanResult result = new ScanResult(ch, captureEngine);
		result.readInput(scan.getInputStream());
		result.parseXML();
		scan.waitFor();
		String logText = "<b>Scan Log:</b><br><br>" + result.getLog().replaceAll("\n", "<br>");

		int channelCount = 0;

		if(result.getResult().size() > 0)
		{
			Vector<Channel> scanResult = result.getResult();
			Channel chData = null;

			for (int x = 0; x < scanResult.size(); x++)
			{
				chData = (Channel) scanResult.get(x);

				if(chData != null)
				{
					buff.append("<tr class='scanChannelResult'>");

					buff.append("<td>\n<input type='text' size='20' name='name" + channelCount
							+ "' value='" + HTMLEncoder.encode(checkName(chData.getName())) + "'>\n");

					buff.append("<input type='hidden' name='networkType" + channelCount
							+ "' value='" + ch.getNetworkType() + "'>\n");

					buff.append("<input type='hidden' name='freq" + channelCount
							+ "' value='" + ch.getFrequency() + "'>\n");

					buff.append("<input type='hidden' name='modulation" + channelCount
							+ "' value='" + ch.getModulation() + "'>\n");

					buff.append("<input type='hidden' name='symbolRate" + channelCount
							+ "' value='" + ch.getSymbolRate() + "'>\n");

					buff.append("<input type='hidden' name='physicalChannel" + channelCount
							+ "' value='" + ch.getPhysicalChannel() + "'>\n");

					buff.append("<input type='hidden' name='band" + channelCount
							+ "' value='" + ch.getBandWidth() + "'>\n");

					buff.append("<input type='hidden' name='polarization" + channelCount
							+ "' value='" + ch.getPolarization() + "'>\n");

					buff.append("<input type='hidden' name='highOsc" + channelCount
							+ "' value='" + ch.getHighOsc() + "'>\n");

					buff.append("<input type='hidden' name='lowOsc" + channelCount
							+ "' value='" + ch.getLowOsc() + "'>\n");

					buff.append("<input type='hidden' name='lnbSwitch" + channelCount
							+ "' value='" + ch.getLNBSwitch() + "'>\n");

					buff.append("<input type='hidden' name='longitude" + channelCount
							+ "' value='" + ch.getLongitude() + "'>\n");

					buff.append("<input type='hidden' name='westLongitude" + channelCount
							+ "' value='" + ch.getWestLongitude() + "'>\n");

					buff.append("<input type='hidden' name='azimuth" + channelCount
							+ "' value='" + ch.getAzimuth() + "'>\n");

					buff.append("<input type='hidden' name='elevation" + channelCount
							+ "' value='" + ch.getElevation() + "'>\n");

					buff.append("<input type='hidden' name='inputRange" + channelCount
							+ "' value='" + ch.getInputRange() + "'>\n");

					buff.append("<input type='hidden' name='pilot" + channelCount
							+ "' value='" + ch.getPilot() + "'>\n");

					buff.append("<input type='hidden' name='rollOff" + channelCount
							+ "' value='" + ch.getRollOff() + "'>\n");

					buff.append("<input type='hidden' name='diseqc" + channelCount
							+ "' value='" + ch.getDiseqc() + "'>\n");

					buff.append("<input type='hidden' name='demodulatorData" + channelCount
							+ "' value='" + ch.getDemodulatorData() + "'>\n");

					buff.append("<input type='hidden' name='demodulatorName" + channelCount
							+ "' value='" + (ch.getDemodulatorName().isEmpty() ? HTMLEncoder.encode(checkAlias(chData.getDemodulatorName())) : HTMLEncoder.encode(checkAlias(ch.getDemodulatorName()))) + "'>\n");

					buff.append("<input type='hidden' name='originalNetworkId" + channelCount
							+ "' value='" + chData.getOriginalNetworkId() + "'>\n");

					buff.append("<input type='hidden' name='transportStreamId" + channelCount
							+ "' value='" + chData.getTransportStreamId() + "'>\n");

					buff.append("<input type='hidden' name='sourceId" + channelCount
							+ "' value='" + chData.getSourceId() + "'>\n");

					buff.append("<input type='hidden' name='characterSet" + channelCount
							+ "' value='" + ch.getCharacterSet() + "'>\n");

					buff.append("<input type='hidden' name='pids" + channelCount 
							+ "' value='" + getPids(chData) + "'>\n"); 

					buff.append("<input type='hidden' name='teletextid" + channelCount
							+ "' value='" + chData.getTeletextPid() + "'>\n");

					buff.append("<input type='hidden' name='teletextpage" + channelCount
							+ "' value='" + chData.getTeletextSubtitlePage() + "'>\n");

					int[] streamData = getFirstOfType(chData, Channel.TYPE_PMT);
					if (streamData == null)
					{
						buff.append("<input type='hidden' name='pmtid" + channelCount
								+ "' value='-1'>\n");
					}
					else
					{
						buff.append("<input type='hidden' name='pmtid" + channelCount
								+ "' value='" + streamData[0] + "'>\n");
					}

					streamData = getFirstOfType(chData, Channel.TYPE_PCR);
					if (streamData == null)
					{
						buff.append("<input type='hidden' name='pcrid" + channelCount
								+ "' value='-1'>\n");
					}
					else
					{
						buff.append("<input type='hidden' name='pcrid" + channelCount
								+ "' value='" + streamData[0] + "'>\n");
					}

					buff.append("</td>\n\n");

					buff.append("<td align='center'>" + chData.getAccessStr());
					buff.append("<input type='hidden' name='access"	+ channelCount
							+ "' value='" + chData.getAccess() + "'></td>\n");

					buff.append("<td align='center'>" + chData.getProgramID());
					buff.append("<input type='hidden' name='programid"
							+ channelCount + "' value='" + chData.getProgramID()
							+ "'>\n");
					buff.append("</td>\n");

					//
					// do the video pid
					//
					buff.append("<td align='center'>");
					int count = getStreamTypeCount(chData, Channel.TYPE_VIDEO_MPG) + getStreamTypeCount(chData, Channel.TYPE_VIDEO_H264);
					if(count == 0)
					{
						buff.append("-");
						buff.append("<input type='hidden' name='videoid" + channelCount
								+ "' value='-1'>\n");                  
					}
					if(count == 1)
					{
						int[] data = getFirstOfType(chData, Channel.TYPE_VIDEO_MPG);
						if(data == null)
							data = getFirstOfType(chData, Channel.TYPE_VIDEO_H264);
						buff.append(data[0]);
						if(data[1] == Channel.TYPE_VIDEO_H264)
							buff.append(" H264");
						else
							buff.append(" MPG");
						buff.append("<input type='hidden' name='videoid" + channelCount
								+ "' value='" + data[0] + ":" + data[1] + "'>\n");
					}
					else if(count > 1)
					{
						buff.append("<SELECT NAME='videoid" + channelCount + "'>\n");
						for(int st = 0; st < chData.getStreams().size(); st++)
						{
							streamData = (int[])chData.getStreams().get(st);
							if(streamData[1] == Channel.TYPE_VIDEO_MPG || streamData[1] == Channel.TYPE_VIDEO_H264)
							{
								buff.append("<OPTION value=\"" + streamData[0] + ":" + streamData[1] + "\">");
								buff.append(streamData[0]);
								if(streamData[1] == Channel.TYPE_VIDEO_H264)
									buff.append(" H264");
								else
									buff.append(" MPG");                        
								buff.append("</OPTION>\n");
							}
						}                  
						buff.append("</SELECT>");
					}
					buff.append("</td>\n");

					//
					// do the audio pid
					//
					buff.append("<td align='center'>");
					count = getStreamTypeCount(chData, Channel.TYPE_AUDIO_AC3);
					count += getStreamTypeCount(chData, Channel.TYPE_AUDIO_AAC);
					count += getStreamTypeCount(chData, Channel.TYPE_AUDIO_MPG);
					if(count == 0)
					{
						buff.append("-");
						buff.append("<input type='hidden' name='audioid" + channelCount
								+ "' value='-1'>\n");                  
					}                
					if(count == 1)
					{
						int[] data = getFirstOfType(chData, Channel.TYPE_AUDIO_AC3);
						if(data == null)
							data = getFirstOfType(chData, Channel.TYPE_AUDIO_AAC);
						if(data == null)
							data = getFirstOfType(chData, Channel.TYPE_AUDIO_MPG);
						buff.append(data[0]);
						if(data[1] == Channel.TYPE_AUDIO_AC3)
							buff.append(" AC3");
						else if(data[1] == Channel.TYPE_AUDIO_AAC)
							buff.append(" AAC");
						else
							buff.append(" MPG");
						buff.append("<input type='hidden' name='audioid" + channelCount
								+ "' value='" + data[0] + ":" + data[1] + "'>\n");
					}
					else if(count > 1)
					{
						buff.append("<SELECT NAME='audioid" + channelCount + "'>\n");
						for(int st = 0; st < chData.getStreams().size(); st++)
						{
							streamData = (int[])chData.getStreams().get(st);
							if(streamData[1] == Channel.TYPE_AUDIO_MPG || streamData[1] == Channel.TYPE_AUDIO_AC3 || streamData[1] == Channel.TYPE_AUDIO_AAC)
							{
								buff.append("<OPTION value=\"" + streamData[0] + ":" + streamData[1] + "\">");
								buff.append(streamData[0]);
								if(streamData[1] == Channel.TYPE_AUDIO_AC3)
									buff.append(" AC3");
								else if(streamData[1] == Channel.TYPE_AUDIO_AAC)
									buff.append(" AAC");
								else
									buff.append(" MPG");                        
								buff.append("</OPTION>\n");
							}
						}                  
						buff.append("</SELECT>");
					}                
					buff.append("</td>\n");


					buff.append("<td align='center'><input type='checkbox' name='add"
							+ channelCount + "' value='add' onClick='unTickAddAll()'></td>\n");

					buff.append("</tr>\n\n");
					channelCount++;
				}
				else
				{
					break;
				}
			}
		}
		else
		{
			buff.append("<tr class='scanChannelResult'>");
			buff.append("<td>No Programs Found</td>\n");
			buff.append("<td align='center'>N/A</td>\n");
			buff.append("<td align='center'>N/A</td>\n");
			buff.append("<td align='center'>N/A</td>\n");
			buff.append("<td align='center'>N/A</td>\n");
			buff.append("</tr>\n\n");
		}

		buff.append("<tr><td colspan='6'>" + result.getLog2().replaceAll("\n", "<br>") + "</td></tr>\n");
		buff.append("</table>\n");

		if(channelCount > 0)
		{
			buff.append("<table class='channelScanResult'><tr class='scanChannelHeading'><td class='scanChannelHeadingDataName'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'>Select FTA</td><td class='scanChannelHeadingData'><input type='checkbox' id='addAllFree' name='addAllFree' value='addAllFree' onClick='tickAllFree()'></td></tr>");
			buff.append("<tr class='scanChannelHeading'><td class='scanChannelHeadingDataName'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'></td><td class='scanChannelHeadingData'>Select All</td><td class='scanChannelHeadingData'><input type='checkbox' id='addAll' name='addAll' value='addAll' onClick='tickAll()'></td></tr></table><br>");

			buff.append("<input type='submit' value='Add Selected'>\n");
		}

		buff.append("</form>\n");

		template.replace("$scanresult", buff.toString());
		template.replace("$scanlog", logText);
		return template.getPageBytes();
	}

	private int getStreamTypeCount(Channel chan, int type)
	{
		Vector<?> streams = chan.getStreams();
		int count = 0;
		for(int x = 0; x < streams.size(); x++)
		{
			int[] streamData = (int[])streams.get(x);
			if(streamData[1] == type)
				count++;
		}

		return count;
	}

	private int[] getFirstOfType(Channel chan, int type)
	{
		Vector<?> streams = chan.getStreams();
		for(int x = 0; x < streams.size(); x++)
		{
			int[] streamData = (int[])streams.get(x);
			if(streamData[1] == type)
				return streamData;
		}
		return null;
	}

	protected String getPids(Channel chan)
	{
		String pids = "";
		Vector<int[]> streams = chan.getStreams();
		for(int x = 0; x < streams.size(); x++)
		{
			int[] streamData = (int[])streams.get(x);
			if (x > 0)
				pids += ",";
			pids += streamData[0];
		}
		return pids;
	}

	private byte[] showStationList() throws Exception
	{
		int country = m_urlData.getParameterInt("country", 0);
		int region = m_urlData.getParameterInt("region", 0);

		String networkType = m_urlData.getParameter("networkType");
		if ( (networkType == null) || (networkType.isEmpty()) )
			networkType = "DVB-T";

		ChannelList list = new ChannelList(null, networkType);
		StringBuffer buff = new StringBuffer(1024);
		Channel[] channelList = list.getStations(country, region);
		list.close();

		buff.append("<input type='hidden' name='country' value='" + country + "'>");
		buff.append("<input type='hidden' name='region' value='" + region + "'>");
		buff.append("<input type='hidden' name='networkType' value='" + networkType + "'>");
		buff.append("<table>\n");

		for (int x = 0; x < channelList.length; x++)
		{
			if(channelList[x] != null)
			{
				buff.append("<tr><td><input type='checkbox' name='add" + x + "' value='add' onClick='unTickAddAll()'></td><td>\n");
				buff.append("<a href='/servlet/" + m_urlData.getServletClass()
						+ "?action=04&country=" + country + "&region=" + region + "&freq=" + channelList[x].getFrequency());
				buff.append("&band=" + channelList[x].getBandWidth() + "&name="
						+ HTMLEncoder.encode(channelList[x].getName())
						+ "&networkType=" + Formatter.encodeURL_UTF8E(networkType)
						+ "' class='noUnder'>");
				buff.append(HTMLEncoder.encode(channelList[x].getName()) + "</a>\n");
				buff.append("</td></tr>\n");
			}
		}

		buff.append("<tr><td><input type='checkbox' id='addAll' name='addAll' value='addAll' onClick='tickAll()'></td><td>\n");
		buff.append("<a href='/servlet/" + m_urlData.getServletClass()
				+ "?action=07&country=" + country);
		buff.append("&region=" + region + "&networkType=" + Formatter.encodeURL_UTF8E(networkType) + "' class='noUnder'>SCAN ALL</a>\n");
		buff.append("</td></tr>\n");

		buff.append("</table>\n");

		PageTemplate template = new PageTemplate(m_headers, m_urlData, "channel-stationlist.html", true);
		template.replace("$stations", buff.toString());
		return template.getPageBytes();
	}

	private byte[] showRegionList() throws Exception
	{
		int country = m_urlData.getParameterInt("country", 0);

		String networkType = m_urlData.getParameter("networkType");
		if ( (networkType == null) || (networkType.isEmpty()) )
			networkType = "DVB-T";

		ChannelList list = new ChannelList(null, networkType);
		StringBuffer buff = new StringBuffer(1024);
		String[] regionList = list.getRegions(country);
		list.close();

		buff.append("<table><tr><td><ul>\n");

		for (int x = 0; x < regionList.length; x++)
		{
			if(regionList[x] != null)
			{
				String[] data = regionList[x].split("\n");
				buff.append("<li>\n");
				buff.append("<a href='/servlet/" + m_urlData.getServletClass()
						+ "?action=03&country=" + country + "&region=" + data[1]
								+ "&networkType=" + Formatter.encodeURL_UTF8E(networkType)
								+ "' class='noUnder'>" + data[0] + "</a>\n");
				buff.append("</li>\n");
			}
		}

		buff.append("</ul></td></tr></table>\n");

		PageTemplate template = new PageTemplate(m_headers, m_urlData, "channel-regionlist.html", true);
		template.replace("$regions", buff.toString());
		return template.getPageBytes();
	}

	private byte[] showCountryList() throws Exception
	{
		String networkType = m_urlData.getParameter("networkType");
		if ( (networkType == null) || (networkType.isEmpty()) )
			networkType = "DVB-T";

		ChannelList list = new ChannelList(null, networkType);
		StringBuffer buff = new StringBuffer(1024);
		String[] countList = list.getCountries();
		list.close();

		buff.append("<table><tr><td><ul>\n");

		for (int x = 0; x < countList.length; x++)
		{
			if(countList[x] != null)
			{
				String[] data = countList[x].split("\n");
				buff.append("<li>\n");
				buff.append("<a href='/servlet/" + m_urlData.getServletClass()
						+ "?action=02&country=" + data[1] + "&networkType=" + Formatter.encodeURL_UTF8E(networkType) + "' class='noUnder'>"
						+ data[0] + "</a>\n");
				buff.append("</li>\n");
			}
		}

		buff.append("</ul></td></tr></table>\n");

		PageTemplate template = new PageTemplate(m_headers, m_urlData, "channel-countrylist.html", true);
		template.replace("$countries", buff.toString());
		return template.getPageBytes();
	}

	private byte[] showChannelOrder(ChannelOrder channelOrder) throws Exception
	{
		if (channelOrder.getCustomOrderingSize() <= 0)
			channelOrder.checkCustomOrderingMissing(m_store.getChannels());

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("tv", "tv", null);

		Element root = doc.getDocumentElement();

		channelOrder.getXML(doc, root, m_store.getChannels(), null, true);

		Element item = doc.createElement("defaultDemodulatorData");
		Text text = doc.createTextNode(Channel.getDefaultDemodulatorData());
		item.appendChild(text);
		root.appendChild(item);
		item = doc.createElement("defaultUserData");
		text = doc.createTextNode(Channel.getDefaultUserData());
		item.appendChild(text);
		root.appendChild(item);

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "surfOrder.xsl", m_headers, m_urlData, true);

		return transformer.transform();
	}

	private byte[] resetCustomOrder() throws Exception
	{
		ChannelOrder channelOrder = new ChannelOrder(m_store.m_channelOrder);
		channelOrder.resetCustomOrdering(m_store.getChannels());

		return showChannelOrder(channelOrder);
	}

	private byte[] sortCustomOrder() throws Exception
	{
		String sort = m_urlData.getParameter("sort");

		if (sort != null && !sort.isEmpty())
		{
			ChannelOrder channelOrder = new ChannelOrder(m_store.m_channelOrder);
			channelOrder.sortCustomOrdering(m_store.getChannels(), Integer.parseInt(sort));
			return showChannelOrder(channelOrder);
		}

		return showChannelOrder(m_store.m_channelOrder);
	}

	private byte[] addMissingToCustomOrder() throws Exception
	{
		ChannelOrder channelOrder = new ChannelOrder(m_store.m_channelOrder);
		channelOrder.checkCustomOrderingMissing(m_store.getChannels());

		return showChannelOrder(channelOrder);
	}

	private byte[] updateCustomOrder() throws Exception
	{
		String chanMatch = m_urlData.getParameter("chanMatch");

		m_store.m_channelOrder.setCustomOrdering(chanMatch, m_store.getChannels());

		String useDefault = m_urlData.getParameter("useDefault");
		m_store.m_channelOrder.setUse(useDefault.equals("0") ? false : true);

		m_store.saveChannels(null);

		//return getServletRedir("?action=14").getBytes("UTF-8");
		return showChannels();
	}

	private byte[] updateDefaultData() throws Exception
	{
		String data = m_urlData.getParameter("defaultDemodulatorData", "");
		Channel.setDefaultDemodulatorData(data);
		data = m_urlData.getParameter("defaultUserData", "");
		Channel.setDefaultUserData(data);

		m_store.saveChannels(null);

		//return getServletRedir("?action=14").getBytes("UTF-8");
		return showChannels();
	}

	private byte[] showChannelGroup() throws Exception
	{
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("tv", "tv", null);

		Element root = doc.getDocumentElement();

		String sChannel = m_urlData.getParameter("channelName");
		Channel channel = m_store.getChannel(sChannel);
		if (channel != null)
		{
			root.setAttribute("channelName", sChannel);
			root.setAttribute("channelNumber", Integer.toString(channel.getNumber()));
			m_store.m_channelOrder.getXML(doc, root, m_store.getChannels(), channel.getChannels(), false);
		}
		else
			m_store.m_channelOrder.getXML(doc, root, m_store.getChannels(), new Vector<String>(), false);

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "channelGroup.xsl", m_headers, m_urlData, true);

		return transformer.transform();
	}

	private byte[] addUpdateChannelGroup() throws Exception
	{
		String update = m_urlData.getParameter("update");
		boolean bUpdate = false;
		if (update != null && update.equals("Update"))
			bUpdate = true;

		String chanMatch = m_urlData.getParameter("chanMatch");
		if (chanMatch != null)
		{
			Vector<String> group = new Vector<String>();
			String[] chanMatches = chanMatch.split("\n");
			for (int x = 0; x < chanMatches.length; x++)
			{
				String sc = chanMatches[x].trim();
				Channel ch = m_store.getChannel(sc);
				if ( (ch != null) && !ch.isGroup() )
					group.add(sc);
			}

			if (group.isEmpty())
				return showChannels();

			String sOldChannel = m_urlData.getParameter("oldChannelName");
			if ( (sOldChannel != null) && !sOldChannel.isEmpty() && bUpdate )
			{
				Channel oldChannel = m_store.getChannel(sOldChannel);
				if (oldChannel != null)
				{
					String sChannel = m_urlData.getParameter("channelName");
					if (sChannel != null)
						oldChannel.setName(sChannel);
					String sChannelNumber = m_urlData.getParameter("channelNumber");
					if (sChannelNumber != null)
					{
						int number = -1;
						try
						{
							number = Integer.parseInt(sChannelNumber.trim());
						}
						catch (Exception e)	{}
						oldChannel.setNumber(number);
					}
					oldChannel.setChannels(group);

					GuideStore guide = GuideStore.getInstance();
					m_store.updateChannel(sOldChannel, oldChannel);
					sChannel = oldChannel.getName();
					updateScheduleItems(sOldChannel, sChannel);
					guide.renameEpgExclusionChannel(sOldChannel, sChannel);
					updateEpgChannelMapping(sOldChannel, sChannel);
					updateChannels(sOldChannel, sChannel);
					m_store.saveChannels(null);
				}
			}
			else
			{
				Channel channel = new Channel();
				String sChannel = m_urlData.getParameter("channelName");
				if (sChannel != null)
					channel.setName(sChannel);
				String sChannelNumber = m_urlData.getParameter("channelNumber");
				if (sChannelNumber != null)
				{
					int number = -1;
					try
					{
						number = Integer.parseInt(sChannelNumber.trim());
					}
					catch (Exception e)	{}
					channel.setNumber(number);
				}
				channel.setChannels(group);
				m_store.addChannel(channel);
				m_store.saveChannels(null);
			}
		}

		return showChannels();
	}

	private Channel getDemodulatorSetting(int index) throws Exception
	{
		String x = index > -1 ? Integer.toString(index) : "";

		String networkType = m_urlData.getParameter("networkType" + x, "DVB-T");
		if (!m_store.isNetworkTypeValid(networkType))
			throw new Exception("Network type not valid: " + networkType);

		String demodulatorName = checkAlias(m_urlData.getParameter("demodulatorName" + x, ""));
		int freq = m_urlData.getParameterInt("freq" + x, -2);
		String modulation = m_urlData.getParameter("modulation" + x, "");
		int symbolRate = m_urlData.getParameterInt("symbolRate" + x, -2);
		int physicalChannel = m_urlData.getParameterInt("physicalChannel" + x, -2);
		int band = m_urlData.getParameterInt("band" + x, -2);
		String polarization = m_urlData.getParameter("polarization" + x, "");
		int highOsc = m_urlData.getParameterInt("highOsc" + x, -2);
		int lowOsc = m_urlData.getParameterInt("lowOsc" + x, -2);
		int lnbSwitch = m_urlData.getParameterInt("lnbSwitch" + x, -2);
		double longitude = m_urlData.getParameterDouble("longitude" + x, -2);
		int westLongitude = m_urlData.getParameterInt("westLongitude" + x, -2);
		double azimuth = m_urlData.getParameterDouble("azimuth" + x, -2);
		double elevation = m_urlData.getParameterDouble("elevation" + x, -2);
		String inputRange = m_urlData.getParameter("inputRange" + x, "");
		String pilot = m_urlData.getParameter("pilot" + x, "");
		String rollOff = m_urlData.getParameter("rollOff" + x, "");
		String diseqc = m_urlData.getParameter("diseqc" + x, "");
		String demodulatorData = m_urlData.getParameter("demodulatorData" + x, "");
		int originalNetworkId = m_urlData.getParameterInt("originalNetworkId" + x, -2);
		int transportStreamId = m_urlData.getParameterInt("transportStreamId" + x, -2);
		int sourceId = m_urlData.getParameterInt("sourceId" + x, -2);
		String characterSet = m_urlData.getParameter("characterSet" + x, "");

		return new Channel(demodulatorName, networkType, freq, modulation, symbolRate, physicalChannel, band, polarization, highOsc, lowOsc, 
				lnbSwitch, longitude, westLongitude, azimuth, elevation, inputRange, pilot, rollOff, diseqc, demodulatorData, 
				originalNetworkId, transportStreamId, sourceId, characterSet);
	}

	protected class Region implements Comparable<Region>
	{
		String m_name;
		Vector<String[]> m_stations = null;

		public int compareTo(Region compareRegion)
		{
			//ascending order
			return m_name.compareTo(compareRegion.m_name);
		}
	}

	private byte[] parseStationList() throws Exception
	{
		String country = "0";
		int regionStart = 0;
		String bandwidth = "7";

		BufferedReader r = new BufferedReader(new FileReader("stationList.txt"));
		String line = null;
		Vector< Region > regions = new Vector< Region >();
		Region region = null;
		Vector<String[]> stations = null;
		String station, frequency1, frequency2;
		Integer f1, f2;

		while ((line = r.readLine()) != null)
		{
			line = line.trim();

			if (line.startsWith("Transmitter"))
			{
				DVScheduler.log(line);
				if ( (region != null) && (stations != null) && (stations.size() > 0) )
				{
					region.m_stations = stations;
					regions.add( region );
				}
				region = new Region();
				String[] data = line.split(":");
				region.m_name = data[data.length-1].trim();
				stations = new Vector<String[]>();
			}
			else
			{
				String[] data = line.split("\t");
				if ( (data.length == 5) && !data[0].trim().equals("Service") && !data[1].trim().equals("Channel") )
				{
					DVScheduler.log(line);
					station = data[0].trim();
					frequency1 = data[2].trim();
					f1 = (int) (Double.parseDouble(frequency1) * 1000);
					frequency2 = data[4].trim();
					f2 = (int) (Double.parseDouble(frequency2) * 1000);
					if (frequency1.equals(frequency2))
						stations.add( new String[]{station, f1.toString()} );
					else
					{
						stations.add( new String[]{station + " (old)", f1.toString()} );
						stations.add( new String[]{station + " (new)", f2.toString()} );
					}
				}
				else if (data.length == 6)
				{
					DVScheduler.log(line);
					station = data[0].trim();
					frequency1 = data[3].trim();
					f1 = (int) (Double.parseDouble(frequency1) * 1000);
					frequency2 = data[5].trim();
					f2 = (int) (Double.parseDouble(frequency2) * 1000);
					if (frequency1.equals(frequency2))
						stations.add( new String[]{station, f1.toString()} );
					else
					{
						stations.add( new String[]{station + " (old)", f1.toString()} );
						stations.add( new String[]{station + " (new)", f2.toString()} );
					}
				}
			}
		}
		r.close();

		if (stations.size() > 0)
		{
			region.m_stations = stations;
			regions.add( region );
		}

		StringBuffer buffer = new StringBuffer();
		buffer.append("HTTP/1.0 200 OK\n");
		buffer.append("Content-Disposition: attachment; filename=\"stationData.txt\"\n");
		buffer.append("Pragma: no-cache\n");
		buffer.append("Cache-Control: no-cache\n");
		buffer.append("\n");

		Collections.sort(regions);
		for (int i=0; i<regions.size(); i++)
		{
			buffer.append("region." + country + "." + (regionStart+i) + "=" + regions.get(i).m_name + "\n");
		}
		buffer.append("\n");

		for (int i=0; i<regions.size(); i++)
		{
			stations = regions.get(i).m_stations;
			for (int j=0; j<stations.size(); j++)
			{
				buffer.append("station." + country + "." + (regionStart+i) + "." + j + ".name=" + stations.get(j)[0] + "\n");
				buffer.append("station." + country + "." + (regionStart+i) + "." + j + ".freq=" + stations.get(j)[1] + "\n");
				buffer.append("station." + country + "." + (regionStart+i) + "." + j + ".band=" + bandwidth + "\n");
			}
		}
		return buffer.toString().getBytes("UTF-8");
	}
}
