/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;
import java.io.*;

class ChannelList
{
	private FileInputStream fileList = null;
	private Properties prop = null;
	private String m_networkType = null;

	public ChannelList(String list, String networkType) throws Exception
	{
		m_networkType = networkType;
		prop = new Properties();
		if (list == null)
		{
			String stationList = "";
			if (m_networkType.equals("DVB-T"))
				stationList = "stationdata.list";
			else if (m_networkType.equals("ATSC"))
				stationList = "stationdataATSC.list";
			else if (m_networkType.equals("QAM"))
				stationList = "stationdataCable.list";
			DataStore store = DataStore.getInstance();
			fileList = new FileInputStream(store.getProperty("path.data", "data")
					+ File.separator + stationList);
		}
		else
			fileList = new FileInputStream(list);
		prop.load(fileList);
	}

	public String[] getCountries() throws Exception
	{
		List<String> countries = new LinkedList<String>();

		String country = "";
		for(int x = 0; x < 1000; x++)
		{
			country = prop.getProperty("country." + x);
			if(country != null)
			{
				country = country.trim();
				countries.add(country + "\n" + x);
			}
		}

		Collections.sort(countries);
		return countries.toArray(new String[0]);
	}

	public String[] getRegions(int country) throws Exception
	{
		List<String> regions = new LinkedList<String>();

		String region = "";
		for(int x = 0; x < 1000; x++)
		{
			region = prop.getProperty("region." + country + "." + x);
			if(region != null)
			{
				region = region.trim();
				regions.add(region + "\n" + x);
			}
		}

		Collections.sort(regions);
		return regions.toArray(new String[0]);
	}

	public Channel[] getStations(int country, int region) throws Exception
	{
		Channel[] channels = new Channel[1000];

		for(int x = 0; x < channels.length; x++)
		{
			String name = prop.getProperty("station." + country + "." + region + "." + x + ".name");
			String freq = prop.getProperty("station." + country + "." + region + "." + x + ".freq");
			String band = prop.getProperty("station." + country + "." + region + "." + x + ".band");         

			//if (name != null && freq != null && band != null)
			if (name != null)
			{
				channels[x] = new Channel(name);
				channels[x].setNetworkType(m_networkType);
				int freqINT = -2;
				try
				{
					freqINT = Integer.parseInt(freq.trim());
					channels[x].setFrequency(freqINT);
				}
				catch(Exception e)
				{
				}
				int bandINT = -2;
				try
				{
					bandINT = Integer.parseInt(band.trim());
					channels[x].setBandWidth(bandINT);
				}
				catch(Exception e)
				{
				}
			}
			else
			{
				channels[x] = null;
			}
		}

		return channels;
	}

	public void close() throws Exception
	{
		fileList.close();
	}
}
