/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;

public class CommandWaitThread implements Runnable
{
	private String command = "";
	private int delay = 0;
	private Process process = null; 
	private boolean finished = false;
	private boolean delaying = false;

	private StringBuffer out = new StringBuffer();
	private StringBuffer err = new StringBuffer();

	private Integer exitCode = null;

	public CommandWaitThread(String com, int pause)
	{
		command = com;
		delay = pause;
	}  

	public boolean isFinished()
	{
		return this.finished;
	}

	public boolean isDelaying()
	{
		return delaying;
	}

	public String getInputStream()
	{
		return out.toString();
	}

	public String getErrorStream()
	{
		return err.toString();
	}   

	public Integer getExitCode()
	{
		return exitCode;
	}

	public void stop()
	{
		try
		{
			if(process != null)
				process.destroy();
		}
		catch(Exception e){}

		this.finished = true;
	}

	public int getDelay()
	{
		return delay;
	}

	public void run()
	{
		delaying = true;
		try
		{      
			if(delay > 0)
			{
				DVScheduler.log("Delaying for " + delay + " seconds before running command");

				while(delay > 0)
				{
					Thread.sleep(1000);
					delay -= 1;
				}
			}

			if(this.finished == false)
			{
				Runtime runner = Runtime.getRuntime();
				DVScheduler.log("Running:" + command);
				process = runner.exec(Formatter.parseCommand(command));
				new ProcessInputReaderThread(process.getInputStream(), out);
				new ProcessInputReaderThread(process.getErrorStream(), err);

				this.delaying = false;

				exitCode = process.waitFor();
			}
			DVScheduler.log("Running command finished");
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		this.delaying = false;
		this.finished = true;
	}

	private class ProcessInputReaderThread implements Runnable
	{
		InputStream stream = null;
		StringBuffer data = null;

		public ProcessInputReaderThread(InputStream in, StringBuffer buff)
		{
			stream = in;
			data = buff;

			new Thread(Thread.currentThread().getThreadGroup(), this , this.getClass().getName()).start();
		}

		public void run()
		{
			try
			{
				int ch;
				while((ch = stream.read()) != -1)
				{
					data.append((char)ch);
				}
			}
			catch (Exception e){}    
			//DVScheduler.log("ProcessInputReaderThread Exiting");
		}
	}
}