/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

public class EpgMatch
{
	private boolean enabled = true;
	private boolean enabledMark = true;
	private long m_id = 0;
	private String m_name = "";

	private Vector<String> matchList = new Vector<String>();

	private boolean m_autoDelete = false;
	private int keepFor = 30;
	private int startBuffer = 4;
	private int endBuffer = 5;
	private int epgBuffer = 5;
	private String postTask = "";
	private String m_captureType = "Capture.deftype";
	private String m_capturePath = "*";
	private String fileNamePattern = "(%y-%m-%d %h-%M) %n [%c]\n%n [%c]\n%n\n%n\\%y-%m-%d";
	private boolean m_bDuplicateDetection = true;
	private boolean m_bMergeDoubleEpisode = false;

	public EpgMatch()
	{
		// Get default values.
		DataStore store = DataStore.getInstance();
		m_id = store.m_rand.nextLong();

		startBuffer = store.getPropertyInt("Schedule.buffer.start", 4);
		endBuffer = store.getPropertyInt("Schedule.buffer.end", 5);
		epgBuffer = store.getPropertyInt("Schedule.buffer.EPG", 5);

		String[] namePatterns = store.getNamePatterns();
		if (namePatterns.length > 0)
			fileNamePattern = namePatterns[0];

		postTask = store.getProperty("Tasks.DefTask", "");
		m_autoDelete = store.isAutoDelByDefault();
		keepFor = store.getPropertyInt("AutoDel.KeepFor", 30);
	}

	public String getName()
	{
		return m_name;
	}

	public String getName2()
	{
		if (m_name.isEmpty() && !matchList.isEmpty())
		{
			String name = matchList.get(0);
			for (int i=1; i<matchList.size(); i++)
			{
				name += ", " + matchList.get(i);
			}
			return name;
		}
		else
			return m_name;
	}

	public void setName(String name)
	{
		m_name = name;
	}

	public Vector<String> getMatchListNames()
	{
		return matchList;
	}

	public void addMatchListName(String name)
	{
		if(!matchList.contains(name))
			matchList.add(name);
	}

	public void remMatchListName(int index)
	{
		matchList.remove(index);
	}   

	public void renameMatchListName(String oldName, String newName)
	{
		if(matchList.contains(oldName))
		{
			while(matchList.remove(oldName));

			if(!matchList.contains(newName))
				matchList.add(newName);         
		}
	} 

	public void removeMatchListName(String name)
	{
		while(matchList.remove(name));
	} 

	public EpgMatch(Node node)
	{
		parseXML(node);
	}

	public boolean isEnabled()
	{
		return enabled;
	}

	public void setEnabled(boolean enable)
	{
		enabled = enable;
	}

	public boolean isEnabledMark()
	{
		return enabledMark;
	}

	public void setEnabledMark(boolean enable)
	{
		enabledMark = enable;
	}

	public long getID()
	{
		return m_id;
	}

	public void setID(long newID)
	{
		m_id = newID;
	}

	public void setCaptureType(String type)
	{
		m_captureType = ((type == null) || type.isEmpty() || type.equals("*UNDEFINED*")) ? "Capture.deftype" : type;
	}

	public String getCaptureType()
	{
		return m_captureType;
	}

	public void setPostTask(String task)
	{
		postTask = ((task == null) || task.equals("*UNDEFINED*")) ? "" : task;
	}

	public String getPostTask()
	{
		return postTask;
	}

	public void setStartBuffer(int buff)
	{
		startBuffer = buff;
	}

	public int getStartBuffer()
	{
		return startBuffer;
	}

	public void setEndBuffer(int buff)
	{
		endBuffer = buff;
	}

	public int getEndBuffer()
	{
		return endBuffer;
	}

	public void setEPGBuffer(int buff)
	{
		epgBuffer = buff;
	}

	public int getEPGBuffer()
	{
		return epgBuffer;
	}

	public int getKeepFor()
	{
		return keepFor;
	}

	public void setKeepFor(int keep)
	{
		keepFor = keep;
	}

	public boolean isAutoDelete()
	{
		return m_autoDelete;
	}

	public void setAutoDelete(boolean del)
	{
		m_autoDelete = del;
	}

	public void setFileNamePattern(String pattern)
	{
		fileNamePattern = ( (pattern == null) || pattern.isEmpty() || pattern.equals("*UNDEFINED*") ) ? "%n" : pattern;
	}

	public String GetFileNamePattern()
	{
		return fileNamePattern;
	}

	public void setCapturePath(String capturePath)
	{
		m_capturePath = ( (capturePath == null) || capturePath.equals("*UNDEFINED*") ) ? "*" : capturePath;
	}

	public String getCapturePath()
	{
		return m_capturePath;
	}

	public void setDuplicateDetection(boolean value)
	{
		m_bDuplicateDetection = value;
	}

	public boolean isDuplicateDetection()
	{
		return m_bDuplicateDetection;
	}

	public void setMergeDoubleEpisode(boolean value)
	{
		m_bMergeDoubleEpisode = value;
	}

	public boolean isMergeDoubleEpisode()
	{
		return m_bMergeDoubleEpisode;
	}

	public boolean getXML(Document doc, Element node, boolean bMatchListDetails)
	{
		node.setAttribute("id", new Long(m_id).toString());
		node.setAttribute("name", getName());
		node.setAttribute("enabled", new Boolean(enabled).toString());
		node.setAttribute("enabledMark", new Boolean(enabledMark).toString());
		node.setAttribute("captureType", m_captureType);

		Element elm = null;
		Element elm2 = null;
		Text text = null;

		DataStore store = DataStore.getInstance();
		elm = doc.createElement("matchList");
		for(int x = 0; x < matchList.size(); x++)
		{
			String name = (String)matchList.get(x);
			elm2 = doc.createElement("matchListName");
			EpgMatchList epgMatchList = store.getEpgMatchList(name);
			if (epgMatchList != null)
			{
				elm2.setAttribute("channels", epgMatchList.getChannels());
				elm2.setAttribute("days", epgMatchList.getDays());
				elm2.setAttribute("spans", epgMatchList.getSpans());
			}
			text = doc.createTextNode(name);
			elm2.appendChild(text);
			elm.appendChild(elm2);         
		}

		node.appendChild(elm);

		elm = doc.createElement("postTask");
		text = doc.createTextNode(postTask);
		elm.appendChild(text);
		node.appendChild(elm);

		elm = doc.createElement("capturePath");
		text = doc.createTextNode(m_capturePath);
		elm.appendChild(text);
		node.appendChild(elm);      

		elm = doc.createElement("fileNamePattern");
		text = doc.createTextNode(fileNamePattern);
		elm.appendChild(text);
		node.appendChild(elm);      

		elm = doc.createElement("autoDel");
		elm.setAttribute("enabled", new Boolean(m_autoDelete).toString());
		elm.setAttribute("keepFor", new Integer(keepFor).toString());
		node.appendChild(elm);

		elm = doc.createElement("buffer");
		elm.setAttribute("start", new Integer(startBuffer).toString());
		elm.setAttribute("end", new Integer(endBuffer).toString());
		elm.setAttribute("epg", new Integer(epgBuffer).toString());
		node.appendChild(elm);

		elm = doc.createElement("duplicateDetection");
		elm.setAttribute("enabled", new Boolean(m_bDuplicateDetection).toString());
		node.appendChild(elm);

		elm = doc.createElement("mergedoubleepisode");
		elm.setAttribute("enabled", new Boolean(m_bMergeDoubleEpisode).toString());
		node.appendChild(elm);

		return true;
	}

	public boolean getXML(Document doc, Element node)
	{
		return getXML(doc, node, false);
	}

	private void parseXML(Node node)
	{
		NamedNodeMap itemAttribs = node.getAttributes();
		m_id = Long.parseLong(itemAttribs.getNamedItem("id").getTextContent());

		Node found = itemAttribs.getNamedItem("name");
		m_name = found == null ? "" : found.getTextContent();

		try
		{
			if("true".equals(itemAttribs.getNamedItem("enabled").getTextContent()))
				enabled = true;
			else
				enabled = false;
		}
		catch (Exception e) {}

		try
		{
			if("true".equals(itemAttribs.getNamedItem("enabledMark").getTextContent()))
				enabledMark = true;
			else
				enabledMark = false;
		}
		catch (Exception e) {}

		m_captureType = itemAttribs.getNamedItem("captureType").getTextContent();

		NodeList nl = node.getChildNodes();

		for(int x = 0; x < nl.getLength(); x++)
		{
			found = nl.item(x);
			if(found.getNodeName().equals("matchList"))
			{
				NodeList nl2 = found.getChildNodes();
				for(int y = 0; y < nl2.getLength(); y++)
				{
					Node matches = nl2.item(y);
					if(matches.getNodeName().equals("matchListName"))
					{
						matchList.add(matches.getTextContent());
					}
				}
			}
			else if(found.getNodeName().equals("postTask"))
			{
				postTask = found.getTextContent();
			}  
			else if(found.getNodeName().equals("capturePath"))
			{
				m_capturePath = found.getTextContent();
			}          
			else if(found.getNodeName().equals("fileNamePattern"))
			{
				fileNamePattern = found.getTextContent();
			}          
			else if(found.getNodeName().equals("autoDel"))
			{
				NamedNodeMap attribs = found.getAttributes();
				Node keep = attribs.getNamedItem("keepFor");
				try
				{
					keepFor = Integer.parseInt(keep.getTextContent());
				}
				catch (Exception e) {}

				keep = attribs.getNamedItem("enabled");
				if("true".equals(keep.getTextContent()))
					m_autoDelete = true;
				else
					m_autoDelete = false;            
			}
			else if(found.getNodeName().equals("buffer"))
			{
				NamedNodeMap attribs = found.getAttributes();
				Node buff = attribs.getNamedItem("start");
				try
				{
					startBuffer = Integer.parseInt(buff.getTextContent());
				}
				catch (Exception e) {}

				buff = attribs.getNamedItem("end");
				try
				{
					endBuffer = Integer.parseInt(buff.getTextContent());
				}
				catch (Exception e) {}

				buff = attribs.getNamedItem("epg");
				try
				{
					epgBuffer = Integer.parseInt(buff.getTextContent());            
				}
				catch (Exception e) {}

			}
			else if(found.getNodeName().equals("duplicateDetection"))
			{
				NamedNodeMap attribs = found.getAttributes();

				Node node2 = attribs.getNamedItem("enabled");
				if("true".equals(node2.getTextContent()))
					m_bDuplicateDetection = true;
				else
					m_bDuplicateDetection = false;
			}
			else if(found.getNodeName().equals("mergedoubleepisode"))
			{
				NamedNodeMap attribs = found.getAttributes();

				Node node2 = attribs.getNamedItem("enabled");
				if("true".equals(node2.getTextContent()))
					m_bMergeDoubleEpisode = true;
				else
					m_bMergeDoubleEpisode = false;
			}
		}
	}

}
