/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.text.FieldPosition;
import java.text.SimpleDateFormat;
import java.util.*;
import java.net.*;
import java.util.zip.*;

public class GuideDataLoader
{
	public GuideDataLoader()
	{
	}

	public Vector<byte[]> getDataFromXMLservice(String programUrl, String channelNames, int days) throws Exception
	{
		Vector<byte[]> resultData = new Vector<byte[]>();

		//Date format 2005-01-22
		SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
		Calendar cal = Calendar.getInstance();

		String[] channelArray = channelNames.split(";");

		for(int y = 0; y < days; y++)
		{      
			StringBuffer buff = new StringBuffer();
			df.format(cal.getTime(), buff, new FieldPosition(0));
			cal.add(Calendar.DATE, 1);

			for(int x = 0; x < channelArray.length; x++)
			{
				String channel = channelArray[x];
				channel = channel.trim();
				if(channel.length() > 0)
				{
					String dataUrl = programUrl;
					dataUrl = dataUrl.replaceAll("<channel>", channel);

					dataUrl = dataUrl.replaceAll("<date>", buff.toString());

					DVScheduler.log("Loading XMLTV data from " + dataUrl);

					Vector<byte[]> data = new Vector<byte[]>();
					int result = getContentUsingThread(new URL(dataUrl), data, "", "");

					if(result != 200)
						return resultData;

					if(data.size() > 0)
					{
						resultData.add((byte[])data.get(0));
					}
				}
			}

		}

		return resultData;
	}

	public Vector<byte[]> getDataFromURL(String loadURL, String username, String password) throws Exception
	{
		Vector<byte[]> resultData = new Vector<byte[]>();

		DVScheduler.log("Loading XMLTV data from URL : " + loadURL);

		Vector<byte[]> data = new Vector<byte[]>();
		getContentUsingThread(new URL(loadURL), data, username, password);

		String pageData = "";
		if(data.size() == 1)
		{
			pageData = new String((byte[])data.get(0));  
		}

		if(pageData.length() > 0 && pageData.indexOf("<XMLTV_FILE_LIST>") > -1)
		{
			DVScheduler.log("Loading XMLTV data from file list");
			String[] fileList = extractNames(pageData);

			int lastIndex = loadURL.lastIndexOf("/");
			if(lastIndex > -1)
			{
				String baseUrl = loadURL.substring(0, lastIndex+1);

				for (int x = 0; x < fileList.length; x++)
				{
					String dataURL = baseUrl + fileList[x];
					DVScheduler.log("Loading XMLTV data from URL : " + dataURL);

					Vector<byte[]> pages = new Vector<byte[]>();
					int result = getContentUsingThread(new URL(dataURL), pages, username, password);

					if(result == 200)
					{
						for(int y = 0; y < pages.size(); y++)
						{
							byte[] pd = (byte[])pages.get(y);

							if(pd.length > 0)
								resultData.add(pd);
						}
					}
				}
			}
		}
		else if(data.size() > 0)
		{
			for(int y = 0; y < data.size(); y++)
			{
				byte[] pd = (byte[])data.get(y);

				if(pd.length > 0)
					resultData.add(pd);
			}
		}

		return resultData;
	}

	private String[] extractNames(String nameList)
	{
		DVScheduler.log("Extracting File list from Listing File");

		String names[] = nameList.split("\n");

		Vector<String> namesOfFiles = new Vector<String>();

		if(names.length > 1)
		{
			for (int x = 1; x < names.length; x++)
			{
				if(names[x].trim().length() > 0)
				{
					namesOfFiles.add(names[x].trim());
					DVScheduler.log("Adding : " + names[x].trim());
				}
			}
		}

		return (String[]) namesOfFiles.toArray(new String[0]);
	}

	private int getContentUsingThread(URL location, Vector<byte[]> buffs, String username, String password)
	{
		Thread loaderThread = null;
		LoaderTread loader = null;
		try
		{
			loader = new LoaderTread(location, username, password);
			loaderThread = new Thread(Thread.currentThread().getThreadGroup(), loader, loader.getClass().getName());
			loaderThread.start();

			while(!loader.isFinished())
			{
				if(loader.isTimedOut(30))
				{
					DVScheduler.log("\nTimeout reached : Killing loader Thread");
					loader.kill();
					loaderThread.interrupt();
					break;
				}
				Thread.sleep(500);
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		loaderThread.interrupt(); 

		if(loader.getResponceCode() != 200)
			return loader.getResponceCode();

		if(location.toString().toUpperCase().indexOf(".ZIP", location.toString().length() - 4) > -1)
		{
			if(unZip(loader.getDataBytes(), buffs) > 0)
				return 200;
			else
				return 404;
		}
		else
		{
			buffs.add(loader.getDataBytes());
			return loader.getResponceCode();
		}
	}    

	private int unZip(byte[] data, Vector<byte[]> extractedXML)
	{
		ByteArrayInputStream bais = new ByteArrayInputStream(data);
		ByteArrayOutputStream baos = null;
		ZipInputStream zis = new ZipInputStream(bais);
		int count = 0;

		try
		{
			ZipEntry ze = zis.getNextEntry();

			while (ze != null)
			{
				if (ze.getName() != null && ze.getName().indexOf(".xml") > -1)
				{
					baos = new ByteArrayOutputStream();
					DVScheduler.log("Extracting : " + ze.getName());

					int len;
					byte[] buf = new byte[1024];
					while ((len = zis.read(buf)) > 0)
					{
						baos.write(buf, 0, len);
					}
					extractedXML.add(baos.toByteArray());
					count++;
				}
				ze = zis.getNextEntry();
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return count;
	}

	public Vector<byte[]> getDataFromFiles(String loadDir, StringBuffer buff, int format)
	{
		return getDataFromFiles(loadDir, buff, format, false);
	}

	public Vector<byte[]> getDataFromFiles(String loadDir, StringBuffer buff, int format, boolean bCopy)
	{
		Vector<byte[]> resultData = new Vector<byte[]>();

		File xmlDir = new File(loadDir);
		String actualFile = "";

		if ( !bCopy && !xmlDir.canRead() )
		{
			DVScheduler.log("Can read is " + xmlDir.canRead() + ": " + xmlDir.getAbsolutePath());
			buff.append("Can read is " + xmlDir.canRead() + ": " + xmlDir.getAbsolutePath());
			if(format == 1)
				buff.append("<br>");
			buff.append("\n");
			DataStore store = DataStore.getInstance();
			String dataPath = store.getProperty("path.userData", "");
			if (dataPath.isEmpty())
				return resultData;
			dataPath += File.separator + "temp";
			// Delete temp folder.
			File fDataPath = new File(dataPath);
			if ( !fDataPath.exists() )
				fDataPath.mkdirs();

			File[] files = fDataPath.listFiles();
			if (files != null)
				for(File f: files)
					f.delete();
			File fCopy = new File(dataPath + File.separator + "toCopy.txt");
			try
			{
				FileWriter writer = new FileWriter(fCopy);
				writer.write(xmlDir.getAbsolutePath());
				writer.close();
			}
			catch (Exception e2)
			{
				DVScheduler.log("Error writing " + fCopy.getAbsolutePath() + " : " + e2.toString());
				return resultData;
			}
			DVScheduler.log("Copying " + xmlDir.getAbsolutePath() + " to " + dataPath);
			buff.append("Copying " + xmlDir.getAbsolutePath() + " to " + dataPath);
			if(format == 1)
				buff.append("<br>");
			buff.append("\n");

			int timer = 0;
			while (timer++ < 30)
			{
				if (!fCopy.exists())
					return getDataFromFiles(dataPath, buff, format, true);
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
				DVScheduler.log("Waiting for copy to finish... ");
			}

			fCopy.delete();
			DVScheduler.log("Cannot copy " + xmlDir.getAbsolutePath());
			buff.append("Cannot copy " + xmlDir.getAbsolutePath());
			if(format == 1)
				buff.append("<br>");
			buff.append("\n");
		}
		else if (xmlDir.isDirectory())
		{
			DVScheduler.log("Loading folder " + xmlDir.getAbsolutePath());
			buff.append("Loading folder " + xmlDir.getAbsolutePath());
			if(format == 1)
				buff.append("<br>");
			buff.append("\n");
			String[] fileList = xmlDir.list();
			for (int x = 0; x < fileList.length; x++)
			{
				if (fileList[x].toLowerCase().endsWith(".xml"))
				{
					actualFile = xmlDir.getPath() + File.separator + fileList[x];

					DVScheduler.log("Loading XMLTV data from " + actualFile);
					buff.append("Loading XMLTV data from " + actualFile);
					if(format == 1)
						buff.append("<br>");
					buff.append("\n");

					byte[] data = null;
					try
					{
						data = getFileContents(actualFile);
					}
					catch(Exception e)
					{
						e.printStackTrace();
					}

					if(data != null && data.length > 10)
						resultData.add(data);
				}
			}
		}
		else if (xmlDir.isFile())
		{
			actualFile = xmlDir.getPath();

			DVScheduler.log("Loading XMLTV data from " + actualFile);
			buff.append("Loading XMLTV data from " + actualFile);
			if(format == 1)
				buff.append("<br>");
			buff.append("\n");

			byte[] data = null;
			try
			{
				data = getFileContents(actualFile);
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}

			if(data != null && data.length > 10)
				resultData.add(data);
		}

		return resultData;
	}

	public byte[] getFileContents(String fileName) throws Exception
	{
		ByteArrayOutputStream dataBytes = new ByteArrayOutputStream();
		FileInputStream fis = new FileInputStream(fileName);

		byte[] buffer = new byte[1024];

		int read = fis.read(buffer);
		if (read < 0)
		{
			fis.close();
			return dataBytes.toByteArray();
		}

		int checkLength = read - 10;
		int nFound = -1;
		checker:
			for (int i=0; i<checkLength; i++)
			{
				switch (buffer[i])
				{
				case 0:
					break checker;
				case '<':
					if ( (buffer[i+1] == '!') && (buffer[i+2] == 'D') && (buffer[i+3] == 'O') && (buffer[i+4] == 'C') && (buffer[i+5] == 'T') && (buffer[i+6] == 'Y') && (buffer[i+7] == 'P') && (buffer[i+8] == 'E') && ((buffer[i+9] == ' ') || (buffer[i+9] == '\t') || (buffer[i+9] == '\r') || (buffer[i+9] == '\n')) )
						break checker;
					else if ( (buffer[i+1] == 't') && (buffer[i+2] == 'v') && ((buffer[i+3] == ' ') || (buffer[i+3] == '\t') || (buffer[i+3] == '>') || (buffer[i+3] == '\r') || (buffer[i+3] == '\n')) )
					{
						nFound = i;
						break checker;
					}
					break;
				}
			}

		if (nFound >= 0)
		{
			dataBytes.write(buffer, 0, nFound);
			dataBytes.write("<!DOCTYPE tv [<!ENTITY % xhtml-lat1 SYSTEM \"xhtml-lat1.ent\"> %xhtml-lat1; <!ENTITY % xhtml-special SYSTEM \"xhtml-special.ent\"> %xhtml-special;  <!ENTITY % xhtml-symbol SYSTEM \"xhtml-symbol.ent\"> %xhtml-symbol;]>".getBytes());
			dataBytes.write(buffer, nFound, read-nFound);
		}
		else
			dataBytes.write(buffer, 0, read);

		read = fis.read(buffer);
		while(read > -1)
		{
			dataBytes.write(buffer, 0, read);
			read = fis.read(buffer);
		}

		fis.close();

		//printHex(ba.toByteArray());

		return dataBytes.toByteArray();
	}

	void printHex(byte[] b)
	{
		for (int i = 0; i < b.length; ++i)
		{
			if(i % 16 == 0)
			{
				System.out.print(Integer.toHexString((i & 0xFFFF) | 0x10000).substring(1, 5) + " - ");
			}

			System.out.print(Integer.toHexString((b[i] & 0xFF) | 0x100).substring(1, 3) + " ");

			if(i % 16 == 15 || i == b.length - 1)
			{
				int j;
				for (j = 16 - i % 16; j > 1; --j)
				{
					System.out.print("   ");
				}

				System.out.print(" - ");
				int start = (i / 16) * 16;
				int end = (b.length < i + 1) ? b.length : (i + 1);

				for (j = start; j < end; ++j)
				{
					if(b[j] >= 32 && b[j] <= 126)
					{
						System.out.print((char) b[j]);
					}
					else
					{
						System.out.print(".");
					}
				}

				System.out.println();
			}
		}
		System.out.println();
	}
}