/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.net.*;

class HTTPAuthenticator extends Authenticator
{
	private String m_username = "";
	private String m_password = "";

	public HTTPAuthenticator(String username, String password)
	{
		m_username = username;
		m_password = password;
	}

	protected PasswordAuthentication getPasswordAuthentication()
	{
		// Get information about the request
		String promptString = getRequestingPrompt();
		String hostname = getRequestingHost();
		InetAddress ipaddr = getRequestingSite();
		String ipAddressString = "";

		if(ipaddr != null)
			ipAddressString = ipaddr.toString();

		DVScheduler.log("HTTPAuthenticator : " + promptString + " : " + hostname + " : " + ipAddressString);

		if (getRequestorType() == Authenticator.RequestorType.PROXY)
		{
			DataStore store = DataStore.getInstance();
			String proxyUsername = store.getProperty("proxy.username", "");
			String proxyPassword = store.getProperty("proxy.password", "");
			if (proxyUsername.length() == 0 && proxyPassword.length() == 0)
			{
				DVScheduler.log("No Username/Password set for proxy server");
				return null;
			}
			else
				return new PasswordAuthentication(proxyUsername, proxyPassword.toCharArray());
		}
		else if (m_username.length() == 0 && m_password.length() == 0)
		{
			DVScheduler.log("No Username/Password set");
			return null;
		}
		else
		{
			return new PasswordAuthentication(m_username, m_password.toCharArray());
		}
	}
}
