/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.net.URLDecoder;
import java.util.*;

//import sun.misc.BASE64Decoder;

public class HTTPRequest
{
	private HashMap<String, String> m_headers = null;

	private HTTPurl m_urlData = null;

	private OutputStream m_outStream = null;

	private DataStore m_store = null;

	public HTTPRequest(String req, HashMap<String, String> head,
			String postData, OutputStream out) throws Exception
			{
		m_outStream = out;
		m_headers = head;
		m_store = DataStore.getInstance();
		m_urlData = new HTTPurl(req, postData, head);
			}

	public void sendErrorMessage(String text) throws Exception
	{
		DVScheduler.log(text);
		PageTemplate page = new PageTemplate(m_headers, m_urlData, "error.html");
		page.replace("$error", text);
		m_outStream.write(page.getPageBytes());
	}

	// /////////////////////////////////////////////////
	// This is the main function to call it handles
	// the request and returns the correct data
	// /////////////////////////////////////////////////
	public void sendResponseData(boolean bUnsecureDownload)
	{
		try
		{
			// if the timer thread is not running then panic!
			if (m_store.timerStatus == -1)
			{
				String timerError = "<html><head><title>Timer Thread Error</title></head><body>"
						+ "<h1>Timer Thread Error</h1>The main timer thread is not running, it has crashed with the following error:<p>"
						+ "<pre>StackTrace:\n"
						+ m_store.timerThreadErrorStack
						+ "</pre><p>"
						+ "</body>";
				m_outStream.write(timerError.getBytes("UTF-8"));
				return;
			}

			//
			// If needed check the username:password for authentication
			//
			if ( (m_urlData.getRequestType() == 4) && bUnsecureDownload )
			{
				Class<?> paramTypes[] =
					{Map.class , HTTPurl.class , OutputStream.class};
				java.lang.reflect.Constructor<?> c = Class.forName(
						m_urlData.getServletClass()).getConstructor(paramTypes);
				Object params[] = {m_headers, m_urlData, m_outStream};
				HTTPResponse resp = (HTTPResponse) c.newInstance(params);

				resp.getResponse2();
				return;
			}

			String auth = (String) m_headers.get("Authorization");
			boolean bUsePasswords = false;
			if ( (m_store.getUserCount() > 0 && !m_store.isTrustedIp( (String) m_headers.get("RemoteAddress"), m_urlData )) || 
					((m_store.getWebGuiProtocol() == DataStore.WEBGUI_HTTPS) && !m_store.getProperty("https.custom.rule", "0").equals("1")) )
				bUsePasswords = true;
			if (bUsePasswords
					&& (auth == null || auth.length() < 1 || !checkAuth(auth)))
			{
				StringBuffer out = new StringBuffer(4096);
				out.append("HTTP/1.0 401 Unauthorized\r\n");
				out.append("WWW-Authenticate: BASIC realm=\"user\"\r\n");
				out.append("Cache-Control: no-cache\r\n\r\n");

				m_outStream.write(out.toString().getBytes("UTF-8"));
				return;
			}

			//
			// We need to build the return data dynamicly :-)
			// This is done by using our dynamic class loader
			// it loads the class specified on the URL and
			// calls the getResponce method
			//
			else if (m_urlData.getRequestType() == 3)
			{
				Class<?> paramTypes[] =
					{Map.class , HTTPurl.class , OutputStream.class};
				java.lang.reflect.Constructor<?> c = Class.forName(
						m_urlData.getServletClass()).getConstructor(paramTypes);
				Object params[] = {m_headers, m_urlData, m_outStream};
				HTTPResponse resp = (HTTPResponse) c.newInstance(params);

				resp.getResponse();
				return;
			}

			//
			// Just return a file from the local system.
			//
			else if (m_urlData.getRequestType() == 2)
			{
				String urlText = URLDecoder.decode(m_urlData.getRequestString().replace("+", "%2B"), "UTF-8");

				/*
				// From v5.39 all "/docs/popup/" is not a complete html but only the content.
				// This can cause error with older themes.
				if (urlText.startsWith("/docs/popup/"))
				{
					PageTemplate template = new PageTemplate(m_headers, m_urlData, "infoHelp.html");

					template.replaceFile("$info", m_store.getHttpFile(m_headers.get("User-Agent"), urlText));

					m_outStream.write(template.getPageBytes());

					return;
				}
				 */

				returnFileContent(urlText);
				return;
			}
			else if (m_urlData.getRequestType() == 4)
			{
				Class<?> paramTypes[] =
					{Map.class , HTTPurl.class , OutputStream.class};
				java.lang.reflect.Constructor<?> c = Class.forName(
						m_urlData.getServletClass()).getConstructor(paramTypes);
				Object params[] = {m_headers, m_urlData, m_outStream};
				HTTPResponse resp = (HTTPResponse) c.newInstance(params);

				resp.getResponse2();
				return;
			}

			//
			// this the the main status index page
			//
			else if (m_urlData.getRequestType() == 1)
			{
				String action = m_urlData.getParameter("action");
				if ("01".equals(action))
				{
					m_outStream.write(new Integer(m_store.getNumCaptureTask()).toString().getBytes("UTF-8"));
					return;
				}
				else if ("02".equals(action))
				{
					m_store.m_nLastRecordingErrorCount = m_store.m_nRecordingErrorCount;

					m_outStream.write(HTTPResponse.getRedirUrlRedir(m_headers, m_urlData, null).getBytes("UTF-8"));

					return;
				}
				else if ("03".equals(action))
				{
					PageTemplate template = new PageTemplate(m_headers, m_urlData, "infoHelp.html");

					template.replaceFile("$info", m_store.getHttpFile(m_headers.get("User-Agent"), "/docs/popup/" + m_urlData.getParameter("name") + ".html"));

					m_outStream.write(template.getPageBytes());

					return;
				}
				else
				{
					SystemStatusData sd = new SystemStatusData();
					m_outStream.write(sd.getStatusXML(m_headers, m_urlData));
					return;
				}
			}

			PageTemplate page = new PageTemplate(m_headers, m_urlData, "error.html");
			page.replace("$error", "Request not known\n\n" + requestInfo());
			m_outStream.write(page.getPageBytes());
		}
		catch (Exception e)
		{
			ByteArrayOutputStream ba = new ByteArrayOutputStream();
			PrintWriter err = new PrintWriter(ba);

			try
			{
				e.printStackTrace(err);
				err.flush();

				PageTemplate page = new PageTemplate(m_headers, m_urlData, "error.html");
				page.replace("$error", HTMLEncoder.encode(m_urlData.toString())
						+ "\n\n" + HTMLEncoder.encode(ba.toString()));
				m_outStream.write(page.getPageBytes());
			}
			catch (Exception e2)
			{
				try
				{
					m_outStream.write(ba.toString().getBytes("UTF-8"));
				}
				catch (Exception e3)
				{
				}
			}

			DVScheduler.log("HTTP Request Exception: " + m_urlData);
			e.printStackTrace();
		}
	}

	// /////////////////////////////////////////////////
	// This returns the file 4kB at a time, this is for
	// returning large files from the system.
	// /////////////////////////////////////////////////
	private int returnFileContent(String fileName) throws Exception
	{
		/*
		 * String[] headKeys = (String[])m_headers.keySet().toArray(new String[0]);
		 * for(int x = 0; x < headKeys.length; x++) {
		 * DVScheduler.log(headKeys[x] + " : " +
		 * (String)m_headers.get(headKeys[x])); }
		 * 
		 * if(m_headers.get("If-Modified-Since") != null && false) { String header =
		 * "HTTP/1.0 304 Not Modified\n\n"; DVScheduler.log("HTTP/1.0 304 Not
		 * Modified"); m_outStream.write(header.getBytes("UTF-8")); outStream.flush();
		 * return 1; }
		 */

		/*
		if ( fileName.endsWith(".xsl") || fileName.endsWith(".xml") )
		{
			throw new Exception("File not allowed!");
		}
		 */

		String httpDir = m_store.getProperty("path.httproot", "http");
		File thisFile;
		if (fileName.startsWith("/themes/"))
		{
			thisFile = new File(httpDir + fileName);
			if (!thisFile.exists())
			{
				String fileName2 = fileName.substring(8);
				thisFile = new File(httpDir + fileName2.substring(fileName2.indexOf('/')));
			}
		}
		/*
		// Prior to v5.39 tooltip.js exists in "http/javascript" and "http/themes/default/javascript" and "http/themes/default-Horizontal/javascript".
		// Due to reorganization in v5.39 this can cause error in older themes (e.g. requires the root js but the theme js is sent instead due to same name).
		else if (fileName.equals("/javascript/tooltip.js"))
		{
			thisFile = new File(httpDir + fileName);
		}
		 */
		else
		{
			thisFile = new File(httpDir + "/themes/" + m_store.getThemeDirectory(m_headers.get("User-Agent")) + fileName);
			if (!thisFile.exists())
			{
				thisFile = new File(httpDir + fileName);
			}
		}

		String requestedFilePath = thisFile.getCanonicalPath();
		File root = new File(httpDir);
		String rootFilePath = root.getCanonicalPath();

		// check requested file in still within the root HTTP store
		if (requestedFilePath.indexOf(rootFilePath) < 0)
		{
			throw new Exception("File out of bounds!");
		}

		FileInputStream fi = new FileInputStream(thisFile);

		long fileLength = thisFile.length();

		int read = 0;
		byte[] bytes = new byte[4096];
		read = fi.read(bytes);

		try
		{
			String header = "";
			header += "HTTP/1.0 200 OK\n";
			if (fileName.toLowerCase().endsWith("html"))
			{
				String data = Formatter.encodeURL(m_urlData.getRequestString(), "UTF-8");
				header += "Set-Cookie: " + "backURL=" + data + "; PATH=/\n";
			}
			header += "Content-Length: " + fileLength + "\n";

			// Add a mine type to the return
			if (fileName.indexOf(".htc") > -1)
				header += "Content-Type: text/plain\n";
			else if (fileName.indexOf(".html") > -1)
				header += "Content-Type: text/html\n";
			/*
			 * //Last-Modified: Fri, 26 Mar 2004 14:03:23 GMT
			 * 
			 * long fileMod = thisFile.lastModified(); if(fileMod > 0) { Calendar
			 * stamp = Calendar.getInstance(); stamp.setTime(new Date(fileMod));
			 * String timeString = store.dayName.get(new
			 * Integer(stamp.get(Calendar.DAY_OF_WEEK))) + ", "; timeString +=
			 * stamp.get(Calendar.DATE) + " " + store.monthNameShort.get(new
			 * Integer(stamp.get(Calendar.MONTH)-1)) + " "; timeString +=
			 * stamp.get(Calendar.YEAR) + " " + stamp.get(Calendar.HOUR_OF_DAY) +
			 * ":" + stamp.get(Calendar.MINUTE) + ":"; timeString +=
			 * stamp.get(Calendar.SECOND) + " GMT";
			 * //DVScheduler.log(timeString); header += "Last-Modified: " +
			 * timeString + "\n"; }
			 */
			header += "\n";
			m_outStream.write(header.getBytes("UTF-8"));

			// Send the contents of the file to the output stream
			// DVScheduler.log("Starting to send file");
			while (read > -1)
			{
				// System.out.print(".");
				m_outStream.write(bytes, 0, read);
				read = fi.read(bytes);
			}
			// DVScheduler.log("Finished : " + fileName);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				fi.close();
			}
			catch (Exception e2)
			{
			}
		}

		return 1;
	}

	// /////////////////////////////////////////////////
	// use the auth header to check password and
	// username parameters
	// /////////////////////////////////////////////////
	private boolean checkAuth(String auth)
	{
		try
		{
			if (!(auth.startsWith("Basic ")))
				return false;

			String justAuth = auth.substring(6, auth.length());

			// BASE64Decoder decoder = new BASE64Decoder();
			// byte[] decodedStr = decoder.decodeBuffer(justAuth);
			// String decAuth = new String(decodedStr);
			String decAuth = Base64.decodeToString(justAuth);

			// DVScheduler.log(justAuth + " " + decAuth);

			if (m_store.checkUser( decAuth, m_urlData ))
				return true;

		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		return false;
	}

	// /////////////////////////////////////////////////
	// return the request URL and headers as a string
	// /////////////////////////////////////////////////
	private String requestInfo() throws Exception
	{
		StringBuffer out = new StringBuffer(1024);

		// Print out the request info
		out.append("Request String = (" + m_urlData.getRequestString() + ")\n");
		out.append("Request Type   = (" + m_urlData.getRequestType() + ")\n\n");

		out.append("Parameter List:\n");
		String[] names = m_urlData.getParameterList();
		for (int x = 0; x < names.length; x++)
			out.append(names[x] + " = " + m_urlData.getParameter(names[x]) + "\n");

		// Print header info
		out.append("\nRequest Headers:\n");
		String[] keys = (String[]) m_headers.keySet().toArray(new String[0]);
		for (int x = 0; x < keys.length; x++)
		{
			out.append(keys[x] + ": " + (String) m_headers.get(keys[x]) + "\n");
		}

		return out.toString();

	}
}
