import java.io.FileInputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.KeyStore;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.TrustManagerFactory;

class HTTPSThread implements Runnable
{
	private String m_keyStore;
	private char[] m_keyStorePassword;
	private char[] m_keyPassword;
	private int m_port;

	public HTTPSThread(int port, String keyStore, String keyStorePassword, String keyPassword)
	{
		DVScheduler.log("HTTPS Thread created");
		m_keyStore = keyStore;
		m_keyStorePassword = keyStorePassword.toCharArray();
		m_keyPassword = keyPassword.isEmpty() ? keyStorePassword.toCharArray() : keyPassword.toCharArray();
		m_port = port;
	}

	public void run()
	{
		try
		{
			KeyStore ks = KeyStore.getInstance("JKS");
			ks.load(new FileInputStream(m_keyStore), m_keyStorePassword);

			KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
			kmf.init(ks, m_keyPassword);

			TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
			tmf.init(ks);

			SSLContext ssl = SSLContext.getInstance("TLS");
			ssl.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);

			SSLServerSocketFactory ssf = ssl.getServerSocketFactory();

			ServerSocket listener = ssf.createServerSocket(m_port);

			Socket server;

			DVScheduler.log("Starting HTTPS server listening on (" + listener.getInetAddress().toString() + ":" + m_port + ")");

			while(true)
			{
				try
				{
					server = listener.accept();
					RequestObject con = new RequestObject(server);
					Thread conThread = new Thread(Thread.currentThread().getThreadGroup(), con, con.getClass().getName());
					conThread.start();
				}
				catch (Exception e1)
				{
					e1.printStackTrace();
				}
			}
		}
		catch (Exception e)
		{
			DVScheduler.log("HTTPS thread Exception: " + e);
			e.printStackTrace();
			System.exit(-1);
		}
	}
}