/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;
import java.text.*;
import java.net.*;

import org.w3c.dom.*;

import javax.xml.parsers.*;

class KBAutoAddRes extends HTTPResponse
{

	public KBAutoAddRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if ("01".equals(action))
		{
			m_outStream.write(showAutoEpgAddForm());
			return;
		}
		if ("02".equals(action))
		{
			m_outStream.write(showAutoEpgOptions());
			return;
		}
		if ("03".equals(action))
		{
			m_outStream.write(runAutoAddTest());
			return;
		}     
		if ("04".equals(action))
		{
			m_outStream.write(moveAutoAddItem());
			return;
		}         
		if ("05".equals(action))
		{
			m_outStream.write(remAutoAddItem());
			return;
		}    
		if ("06".equals(action))
		{
			m_outStream.write(addAutoEpgString());
			return;
		}        
		else if ("07".equals(action))
		{
			m_outStream.write(enableEpgMatchItem());
			return;
		}   
		else if ("08".equals(action))
		{
			m_outStream.write(showMatchListMenu());
			return;
		}      
		else if ("09".equals(action))
		{
			m_outStream.write(showAddMatchList());
			return;
		}  
		else if ("10".equals(action))
		{
			m_outStream.write(showDelMatchList());
			return;
		}     
		else if ("11".equals(action))
		{
			m_outStream.write(delMatchList());
			return;
		}   
		else if ("12".equals(action))
		{
			m_outStream.write(addMatchList());
			return;
		}     
		else if ("13".equals(action))
		{
			m_outStream.write(createAutoAddFromItem());
			return;
		}       
		else
		{
			m_outStream.write(getAutoAddTable());
			return;
		}
	}

	private byte[] createAutoAddFromItem() throws Exception
	{
		String backURL = m_urlData.getCookie("backURL");
		try {
			backURL = URLDecoder.decode(backURL, "UTF-8");
		} catch (Exception e) {
		}
		if (backURL == null || backURL.length() == 0)
			backURL = "/servlet/" + m_urlData.getServletClass();
		
		HashMap<String, EpgMatchList> matchLists = m_store.getEpgMatches();

		String itemID = m_urlData.getParameter("itemID");
		String wsChan = m_urlData.getParameter("chan");

		GuideStore guide = GuideStore.getInstance();
		String epgChan = guide.getEpgChannelIdFromMap(wsChan);

		GuideItem item = guide.getProgram(epgChan, itemID);

		if(item == null)
		{
			StringBuffer out = new StringBuffer(256);
			out.append("HTTP/1.0 302 Moved Temporarily\n");
			out.append("Location: " + backURL + "\n\n");
			return out.toString().getBytes();
		}

		String name = item.getName();
		int nextIndex = 0;
		boolean useInt = false;
		String matchListName = name + " (" + wsChan +")";

		if(matchLists.containsKey(matchListName))
		{
			useInt = true;
			while(matchLists.containsKey(matchListName + "_" + nextIndex))
			{
				nextIndex++;
			}
		}

		if(useInt)
			matchListName = matchListName + "_" + nextIndex;

		EpgMatchList newMatchList = new EpgMatchList();
		Vector<EpgMatchListItem> items = newMatchList.getMatchList();

		EpgMatchListItem newItemTitle = new EpgMatchListItem(EpgMatchListItem.TYPE_TEXT);
		newItemTitle.setTextSearchData(name, EpgMatchListItem.FIELD_TITLE, true, EpgMatchListItem.FLAG_NONE);
		items.add(newItemTitle);

		EpgMatchListItem newItemChan = new EpgMatchListItem(EpgMatchListItem.TYPE_TEXT);
		newItemChan.setTextSearchData(wsChan, EpgMatchListItem.FIELD_CHANNEL, true, EpgMatchListItem.FLAG_NONE);
		items.add(newItemChan);      

		matchLists.put(matchListName, newMatchList);

		// save out new MatchList
		m_store.saveEpgMatches(null);

		// create the new AA item
		EpgMatch epgMatch = new EpgMatch();
		epgMatch.setName( m_store.getEpgAutoAddUniqueName(name) );
		epgMatch.getMatchListNames().add(matchListName);

		int keepFor = 30;
		try
		{
			keepFor = Integer.parseInt(m_store.getProperty("AutoDel.KeepFor", "30"));
		}
		catch (Exception e){}       

		epgMatch.setKeepFor(keepFor);
		epgMatch.setAutoDelete(m_store.isAutoDelByDefault());

		int startBuff = 0;
		int endBuffer = 0;
		try
		{
			startBuff = Integer.parseInt(m_store.getProperty("Schedule.buffer.start", "0"));
			endBuffer = Integer.parseInt(m_store.getProperty("Schedule.buffer.end", "0"));
		}
		catch (Exception e){}       
		epgMatch.setStartBuffer(startBuff);
		epgMatch.setEndBuffer(endBuffer);

		epgMatch.setPostTask(m_store.getProperty("Tasks.DefTask", ""));

		String[] namePatterns = m_store.getNamePatterns();
		if(namePatterns.length > 0)
		{
			epgMatch.setFileNamePattern(namePatterns[0]);
		}
		else
		{
			epgMatch.setFileNamePattern("(%y-%m-%d %h-%M) %n [%c]\n%n [%c]\n%n\n%n\\%y-%m-%d");
		}      

		String capType = "Capture.deftype";
		/*
      String capType = "";
      try
      {
         capType = store.getProperty("Capture.deftype", "");
      }
      catch (Exception e){}
		 */

		epgMatch.setCaptureType(capType); 

		// save new auto-add
		m_store.addEpgMatch(epgMatch, 0, 1);

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: " + backURL + "\n\n");
		return out.toString().getBytes();        
	}

	private byte[] addMatchList() throws Exception
	{
		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch(Exception e){}

		EpgMatch item = (EpgMatch)m_store.getEpgAutoAdds().get(index);
		if(item == null)
		{
			String out = "HTTP/1.0 302 Moved Temporarily\nLocation: " +
			"/servlet/" + m_urlData.getServletClass() + "\n\n";

			return out.getBytes();
		}

		String name = m_urlData.getParameter("name");

		if(name != null)
		{
			if(!item.getMatchListNames().contains(name))
			{
				item.getMatchListNames().add(name);
				m_store.saveEpgMatches(null);
			}
		}

		StringBuffer buff = new StringBuffer(256);

		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return buff.toString().getBytes();      
	}   

	private byte[] delMatchList() throws Exception
	{
		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch(Exception e){}

		EpgMatch item = (EpgMatch)m_store.getEpgAutoAdds().get(index);
		if(item == null)
		{
			String out = "HTTP/1.0 302 Moved Temporarily\nLocation: " +
			"/servlet/" + m_urlData.getServletClass() + "\n\n";

			return out.getBytes();
		}

		String name = m_urlData.getParameter("name");

		if(name != null)
		{
			item.getMatchListNames().remove(name);
			m_store.saveEpgMatches(null);
		}

		StringBuffer buff = new StringBuffer(256);

		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return buff.toString().getBytes();      
	}

	private byte[] showDelMatchList() throws Exception
	{
		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch(Exception e){}

		EpgMatch item = (EpgMatch)m_store.getEpgAutoAdds().get(index);
		if(item == null)
		{
			String out = "HTTP/1.0 302 Moved Temporarily\nLocation: " +
			"/servlet/" + m_urlData.getServletClass() + "\n\n";

			return out.getBytes();
		}

		String start = m_urlData.getParameter("start");
		if(start == null || start.length() == 0)
			start = "0";

		String show = m_urlData.getParameter("show");
		if(show == null || show.length() == 0)
			show = "10";

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "buttons", null); 

		Element root = doc.getDocumentElement();      

		root.setAttribute("start" , start);
		root.setAttribute("show" , show);       

		root.setAttribute("back" , "/servlet/" + m_urlData.getServletClass() + 
				"?action=08&index=" + index);      

		root.setAttribute("title" , "Select a Match List to Delete it");      

		Element button = null;
		Element elm = null;
		Text text = null;      

		button = doc.createElement("mainurl");
		text = doc.createTextNode("/servlet/" + m_urlData.getServletClass() + "?action=10&index=" + index + "&");
		button.appendChild(text);
		root.appendChild(button);        

		String[] keys = (String[])item.getMatchListNames().toArray(new String[0]);
		Arrays.sort(keys, String.CASE_INSENSITIVE_ORDER);

		for(int x = 0; x < keys.length; x++)
		{
			String action = "/servlet/KBAutoAddRes?action=11&index=" + index + "&name=" +
			Formatter.encodeURL(keys[x], "UTF-8");    

			button = doc.createElement("button");
			button.setAttribute("name", keys[x]);
			elm = doc.createElement("url");
			text = doc.createTextNode(action);
			elm.appendChild(text);
			button.appendChild(elm);
			elm = doc.createElement("confirm");
			text = doc.createTextNode("true");
			elm.appendChild(text);   
			button.appendChild(elm);
			root.appendChild(button);         
		}

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "kb-list.xsl", m_headers, m_urlData);
		return transformer.transform();         
	}   

	private byte[] showAddMatchList() throws Exception
	{
		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch(Exception e){}

		EpgMatch item = (EpgMatch)m_store.getEpgAutoAdds().get(index);
		if(item == null)
		{
			String out = "HTTP/1.0 302 Moved Temporarily\nLocation: " +
			"/servlet/" + m_urlData.getServletClass() + "\n\n";

			return out.getBytes();
		}

		String start = m_urlData.getParameter("start");
		if(start == null || start.length() == 0)
			start = "0";

		String show = m_urlData.getParameter("show");
		if(show == null || show.length() == 0)
			show = "10";

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "buttons", null); 

		Element root = doc.getDocumentElement();      

		root.setAttribute("start" , start);
		root.setAttribute("show" , show);       

		root.setAttribute("back" , "/servlet/" + m_urlData.getServletClass() + 
				"?action=08&index=" + index);      

		root.setAttribute("title" , "Select a Match List to Add it");      

		Element button = null;
		Element elm = null;
		Text text = null;      

		button = doc.createElement("mainurl");
		text = doc.createTextNode("/servlet/" + m_urlData.getServletClass() + "?action=09&index=" + index + "&");
		button.appendChild(text);
		root.appendChild(button);      

		HashMap<String, EpgMatchList> matches = m_store.getEpgMatches();
		String[] keys = (String[])matches.keySet().toArray(new String[0]);
		Arrays.sort(keys, String.CASE_INSENSITIVE_ORDER);

		for(int x = 0; x < keys.length; x++)
		{
			String action = "/servlet/KBAutoAddRes?action=12&index=" + index + "&name=" +
			Formatter.encodeURL(keys[x], "UTF-8");    

			button = doc.createElement("button");
			button.setAttribute("name", keys[x]);
			elm = doc.createElement("url");
			text = doc.createTextNode(action);
			elm.appendChild(text);
			button.appendChild(elm);
			elm = doc.createElement("confirm");
			text = doc.createTextNode("true");
			elm.appendChild(text);   
			button.appendChild(elm);
			root.appendChild(button);         
		}

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "kb-list.xsl", m_headers, m_urlData);
		return transformer.transform();         
	}

	private byte[] showMatchListMenu() throws Exception
	{
		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch(Exception e){}

		EpgMatch item = (EpgMatch)m_store.getEpgAutoAdds().get(index);
		if(item == null)
		{
			String out = "HTTP/1.0 302 Moved Temporarily\nLocation: " +
			"/servlet/" + m_urlData.getServletClass() + "\n\n";

			return out.getBytes();
		}

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "buttons", null); 

		Element root = doc.getDocumentElement();

		root.setAttribute("back" , "/servlet/" + m_urlData.getServletClass() + 
				"?action=02&index=" + index);

		root.setAttribute("title" , "Auto-Add Match List Menu");

		Element button = null;
		Element elm = null;
		Text text = null;
		String actionURL = "";

		button = doc.createElement("button");
		button.setAttribute("name" , "Back");
		elm = doc.createElement("url");
		actionURL = "/servlet/" + m_urlData.getServletClass() + "?action=02&index=" + index;
		text = doc.createTextNode(actionURL);
		elm.appendChild(text);
		button.appendChild(elm);  
		root.appendChild(button);      

		if(item.getMatchListNames().size() > 0)
		{
			button = doc.createElement("button");
			button.setAttribute("name", "Show Current");
			elm = doc.createElement("url");
			actionURL = "/servlet/" + m_urlData.getServletClass() + "?action=10&index=" + index;
			text = doc.createTextNode(actionURL);
			elm.appendChild(text);
			button.appendChild(elm);
			root.appendChild(button);
		}

		if(m_store.getEpgMatches().size() > 0)
		{
			button = doc.createElement("button");
			button.setAttribute("name", "Add");
			elm = doc.createElement("url");
			actionURL = "/servlet/" + m_urlData.getServletClass() + "?action=09&index=" + index;
			text = doc.createTextNode(actionURL);
			elm.appendChild(text);
			button.appendChild(elm);
			root.appendChild(button);
		}      

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "kb-buttons.xsl", m_headers, m_urlData);
		return transformer.transform();      
	}

	private byte[] enableEpgMatchItem() throws Exception
	{
		int indexOf = Integer.parseInt(m_urlData.getParameter("index"));

		Vector<?> list = m_store.getEpgAutoAdds();
		EpgMatch epgMatcher = (EpgMatch) list.get(indexOf);

		if(epgMatcher != null)
		{
			if("true".equals(m_urlData.getParameter("enabled")))
				epgMatcher.setEnabled(true);
			else
				epgMatcher.setEnabled(false);

			m_store.saveEpgAutoList(null);
		}

		String out = "HTTP/1.0 302 Moved Temporarily\nLocation: " +
		"/servlet/" + m_urlData.getServletClass() + "\n\n";

		return out.getBytes(); 
	}

	private byte[] addAutoEpgString() throws Exception
	{
		String ad = m_urlData.getParameter("autoDel");
		boolean autoDel = false;
		if (ad != null && ad.equalsIgnoreCase("true"))
			autoDel = true;

		int keepFor = 30;
		try
		{
			keepFor = Integer.parseInt(m_urlData.getParameter("keepFor"));
		}
		catch (Exception e)
		{
		}

		int startBuffer = 0;
		int endBuffer = 0;
		try
		{
			startBuffer = Integer.parseInt(m_urlData.getParameter("startbuffer"));
			endBuffer = Integer.parseInt(m_urlData.getParameter("endbuffer"));
		}
		catch (Exception e)
		{
		}

		String postTask = m_urlData.getParameter("task");
		if (postTask == null || postTask.equalsIgnoreCase("none"))
			postTask = "";

		String namePattern = m_urlData.getParameter("filenamePatterns");
		if(namePattern == null || namePattern.length() == 0)
			namePattern = "(%y-%m-%d %h-%M) %n [%c]\n%n [%c]\n%n\n%n\\%y-%m-%d";

		String capType = m_urlData.getParameter("captype");
		if ( (capType == null) || capType.isEmpty() )
			capType = "Capture.deftype";

		EpgMatch epgMatch = null;

		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch (Exception e)
		{
		}

		if (index > -1)
		{
			epgMatch = (EpgMatch)m_store.getEpgAutoAdds().get(index);
		}
		else
		{
			epgMatch = new EpgMatch();
			m_store.addEpgMatch(epgMatch, 0);
		}

		epgMatch.setKeepFor(keepFor);
		epgMatch.setAutoDelete(autoDel);

		epgMatch.setStartBuffer(startBuffer);
		epgMatch.setEndBuffer(endBuffer);
		epgMatch.setPostTask(postTask);
		epgMatch.setFileNamePattern(namePattern);
		epgMatch.setCaptureType(capType);

		m_store.saveEpgAutoList(null);

		StringBuffer buff = new StringBuffer(256);
		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");
		return buff.toString().getBytes();
	}

	private byte[] moveAutoAddItem() throws Exception
	{    
		int id = -1;
		try
		{
			id = Integer.parseInt(m_urlData.getParameter("id"));
		}
		catch(Exception e){}

		int dir = 1;
		try
		{
			dir = Integer.parseInt(m_urlData.getParameter("dir"));
		}
		catch(Exception e){}

		boolean direction = false;
		if(dir == 1)
			direction = true;

		if(id != -1)
			m_store.moveEpgItem(id, direction);

		StringBuffer buff = new StringBuffer();
		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /servlet/KBAutoAddRes\n\n");

		return buff.toString().getBytes();
	}   

	private byte[] runAutoAddTest() throws Exception
	{
		int index = Integer.parseInt(m_urlData.getParameter("index"));
		EpgMatch epgMatcher = (EpgMatch)m_store.getEpgAutoAdds().get(index);

		HashMap<String, Vector<GuideItem>> results = new HashMap<String, Vector<GuideItem>>();

		Vector<?> matchNames = epgMatcher.getMatchListNames();
		HashMap<?, ?> matchLists = m_store.getEpgMatches();

		GuideStore guide = GuideStore.getInstance();

		EpgMatchList matcher = null;

		for(int nameIndex = 0; nameIndex < matchNames.size(); nameIndex++)
		{
			String matchListName = (String)matchNames.get(nameIndex);
			matcher = (EpgMatchList)matchLists.get(matchListName);
			if(matcher != null)
			{
				guide.searchEPG(matcher, results);
			}
		}      

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "buttons", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("id" , "");
		root.setAttribute("url" , "/servlet/KBAutoAddRes?action=02&index=" + index);
		root.setAttribute("filter" , ""); 

		Element logitem = null;
		Element elm = null;
		Text text = null;   	

		Vector<?> channelMap = guide.getOldChannelMap();
		Set<?> wsChannels = m_store.getChannels().keySet();

		SimpleDateFormat df = new SimpleDateFormat("EEE MMM d h:mm aa");

		int count = 0;

		for(int y = 0; y < channelMap.size(); y++)
		{
			String[] map = (String[])channelMap.get(y);
			Vector<?> result = (Vector<?>)results.get(map[0]);

			if(result.size() > 0 && wsChannels.contains(map[0]))
			{
				logitem = doc.createElement("logitem");
				logitem.setAttribute("type", "1");
				elm = doc.createElement("line");
				text = doc.createTextNode(map[0]);
				elm.appendChild(text);
				logitem.appendChild(elm);
				root.appendChild(logitem);  

				for(int x = 0; x < result.size(); x++)
				{
					GuideItem item = (GuideItem)result.get(x);

					logitem = doc.createElement("logitem");
					logitem.setAttribute("type", "0");
					elm = doc.createElement("line");

					String matchText = item.getName();
					matchText += " (" + df.format(item.getStart(), new StringBuffer(), new FieldPosition(0)).toString() + ")";

					text = doc.createTextNode(matchText);
					elm.appendChild(text);
					logitem.appendChild(elm);
					root.appendChild(logitem);  

					count++;
				}
			}
		}

		if(count == 0)
		{
			logitem = doc.createElement("logitem");
			logitem.setAttribute("type", "0");
			elm = doc.createElement("line");
			text = doc.createTextNode("No Hits");
			elm.appendChild(text);
			logitem.appendChild(elm);
			root.appendChild(logitem);           
		}

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "kb-searchtest.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] showAutoEpgOptions() throws Exception
	{
		int index = -1;
		try
		{
			index = Integer.parseInt(m_urlData.getParameter("index"));
		}
		catch(Exception e){}

		EpgMatch epgMatcher = (EpgMatch)(m_store.getEpgAutoAdds().get(index));

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "buttons", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("back", "/servlet/KBAutoAddRes");

		Element button = null;
		Element elm = null;
		Text text = null;

		// back button

		button = doc.createElement("button");
		button.setAttribute("name", "Back");
		elm = doc.createElement("url");
		text = doc.createTextNode("/servlet/KBAutoAddRes");
		elm.appendChild(text);
		button.appendChild(elm);
		root.appendChild(button);        

		//enable disable
		button = doc.createElement("button");
		String action = "/servlet/KBAutoAddRes?action=07&index=" + index;

		if(epgMatcher.isEnabled())
		{
			button.setAttribute("name", "Disable");
			action += "&enabled=false";  
		}
		else
		{
			button.setAttribute("name", "Enable");
			action += "&enabled=true";
		}

		elm = doc.createElement("url");
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);
		elm = doc.createElement("confirm");
		text = doc.createTextNode("false");
		elm.appendChild(text);   
		button.appendChild(elm);
		root.appendChild(button);


		// Match List Button

		action = "/servlet/" + m_urlData.getServletClass() + "?action=08&index=" + index;

		button = doc.createElement("button");
		button.setAttribute("name", "Match Lists");
		elm = doc.createElement("url");
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);
		root.appendChild(button);

		// Edit button

		action = "/servlet/" + m_urlData.getServletClass() + "?action=01&index=" + index;

		button = doc.createElement("button");
		button.setAttribute("name", "Edit Options");
		elm = doc.createElement("url");
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);
		root.appendChild(button);

		// show matches

		action = "/servlet/" + m_urlData.getServletClass() + "?action=03&index=" + index;

		button = doc.createElement("button");
		button.setAttribute("name", "Show Matches");
		elm = doc.createElement("url");
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);
		root.appendChild(button);      

		//move up
		action = "/servlet/" + m_urlData.getServletClass() + "?action=04&id=" +
		index + "&dir=0"; 

		button = doc.createElement("button");
		button.setAttribute("name", "Move Up");
		elm = doc.createElement("url");
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);
		elm = doc.createElement("confirm");
		text = doc.createTextNode("false");
		elm.appendChild(text);   
		button.appendChild(elm);
		root.appendChild(button);      

		//move down
		action = "/servlet/" + m_urlData.getServletClass() + "?action=04&id=" +
		index + "&dir=1"; 

		button = doc.createElement("button");
		button.setAttribute("name", "Move Down");
		elm = doc.createElement("url");
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);
		elm = doc.createElement("confirm");
		text = doc.createTextNode("false");
		elm.appendChild(text);   
		button.appendChild(elm);
		root.appendChild(button);       

		//delete
		action = "/servlet/KBAutoAddRes?action=05&id=" + index;    

		button = doc.createElement("button");
		button.setAttribute("name", "Delete");
		elm = doc.createElement("url");
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);
		elm = doc.createElement("confirm");
		text = doc.createTextNode("true");
		elm.appendChild(text);   
		button.appendChild(elm);
		root.appendChild(button);

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "kb-buttons.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] remAutoAddItem() throws Exception
	{
		String idString = m_urlData.getParameter("id");

		int id = -1;

		if (idString != null)
		{
			try
			{
				id = Integer.parseInt(idString);
			}
			catch (Exception e)
			{
			}
		}

		String all = m_urlData.getParameter("all");

		if (id > -1)
		{
			EpgMatch item = (EpgMatch)m_store.getEpgAutoAdds().get(id);

			if(item != null)
			{
				String[] unUsed = noSharedMatchLists(item);

				if(unUsed.length == 0 || (unUsed.length > 0 && "0".equalsIgnoreCase(all)))
				{
					m_store.removeEpgMatch(id);
				}
				else if(unUsed.length > 0 && "1".equalsIgnoreCase(all))
				{
					m_store.removeEpgMatch(id);

					for(int x = 0; x < unUsed.length; x++)
					{
						m_store.getEpgMatches().remove(unUsed[x]);
					}
					m_store.saveEpgMatches(null);               
				}  
				else
				{

					// Creation of an XML document
					DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
					DocumentBuilder db = dbf.newDocumentBuilder();
					DOMImplementation di = db.getDOMImplementation();
					Document doc = di.createDocument("", "buttons", null); 

					Element root = doc.getDocumentElement();               

					root.setAttribute("back" , "/servlet/" + m_urlData.getServletClass());

					root.setAttribute("title" , "Delete any unused Match Lists as Well?");

					Element button = null;
					Element elm = null;
					Text text = null;
					String actionURL = "";               

					button = doc.createElement("button");
					button.setAttribute("name" , "Yes");
					elm = doc.createElement("url");
					actionURL = "/servlet/" + m_urlData.getServletClass() + "?action=05&id=" + id + "&all=1";
					text = doc.createTextNode(actionURL);
					elm.appendChild(text);
					button.appendChild(elm);  
					root.appendChild(button);   

					button = doc.createElement("button");
					button.setAttribute("name" , "No");
					elm = doc.createElement("url");
					actionURL = "/servlet/" + m_urlData.getServletClass() + "?action=05&id=" + id + "&all=0";
					text = doc.createTextNode(actionURL);
					elm.appendChild(text);
					button.appendChild(elm);  
					root.appendChild(button);                

					//
					// Do transform and return data
					//
					XSL transformer = new XSL(doc, "kb-buttons.xsl", m_headers, m_urlData);
					return transformer.transform();                 

				}
			}
		}

		StringBuffer buff = new StringBuffer(256);
		buff.append("HTTP/1.0 302 Moved Temporarily\n");
		buff.append("Location: /servlet/KBAutoAddRes\n\n");
		return buff.toString().getBytes();
	}   

	private String[] noSharedMatchLists(EpgMatch item)
	{
		Vector<String> unUsed = new Vector<String>();
		EpgMatch[] items = (EpgMatch[])m_store.getEpgAutoAdds().toArray(new EpgMatch[0]);

		for(int x = 0; x < item.getMatchListNames().size(); x++)
		{
			boolean used = false;
			String name = (String)item.getMatchListNames().get(x);

			for(int y = 0; y < items.length; y++)
			{
				if(item != items[y] && items[y].getMatchListNames().contains(name))
					used = true;
			}

			if(used == false)
				unUsed.add(name);
		}

		return (String[])unUsed.toArray(new String[0]);
	}

	private byte[] showAutoEpgAddForm() throws Exception
	{
		Vector<?> list = m_store.getEpgAutoAdds();
		EpgMatch epgMatcher = null;

		String index = m_urlData.getParameter("index");
		if (index == null)
			index = "";
		int indexOf = -1;

		try
		{
			indexOf = Integer.parseInt(index);
		}
		catch (Exception e)
		{
		}

		if (indexOf > -1 && indexOf < list.size())
		{
			epgMatcher = (EpgMatch) list.get(indexOf);
		}

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "item_form", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("index", new Integer(indexOf).toString());
		root.setAttribute("delete", "No");

		//
		// form elements
		//

		// Title

		Element formEl = null;

		//Start Buffer Minutes
		formEl = doc.createElement("startBuffer");
		formEl.setAttribute("Name", "Start");
		formEl.setAttribute("max", "59");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "1");
		if (indexOf > -1 && indexOf < list.size())
			formEl.setAttribute("value", new Integer(epgMatcher.getStartBuffer()).toString());
		else
			formEl.setAttribute("value", "5");
		root.appendChild(formEl);

		//End Buffer Minutes
		formEl = doc.createElement("endBuffer");
		formEl.setAttribute("Name", "End");
		formEl.setAttribute("max", "400");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "5");
		if (indexOf > -1 && indexOf < list.size())
			formEl.setAttribute("value", new Integer(epgMatcher.getEndBuffer()).toString());
		else
			formEl.setAttribute("value", "10");
		root.appendChild(formEl);

		//Referer
		formEl = doc.createElement("referer");
		Text text = doc.createTextNode("/servlet/KBAutoAddRes");
		formEl.appendChild(text);
		root.appendChild(formEl);

		//Capture Type
		formEl = doc.createElement("captureType");
		/*
		formEl.setAttribute("Name", "Capture Type");
		formEl.setAttribute("max", "10");
		formEl.setAttribute("min", "0");
		formEl.setAttribute("amount", "1");
		 */
		getCapturetypeList(doc, formEl);
		if (indexOf > -1 && indexOf < list.size())
			formEl.setAttribute("value", epgMatcher.getCaptureType());
		else
			//formEl.setAttribute("value", store.getProperty("Capture.deftype", ""));
			formEl.setAttribute("value", "Capture.deftype");
		root.appendChild(formEl);

		//Auto Delete
		formEl = doc.createElement("autoDel");
		formEl.setAttribute("Name", "Auto Delete");
		if (indexOf > -1 && indexOf < list.size())
			if (epgMatcher.isAutoDelete())
				formEl.setAttribute("value", "True");
			else
				formEl.setAttribute("value", "False");
		else
			formEl.setAttribute("value", "False");
		root.appendChild(formEl);

		//Keep For
		formEl = doc.createElement("keepfor");
		formEl.setAttribute("Name", "keep For");
		formEl.setAttribute("max", "120");
		formEl.setAttribute("min", "1");
		formEl.setAttribute("amount", "1");
		if (indexOf > -1 && indexOf < list.size())
			formEl.setAttribute("value", new Integer(epgMatcher.getKeepFor()).toString());
		else
		{
			String keep = m_store.getProperty("AutoDel.KeepFor", "30");
			formEl.setAttribute("value", keep);
		}
		root.appendChild(formEl);

		//Post Task
		formEl = doc.createElement("posttask");
		getTaskList(doc, formEl);
		if (indexOf > -1 && indexOf < list.size())
			formEl.setAttribute("value", epgMatcher.getPostTask());
		else
			formEl.setAttribute("value", "");
		root.appendChild(formEl);

		//Filename Patterns
		formEl = doc.createElement("filenamePatterns");
		getNamePatterns(doc, formEl);
		if (indexOf > -1 && indexOf < list.size())
			formEl.setAttribute("value", epgMatcher.GetFileNamePattern());
		else
			formEl.setAttribute("value", "");
		root.appendChild(formEl);      

		//
		// Do transform and return data
		//
		XSL transformer = new XSL(doc, "kb-aa-details.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] getAutoAddTable() throws Exception
	{
		Vector<?> list = m_store.getEpgAutoAdds();
		EpgMatch epgMatcher = null;

		String start = m_urlData.getParameter("start");
		if(start == null)
			start = "0";

		String show = m_urlData.getParameter("show");
		if(show == null)
			show = "10";

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "schedules", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("start", start);
		root.setAttribute("show", show);
		root.setAttribute("total", new Integer(list.size()).toString());

		Element aa = null;
		Element elm = null;
		Text text = null;

		aa = doc.createElement("mainurl");
		text = doc.createTextNode("/servlet/KBAutoAddRes?");
		aa.appendChild(text);
		root.appendChild(aa);         

		for (int x = 0; x < list.size(); x++)
		{
			epgMatcher = (EpgMatch) list.get(x);

			aa = doc.createElement("aaitem");
			aa.setAttribute("index", new Integer(x).toString());

			Vector<?> items = epgMatcher.getMatchListNames();
			String names = "";
			for(int q = 0; q < items.size(); q++)
			{
				String name = (String)items.get(q);
				if(q == items.size()-1)
				{
					names += name;
				}
				else
					names += name + ", ";
			}         

			if(names.length() == 0)
				names = "No Match List Associated";

			elm = doc.createElement("title");
			text = doc.createTextNode(names);
			elm.appendChild(text);
			aa.appendChild(elm);

			// Is Enabled
			elm = doc.createElement("enabled");
			text = doc.createTextNode(new Boolean(epgMatcher.isEnabled()).toString());
			elm.appendChild(text);
			aa.appendChild(elm);

			//StartBuffer
			elm = doc.createElement("startbuffer");
			text = doc.createTextNode(new Integer(epgMatcher.getStartBuffer()).toString());
			elm.appendChild(text);
			aa.appendChild(elm);

			//EndBuffer
			elm = doc.createElement("endbuffer");
			text = doc.createTextNode(new Integer(epgMatcher.getEndBuffer()).toString());
			elm.appendChild(text);
			aa.appendChild(elm);

			//Capture Type
			elm = doc.createElement("capturetype");
			text = doc.createTextNode(epgMatcher.getCaptureType());
			elm.appendChild(text);
			aa.appendChild(elm);

			String action = "/servlet/" + m_urlData.getServletClass() + "?action=02&index=" + Formatter.encodeURL(new Integer(x).toString(), "UTF-8");

			elm = doc.createElement("action");
			text = doc.createTextNode(action);
			elm.appendChild(text);
			aa.appendChild(elm);

			root.appendChild(aa);
		}

		// Do transform and return data
		XSL transformer = new XSL(doc, "kb-aa-list.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private void getCapturetypeList(Document doc, Element formEl)
	{
		Element option = null;
		Text text = null;

		Vector<String> capTypes = m_store.getCaptureTypes();

		for (int x = 0; x < capTypes.size(); x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(capTypes.get(x));
			option.appendChild(text);
			formEl.appendChild(option);
		}
	}

	private String getNamePatterns(Document doc, Element formEl)
	{
		Element option = null;
		Text text = null;

		StringBuffer buff = new StringBuffer(1024);
		String[] namePatterns = m_store.getNamePatterns();

		for (int x = 0; x < namePatterns.length; x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(namePatterns[x]);
			option.appendChild(text);
			formEl.appendChild(option);
		}

		return buff.toString();
	}   

	private String getTaskList(Document doc, Element formEl)
	{
		Element option = null;
		Text text = null;

		StringBuffer buff = new StringBuffer(1024);
		HashMap<String, TaskCommand> tasks = m_store.getTaskList();

		String[] keys = (String[]) tasks.keySet().toArray(new String[0]);

		Arrays.sort(keys);

		option = doc.createElement("option");
		text = doc.createTextNode("none");
		option.appendChild(text);
		formEl.appendChild(option);

		for (int x = 0; x < keys.length; x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(keys[x]);
			option.appendChild(text);
			formEl.appendChild(option);
		}

		return buff.toString();
	}
}
