/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;
import java.text.*;

import org.w3c.dom.*;

import javax.xml.parsers.*;

class KBEpgDataRes extends HTTPResponse
{

	public KBEpgDataRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if ("01".equals(action))
		{
			m_outStream.write(showEPG());
			return;
		}
		else if ("03".equals(action))
		{
			m_outStream.write(reloadXMLTVdata());
			return;
		}
		else if ("04".equals(action))
		{
			m_outStream.write(rescanXMLTVdata());
			return;
		}
		else if ("05".equals(action))
		{
			m_outStream.write(showProgInfo());
			return;
		}      
		else
		{
			m_outStream.write(showEpgIndex());
			return;
		}
	}

	private byte[] showProgInfo() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();

		String id = m_urlData.getParameter("id");
		String wsChan = m_urlData.getParameter("channel");
		String epgChan = guide.getEpgChannelIdFromMap(wsChan);
		GuideItem item = guide.getProgram(epgChan, id);

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "proginfo", null);
		Element root = doc.getDocumentElement();
		root.setAttribute("id", id);

		Calendar cal = Calendar.getInstance();
		cal.setTime(item.getStart());      

		root.setAttribute("hour", new Integer(cal.get(Calendar.HOUR_OF_DAY)).toString());
		root.setAttribute("day", new Integer(cal.get(Calendar.DATE)).toString());
		root.setAttribute("month", new Integer(cal.get(Calendar.MONTH)+1).toString());
		root.setAttribute("year", new Integer(cal.get(Calendar.YEAR)).toString());

		Element itemEl = null;
		Text text = null;   

		itemEl = doc.createElement("title");
		text = doc.createTextNode(item.getName());
		itemEl.appendChild(text);
		root.appendChild(itemEl);   

		itemEl = doc.createElement("sub-title");
		text = doc.createTextNode(item.getSubName());
		itemEl.appendChild(text);
		root.appendChild(itemEl);      

		SimpleDateFormat df = new SimpleDateFormat("EEE MMM d h:mm aa");

		itemEl = doc.createElement("start");
		text = doc.createTextNode(df.format(item.getStart()));
		itemEl.appendChild(text);
		root.appendChild(itemEl);

		itemEl = doc.createElement("stop");
		text = doc.createTextNode(df.format(item.getStop()));
		itemEl.appendChild(text);
		root.appendChild(itemEl);      

		itemEl = doc.createElement("duration");
		text = doc.createTextNode(new Integer(item.getDuration()).toString());
		itemEl.appendChild(text);
		root.appendChild(itemEl);        

		itemEl = doc.createElement("channel");
		text = doc.createTextNode(wsChan);
		itemEl.appendChild(text);
		root.appendChild(itemEl);  

		Vector<String> categories = item.getCategories();
		if (categories != null)
		{
			for (int i=0; i<categories.size(); i++)
			{
				itemEl = doc.createElement("category");
				text = doc.createTextNode(categories.get(i));
				itemEl.appendChild(text);
				root.appendChild(itemEl);        
			}
		}

		itemEl = doc.createElement("description");
		text = doc.createTextNode(item.getDescription());
		itemEl.appendChild(text);
		root.appendChild(itemEl); 

		// Add URl Creation
		String addLink = "/servlet/KBScheduleDataRes?action=11" +
				"&channel=" + Formatter.encodeURL(wsChan, "UTF-8") + "&id=" + item.toString();

		Element addUrlElement = doc.createElement("addURL");
		Text addUrlText = doc.createTextNode(addLink);
		addUrlElement.appendChild(addUrlText);
		root.appendChild(addUrlElement);

		String autoAddString = "/servlet/KBAutoAddRes?action=13&itemID=" + id + "&chan=" + 
				Formatter.encodeURL(wsChan, "UTF-8");

		Element autoAddUrlElement = doc.createElement("autoAddURL");
		Text autoAddUrlText = doc.createTextNode(autoAddString);
		autoAddUrlElement.appendChild(autoAddUrlText);
		root.appendChild(autoAddUrlElement);      

		itemEl = doc.createElement("url-source");
		text = doc.createTextNode(item.getURL());
		itemEl.appendChild(text);
		root.appendChild(itemEl);   

		// Do transform and return data
		XSL transformer = new XSL(doc, "kb-showproginfo.xsl", m_headers, m_urlData);
		return transformer.transform();      
	}

	private byte[] rescanXMLTVdata() throws Exception
	{
		StringBuffer buff1 = new StringBuffer();

		GuideStore guide = GuideStore.getInstance();

		m_store.removeSchedulesOfTypeEpg(buff1, 0);
		guide.addEpgMatches(buff1, 0, false);
		guide.saveEpg();
		m_store.saveSchedules(null);

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "log", null);
		Element root = doc.getDocumentElement();
		root.setAttribute("id", "");

		Element logitem = null;
		Element elm = null;
		Text text = null;

		String[] lines = buff1.toString().split("\n");
		for (int x = 0; x < lines.length; x++)
		{
			logitem = doc.createElement("logitem");
			elm = doc.createElement("line");
			text = doc.createTextNode(lines[x]);
			elm.appendChild(text);
			logitem.appendChild(elm);
			root.appendChild(logitem);
		}

		//		Do transform and return data
		XSL transformer = new XSL(doc, "kb-redo.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] showEPG() throws Exception
	{
		Calendar now = Calendar.getInstance();
		now.set(Calendar.SECOND, 0);
		now.set(Calendar.MILLISECOND, 0);

		int year = -1;
		try
		{
			year = Integer.parseInt(m_urlData.getParameter("year"));
		}
		catch (Exception e)
		{
		}
		if (year == -1)
			year = now.get(Calendar.YEAR);

		int month = -1;
		try
		{
			month = Integer.parseInt(m_urlData.getParameter("month"));
		}
		catch (Exception e)
		{
		}
		if (month == -1)
			month = now.get(Calendar.MONTH) + 1;

		int day = -1;
		try
		{
			day = Integer.parseInt(m_urlData.getParameter("day"));
		}
		catch (Exception e)
		{
		}
		if (day == -1)
			day = now.get(Calendar.DATE);

		int startHour = -1;
		try
		{
			startHour = Integer.parseInt(m_urlData.getParameter("start"));
		}
		catch (Exception e)
		{
		}
		if (startHour == -1)
			startHour = now.get(Calendar.HOUR_OF_DAY);

		int timeSpan = 3;
		try
		{
			timeSpan = Integer.parseInt(m_urlData.getParameter("span"));
		}
		catch (Exception e)
		{
		}

		String selected = m_urlData.getParameter("selected");
		if(selected == null || selected.length() == 0)
			selected = "";

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "epg", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("year", new Integer(year).toString());
		root.setAttribute("month", new Integer(month).toString());
		root.setAttribute("day", new Integer(day).toString());
		root.setAttribute("start", new Integer(startHour).toString());
		root.setAttribute("show", new Integer(timeSpan).toString());

		Element itemEl = null;
		Element elm = null;
		Text text = null;

		// add now time/date
		Calendar startPointer = Calendar.getInstance();
		startPointer.set(Calendar.SECOND, 0);
		startPointer.set(Calendar.MINUTE, 0);
		startPointer.set(Calendar.MILLISECOND, 0);
		startPointer.set(Calendar.YEAR, year);
		startPointer.set(Calendar.MONTH, month-1);
		startPointer.set(Calendar.DATE, day);
		startPointer.set(Calendar.HOUR_OF_DAY, startHour);

		long nowLong = new Date().getTime();
		long startLong = startPointer.getTime().getTime();
		long minPast = (nowLong - startLong) / (1000 * 60);

		if(minPast < (timeSpan * 60) && minPast > -1)
		{
			itemEl = doc.createElement("now_pointer");
			itemEl.setAttribute("min", new Integer((int)minPast).toString());
			root.appendChild(itemEl);   
		}

		//////////////////////////////////////////////////////////////////
		// Add the time items
		//////////////////////////////////////////////////////////////////

		int hour = startHour;

		String xm = "am";

		if (hour > 12)
			hour = startHour - 12;
		if (startHour >= 12)
			xm = "pm";

		if (hour == 0)
			hour = 12;

		int min = 0;
		int totalCols = timeSpan * 2;

		for (int x = 0; x < totalCols; x++)
		{

			itemEl = doc.createElement("time");
			text = doc.createTextNode(hour + ":" + intToStr(min) + xm);
			itemEl.appendChild(text);
			root.appendChild(itemEl);

			min += 30;
			if (min == 60)
			{
				hour += 1;
				min = 0;
			}

			if (hour == 12 && min == 0)
			{
				if (xm.equals("am"))
					xm = "pm";
				else
					xm = "am";
			}

			if (hour == 13)
			{
				hour = 1;
			}
		}

		//////////////////////////////////////////////////////////////////
		// Add the channels
		//////////////////////////////////////////////////////////////////

		GuideStore epgStore = GuideStore.getInstance();
		Vector<?> channelMap = epgStore.getOldChannelMap();

		for (int x = 0; x < channelMap.size(); x++)
		{
			String[] map = (String[]) channelMap.get(x);

			Element channel = doc.createElement("channel");
			channel.setAttribute("id", map[0]);

			Element disName = doc.createElement("display-name");
			Text chaName = doc.createTextNode(map[0]);
			disName.appendChild(chaName);

			channel.appendChild(disName);

			root.appendChild(channel);

		}

		// ////////////////////////////////////////////////////////////////
		// Add pre/next navigation links
		//////////////////////////////////////////////////////////////////

		Calendar start = Calendar.getInstance();
		start.set(Calendar.YEAR, year);
		start.set(Calendar.MONTH, month - 1);
		start.set(Calendar.DATE, day);
		start.set(Calendar.HOUR_OF_DAY, startHour);
		start.set(Calendar.MINUTE, 0);
		start.set(Calendar.SECOND, 0);
		start.add(Calendar.SECOND, -1);
		start.set(Calendar.MILLISECOND, 0);

		Calendar end = Calendar.getInstance();
		end.setTime(start.getTime());
		end.add(Calendar.HOUR_OF_DAY, timeSpan);

		Calendar startTime = Calendar.getInstance();
		startTime.set(Calendar.MILLISECOND, 0);

		start.add(Calendar.HOUR_OF_DAY, (-1 * (timeSpan - 1)));

		String prevLink = "/servlet/" + m_urlData.getServletClass() + "?action=01&" + "year=" + start.get(Calendar.YEAR) + "&" + "month=" + (start.get(Calendar.MONTH) + 1) + "&" + "day=" + start.get(Calendar.DATE) + "&" + "start=" + start.get(Calendar.HOUR_OF_DAY) + "&" + "span=" + timeSpan;

		// set to this time span
		start.add(Calendar.HOUR_OF_DAY, (timeSpan - 1));

		root.setAttribute("title", m_store.dayName.get(new Integer(start.get(Calendar.DAY_OF_WEEK))) + " (" + start.get(Calendar.DATE) + "/" + (start.get(Calendar.MONTH) + 1) + "/" + start.get(Calendar.YEAR) + ")");

		// work out next time span
		start.add(Calendar.HOUR_OF_DAY, (timeSpan + 1));

		String nextLink = "/servlet/" + m_urlData.getServletClass() + "?action=01&" + "year=" + start.get(Calendar.YEAR) + "&" + "month=" + (start.get(Calendar.MONTH) + 1) + "&" + "day=" + start.get(Calendar.DATE) + "&" + "start=" + start.get(Calendar.HOUR_OF_DAY) + "&" + "span=" + timeSpan;

		start.add(Calendar.HOUR_OF_DAY, (-1 * (timeSpan + 1)));

		itemEl = doc.createElement("navigation");

		elm = doc.createElement("next");
		text = doc.createTextNode(nextLink);
		elm.appendChild(text);
		itemEl.appendChild(elm);

		elm = doc.createElement("previous");
		text = doc.createTextNode(prevLink);
		elm.appendChild(text);
		itemEl.appendChild(elm);

		elm = doc.createElement("selected");
		text = doc.createTextNode(selected);
		elm.appendChild(text);
		itemEl.appendChild(elm);

		root.appendChild(itemEl);

		//////////////////////////////////////////////////////////////////
		// Add the programs
		//////////////////////////////////////////////////////////////////

		SimpleDateFormat dateFormater = new SimpleDateFormat("yyyyMMddHHmmss Z");
		StringBuffer dateBuff = new StringBuffer();

		Set<?> wsChannels = m_store.getChannels().keySet();

		boolean channelMapped = true;
		for (int x = 0; x < channelMap.size(); x++)
		{
			String[] map = (String[]) channelMap.get(x);
			String channelName = map[0];

			if (channelName == null || !wsChannels.contains(map[0]))
			{
				channelName = "Not Mapped";
				channelMapped = false;
			}
			else
			{
				channelMapped = true;

				GuideItem[] programs = epgStore.getProgramsInc(start.getTime(), end.getTime(), map[1]);

				int colCount = 0;

				for (int y = 0; y < programs.length; y++)
				{
					GuideItem item = programs[y];

					///////////////////////////////////////////////////////////////////////////////////////////////////////////
					// add start padding if needed
					//
					start.add(Calendar.SECOND, 1);
					startTime.setTime(item.getStart());
					long pastStart = startTime.getTime().getTime() - start.getTime().getTime();
					if (y == 0 && pastStart > 0)
					{
						Element program_PH = doc.createElement("programme");

						dateBuff = new StringBuffer();
						dateFormater.format(start.getTime(), dateBuff, new FieldPosition(0));
						program_PH.setAttribute("start", dateBuff.toString());

						dateBuff = new StringBuffer();
						dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
						program_PH.setAttribute("stop", dateBuff.toString());

						program_PH.setAttribute("channel", channelName);

						Element titleElement = doc.createElement("title");
						Text titleText = doc.createTextNode("EMPTY");
						titleElement.appendChild(titleText);
						program_PH.appendChild(titleElement);

						Element subTitleElement = doc.createElement("sub-title");
						Text subTitleText = doc.createTextNode("empty");
						subTitleElement.appendChild(subTitleText);
						program_PH.appendChild(subTitleElement);

						Element descElement = doc.createElement("desc");
						Text descText = doc.createTextNode("empty");
						descElement.appendChild(descText);
						program_PH.appendChild(descElement);

						Element lengthElement = doc.createElement("length");
						lengthElement.setAttribute("units", "minutes");
						Text lengthText = doc.createTextNode(new Long(pastStart / (1000 * 60)).toString());
						lengthElement.appendChild(lengthText);
						program_PH.appendChild(lengthElement);

						Element programLengthElement = doc.createElement("programLength");
						programLengthElement.setAttribute("units", "minutes");
						Text programLengthText = doc.createTextNode(new Long(pastStart / (1000 * 60)).toString());
						programLengthElement.appendChild(programLengthText);
						program_PH.appendChild(programLengthElement);

						root.appendChild(program_PH);

						colCount += (int) (pastStart / (1000 * 60));
					}
					start.add(Calendar.SECOND, -1);
					///////////////////////////////////////////////////////////////////////////////////////////////////////////

					///////////////////////////////////////////////////////////////////////////////////////////////////////////
					// Add padding item if needed
					//    
					if (y > 0)
					{
						long skip = item.getStart().getTime() - (programs[y - 1].getStart().getTime() + (programs[y - 1].getDuration() * 1000 * 60));
						if (skip > 0)
						{
							//DVScheduler.log("Skipping : " + skip);

							Element program_PH = doc.createElement("programme");

							dateBuff = new StringBuffer();
							dateFormater.format(programs[y - 1].getStop(), dateBuff, new FieldPosition(0));
							program_PH.setAttribute("start", dateBuff.toString());

							dateBuff = new StringBuffer();
							dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
							program_PH.setAttribute("stop", dateBuff.toString());

							program_PH.setAttribute("channel", channelName);

							Element titleElement = doc.createElement("title");
							Text titleText = doc.createTextNode("EMPTY");
							titleElement.appendChild(titleText);
							program_PH.appendChild(titleElement);

							Element subTitleElement = doc.createElement("sub-title");
							Text subTitleText = doc.createTextNode("empty");
							subTitleElement.appendChild(subTitleText);
							program_PH.appendChild(subTitleElement);

							Element descElement = doc.createElement("desc");
							Text descText = doc.createTextNode("empty");
							descElement.appendChild(descText);
							program_PH.appendChild(descElement);

							Element lengthElement = doc.createElement("length");
							lengthElement.setAttribute("units", "minutes");
							Text lengthText = doc.createTextNode(new Long(skip / (1000 * 60)).toString());
							lengthElement.appendChild(lengthText);
							program_PH.appendChild(lengthElement);

							Element programLengthElement = doc.createElement("programLength");
							programLengthElement.setAttribute("units", "minutes");
							Text programLengthText = doc.createTextNode(new Long(pastStart / (1000 * 60)).toString());
							programLengthElement.appendChild(programLengthText);
							program_PH.appendChild(programLengthElement);

							root.appendChild(program_PH);

							colCount += (int) (skip / (1000 * 60));
						}
					}
					///////////////////////////////////////////////////////////////////////////////////////////////////////////

					Element program = doc.createElement("programme");

					dateBuff = new StringBuffer();
					dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
					program.setAttribute("start", dateBuff.toString());

					dateBuff = new StringBuffer();
					dateFormater.format(item.getStop(), dateBuff, new FieldPosition(0));
					program.setAttribute("stop", dateBuff.toString());

					program.setAttribute("channel", channelName);

					Element titleElement = doc.createElement("title");
					Text titleText = doc.createTextNode(removeChars(item.getName()));
					titleElement.appendChild(titleText);
					program.appendChild(titleElement);

					Element subTitleElement = doc.createElement("sub-title");
					Text subTitleText = doc.createTextNode(removeChars(item.getSubName()));
					subTitleElement.appendChild(subTitleText);
					program.appendChild(subTitleElement);

					Vector<String> categories = item.getCategories();
					if (categories != null)
					{
						for (int i=0; i<categories.size(); i++)
						{
							Element catElement = doc.createElement("category");
							Text catText = doc.createTextNode(categories.get(i));
							catElement.appendChild(catText);
							program.appendChild(catElement);
						}
					}

					Element descElement = doc.createElement("desc");
					Text descText = doc.createTextNode(removeChars(item.getDescription()));
					descElement.appendChild(descText);
					program.appendChild(descElement);

					int fits = 0;
					int colSpan = item.getDuration();
					if (item.getStart().getTime() < start.getTime().getTime() && item.getStop().getTime() > end.getTime().getTime())
					{
						fits = 1;
						colSpan = (timeSpan * 60);
					}
					else if (y == 0 && start.getTime().getTime() > item.getStart().getTime())
					{
						fits = 2;
						colSpan -= ((start.getTime().getTime() - item.getStart().getTime()) / (1000 * 60)) + 1;
					}
					else if (y == programs.length - 1 && (item.getStop().getTime() - 5000) > end.getTime().getTime())
					{
						fits = 3;
						colSpan = (timeSpan * 60) - colCount;
					}

					colCount += colSpan;

					Element lengthElement = doc.createElement("length");
					lengthElement.setAttribute("units", "minutes");
					lengthElement.setAttribute("fits", new Integer(fits).toString());
					Text lengthText = doc.createTextNode(new Integer(colSpan).toString());
					lengthElement.appendChild(lengthText);
					program.appendChild(lengthElement);

					Element programLengthElement = doc.createElement("programLength");
					programLengthElement.setAttribute("units", "minutes");
					Text programLengthText = doc.createTextNode(new Long(item.getDuration()).toString());
					programLengthElement.appendChild(programLengthText);
					program.appendChild(programLengthElement);

					String addLink = "";
					if (channelMapped)
					{
						addLink = "/servlet/KBScheduleDataRes?action=03&" + "buffer=yes&" + "hour=" + startTime.get(Calendar.HOUR_OF_DAY) + "&" + "min=" + startTime.get(Calendar.MINUTE) + "&" + "channel=" + Formatter.encodeURL(map[0], "UTF-8") + "&" + "type=0&" + "duration=" + item.getDuration() + "&" + "name=" + Formatter.encodeURL(item.getName(), "UTF-8") + "&" + "day=" + startTime.get(Calendar.DATE) + "&" + "month=" + startTime.get(Calendar.MONTH) + "&" + "year=" + startTime.get(Calendar.YEAR);
					}

					Element infoUrlElement = doc.createElement("progAdd");
					Text infoUrlText = doc.createTextNode(addLink);
					infoUrlElement.appendChild(infoUrlText);
					program.appendChild(infoUrlElement);

					String detailsLink = "";
					if (channelMapped)
					{               
						detailsLink = "/servlet/KBEpgDataRes?action=05&" + "channel=" + Formatter.encodeURL(map[0], "UTF-8") + "&id=" + Formatter.encodeURL(item.toString(), "UTF-8");
					}

					Element detailsUrlElement = doc.createElement("showDetails");
					Text detailsUrlText = doc.createTextNode(detailsLink);
					detailsUrlElement.appendChild(detailsUrlText);
					program.appendChild(detailsUrlElement);               

					root.appendChild(program);
				}
			}
		}

		//////////////////////////////////////////////////////////////////
		// Add Schedules
		//////////////////////////////////////////////////////////////////

		Text textNode = null;
		Element elementNaode = null;

		for (int x = 0; x < channelMap.size(); x++)
		{
			String[] map = (String[]) channelMap.get(x);

			int padding = 0;
			int loops = 0;

			String chanName = map[0];

			if (chanName != null)
			{
				Vector<ScheduleItem> items = new Vector<ScheduleItem>();
				m_store.getSchedulesWhenInc(start.getTime(), end.getTime(), chanName, items);

				ScheduleItem[] schedules = items.toArray(new ScheduleItem[0]);

				while (schedules.length > 0)
				{
					Vector<ScheduleItem> overlap = new Vector<ScheduleItem>();

					int colCount = 0;

					for (int sch = 0; sch < schedules.length; sch++)
					{
						ScheduleItem item = schedules[sch];
						startTime.setTime(item.getStart());
						startTime.set(Calendar.MILLISECOND, 0);

						if (sch > 0 && ((item.getStart().getTime() < schedules[sch - 1].getStop().getTime() && item.getStart().getTime() > schedules[sch - 1].getStart().getTime()) || (item.getStop().getTime() > schedules[sch - 1].getStart().getTime() && item.getStart().getTime() < schedules[sch - 1].getStop().getTime())))
						{
							overlap.add(item);
						}
						else
						{

							padding = 0;

							///////////////////////////////////////////////////////////////////////////////////////////////////////////
							// add start padding if needed
							//
							//start.add(Calendar.SECOND, 1);
							startTime.setTime(item.getStart());
							long pastStart = startTime.getTime().getTime() - start.getTime().getTime();
							if (sch == 0 && pastStart > 0)
							{
								padding = (int) (pastStart / (1000 * 60));
								colCount += (int) (pastStart / (1000 * 60));
							}
							start.add(Calendar.SECOND, -1);
							///////////////////////////////////////////////////////////////////////////////////////////////////////////

							///////////////////////////////////////////////////////////////////////////////////////////////////////////
							// Add padding item if needed
							//    
							if (sch > 0)
							{
								long skip = item.getStart().getTime() - (schedules[sch - 1].getStart().getTime() + (schedules[sch - 1].getDuration() * 1000 * 60));
								if (skip > 0)
								{
									//DVScheduler.log("Skipping : " + skip);

									padding = ((int) (pastStart / (1000 * 60))) - colCount;
									colCount += (int) (skip / (1000 * 60));
								}
							}
							///////////////////////////////////////////////////////////////////////////////////////////////////////////

							int fits = 0;
							int colSpan = item.getDuration();
							if (item.getStart().getTime() < start.getTime().getTime() && item.getStop().getTime() > end.getTime().getTime())
							{
								// Just make it the full show width if it start before
								// and ends after our show span
								colSpan = (timeSpan * 60);
								fits = 1;
							}
							else if (sch == 0 && start.getTime().getTime() > item.getStart().getTime())
							{
								// if it is the first one and starts before our show
								// span then cut the begining off
								colSpan -= ((start.getTime().getTime() - item.getStart().getTime()) / (1000 * 60)) + 1;
								fits = 2;
							}
							else if (item.getStop().getTime() > end.getTime().getTime())
							{
								// if it end safter your show span cut the end off
								long temp = end.getTime().getTime() - item.getStart().getTime();
								temp = temp / (1000 * 60);
								colSpan = (int) temp + 1;
								fits = 3;

								//colSpan = (timeSpan * 60) - colCount;
							}

							colCount += colSpan;

							Element schedule = doc.createElement("schedule");

							dateBuff = new StringBuffer();
							dateFormater.format(item.getStart(), dateBuff, new FieldPosition(0));
							schedule.setAttribute("start", dateBuff.toString());

							dateBuff = new StringBuffer();
							dateFormater.format(item.getStop(), dateBuff, new FieldPosition(0));
							schedule.setAttribute("stop", dateBuff.toString());

							schedule.setAttribute("duration", new Integer(item.getDuration()).toString());
							schedule.setAttribute("span", new Integer(colSpan).toString());
							schedule.setAttribute("prePadding", new Integer(padding).toString());
							schedule.setAttribute("overlapCount", new Integer(loops).toString());
							schedule.setAttribute("channel", chanName);
							schedule.setAttribute("fits", new Integer(fits).toString());

							elementNaode = doc.createElement("id");
							textNode = doc.createTextNode(item.toString());
							elementNaode.appendChild(textNode);
							schedule.appendChild(elementNaode);

							elementNaode = doc.createElement("title");
							textNode = doc.createTextNode(removeChars(item.getName()));
							elementNaode.appendChild(textNode);
							schedule.appendChild(elementNaode);

							elementNaode = doc.createElement("itemState");
							textNode = doc.createTextNode(new Integer(item.getState()).toString());
							elementNaode.appendChild(textNode);
							schedule.appendChild(elementNaode);

							elementNaode = doc.createElement("itemStatus");
							textNode = doc.createTextNode(item.getStatus());
							elementNaode.appendChild(textNode);
							schedule.appendChild(elementNaode);

							String action = "/servlet/KBScheduleDataRes?action=04&id=" + item.toString();

							elementNaode = doc.createElement("progEdit");
							textNode = doc.createTextNode(action);
							elementNaode.appendChild(textNode);
							schedule.appendChild(elementNaode);

							root.appendChild(schedule);
						}

					}

					schedules = (ScheduleItem[]) overlap.toArray(new ScheduleItem[0]);
					loops++;

				}
			}
		}

		// Do transform and return data
		XSL transformer = new XSL(doc, "kb-epg.xsl", m_headers, m_urlData);

		// set current page to check point for back
		// remove the selected= first
		String request = m_urlData.getRequestString();
		int indexOf = request.indexOf("&selected=");
		if(indexOf > -1)
		{
			request = request.substring(0, indexOf);
		}
		transformer.addCookie("backURL", request);

		return transformer.transform();
	}

	private byte[] showEpgIndex() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();
		Vector<?> links = guide.getEpgLinks(null);

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "files", null);
		Element root = doc.getDocumentElement();

		root.setAttribute("back", m_store.getTargetUrl("epg-sub", "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons", m_headers.get("User-Agent")));

		Element button = null;
		Element elm = null;
		Text text = null;

		button = doc.createElement("button");
		button.setAttribute("name", "Back");
		elm = doc.createElement("url");
		text = doc.createTextNode(m_store.getTargetUrl("epg-sub", "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons", m_headers.get("User-Agent")));
		elm.appendChild(text);
		button.appendChild(elm);
		root.appendChild(button);

		for (int x = 0; x < links.size(); x++)
		{
			String[] item = (String[]) links.get(x);
			button = doc.createElement("button");
			button.setAttribute("name", item[1]);
			elm = doc.createElement("url");
			text = doc.createTextNode("/servlet/" + m_urlData.getServletClass() + "?action=01&" + item[0]);
			elm.appendChild(text);
			button.appendChild(elm);
			root.appendChild(button);
		}

		// Do transform and return data
		XSL transformer = new XSL(doc, "kb-buttons.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] reloadXMLTVdata() throws Exception
	{
		StringBuffer buff1 = new StringBuffer();

		GuideStore guide = GuideStore.getInstance();

		guide.loadXmltv(buff1, 0, 0);

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "log", null);
		Element root = doc.getDocumentElement();
		root.setAttribute("id", "");

		Element logitem = null;
		Element elm = null;
		Text text = null;

		String[] lines = buff1.toString().split("\n");
		for (int x = 0; x < lines.length; x++)
		{
			logitem = doc.createElement("logitem");
			elm = doc.createElement("line");
			text = doc.createTextNode(lines[x]);
			elm.appendChild(text);
			logitem.appendChild(elm);
			root.appendChild(logitem);
		}

		// Do transform and return data
		XSL transformer = new XSL(doc, "kb-redo.xsl", m_headers, m_urlData);
		return transformer.transform(); 
	}

	private String intToStr(int num)
	{
		if (num < 10)
			return "0" + num;
		else
			return new Integer(num).toString();

	}

	private String removeChars(String data)
	{
		data = data.replaceAll("'", "`");
		data = data.replaceAll("\"", "`");

		return data;
	}
}
