/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.text.*;
import java.util.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;

class KBEpgItemsRes extends HTTPResponse
{

	public KBEpgItemsRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if("01".equals(action))
		{
			m_outStream.write(showNames());
			return;
		}
		else if("02".equals(action))
		{
			m_outStream.write(showItemInstances());
			return;
		}
		else if("03".equals(action))
		{
			m_outStream.write(showSearchForm());
			return;
		}
		else if("04".equals(action))
		{
			m_outStream.write(showSearchResults());
			return;
		}
		else
		{
			m_outStream.write(showFirstLetter());
			return;
		}
	}

	private byte[] showItemInstances() throws Exception
	{
		String name = m_urlData.getParameter("name");
		if(name == null)
			name = "";

		String start = m_urlData.getParameter("start");
		if(start == null)
			start = "0";

		String show = m_urlData.getParameter("show");
		if(show == null)
			show = "10";

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "buttons", null); 

		Element root = doc.getDocumentElement();

		root.setAttribute("title" , name);

		root.setAttribute("start" , start);
		root.setAttribute("show" , show);

		root.setAttribute("back" , "/servlet/ApplyTransformRes?xml=epg-index&xsl=kb-buttons");

		Element button = null;
		Element elm = null;
		Text text = null;  

		button = doc.createElement("mainurl");
		text = doc.createTextNode("/servlet/KBEpgItemsRes?action=02&name=" + Formatter.encodeURL(name, "UTF-8") + "&");
		button.appendChild(text);
		root.appendChild(button);

		SimpleDateFormat df = new SimpleDateFormat("EEE MMM d h:mm aa");

		GuideStore guide = GuideStore.getInstance();
		Vector<?> chanMap = guide.getOldChannelMap();

		for (int q = 0; q < chanMap.size(); q++)
		{
			String[] map = (String[])chanMap.get(q);

			GuideItem[] items = guide.getItems(name, map[1]);

			for (int x = 0; x < items.length; x++)
			{
				String butText = df.format(items[x].getStart());
				if(map[0] != null)
					butText += " (" + map[0] + ")";
				else
					butText += " (Not Mapped)";

				button = doc.createElement("button");
				button.setAttribute("name", butText);
				elm = doc.createElement("url");

				Calendar cal = Calendar.getInstance();
				cal.setTime(items[x].getStart());

				String buttonUrl = "/servlet/KBEpgDataRes?action=05&channel=" + 
				Formatter.encodeURL(map[0], "UTF-8") + 
				"&id=" + items[x].toString();

				text = doc.createTextNode(buttonUrl);
				elm.appendChild(text);
				button.appendChild(elm);
				root.appendChild(button);
			}
		}

		// send back data
		XSL transformer = new XSL(doc, "kb-list.xsl", m_headers, m_urlData);
		return transformer.transform();
	}   

	private byte[] showSearchForm() throws Exception
	{
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "search", null); 

		Element root = doc.getDocumentElement();

		root.setAttribute("back" , "/servlet/ApplyTransformRes?xml=epg-index&xsl=kb-buttons");

		//Channel
		Element formEl = doc.createElement("channel");
		getChannelList(doc, formEl);
		formEl.setAttribute("value", "Any");
		root.appendChild(formEl);

		// Category
		formEl = doc.createElement("category");
		getCatList(doc, formEl);
		formEl.setAttribute("value", "Any");
		root.appendChild(formEl);           

		// send back data
		XSL transformer = new XSL(doc, "kb-SearchEpg.xsl", m_headers, m_urlData);
		return transformer.transform();
	}      

	private void getChannelList(Document doc, Element formEl)
	{
		Element option = null;
		Text text = null;

		HashMap<String, Channel> channels = m_store.getChannels();
		String[] keys = (String[]) channels.keySet().toArray(new String[0]);
		Arrays.sort(keys);

		option = doc.createElement("option");
		text = doc.createTextNode("Any");
		option.appendChild(text);
		formEl.appendChild(option);

		for (int x = 0; x < keys.length; x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(keys[x]);
			option.appendChild(text);
			formEl.appendChild(option);
		}

	}

	private void getCatList(Document doc, Element formEl)
	{
		Element option = null;
		Text text = null;

		GuideStore guide = GuideStore.getInstance();
		String[] cats = guide.getCategoryStrings();

		option = doc.createElement("option");
		text = doc.createTextNode("Any");
		option.appendChild(text);
		formEl.appendChild(option);      

		for (int x = 0; x < cats.length; x++)
		{
			option = doc.createElement("option");
			text = doc.createTextNode(cats[x]);
			option.appendChild(text);
			formEl.appendChild(option);
		}
	}

	private byte[] showSearchResults() throws Exception
	{
		String name = m_urlData.getParameter("name");
		if(name == null || name.length() == 0)
			name = "";

		String cat = m_urlData.getParameter("cat");
		if(cat == null || cat.length() == 0 || cat.equals("Any"))
			cat = "";

		String chan = m_urlData.getParameter("chan");
		if(chan == null || chan.length() == 0 || chan.equals("Any"))
			chan = "";

		String start = m_urlData.getParameter("start");
		if(start == null || start.length() == 0)
			start = "0";

		String show = m_urlData.getParameter("show");
		if(show == null || show.length() == 0)
			show = "10";

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "buttons", null); 

		Element root = doc.getDocumentElement();

		root.setAttribute("back" , "/servlet/KBEpgItemsRes?action=03");

		root.setAttribute("start" , start);
		root.setAttribute("show" , show);

		Element button = null;
		Element elm = null;
		Text text = null;

		button = doc.createElement("mainurl");
		String actionText = "/servlet/KBEpgItemsRes?action=04" +
		"&name=" + Formatter.encodeURL(name, "UTF-8") + 
		"&cat=" + Formatter.encodeURL(cat, "UTF-8") + 
		"&chan=" + Formatter.encodeURL(chan, "UTF-8") + "&";
		text = doc.createTextNode(actionText);
		button.appendChild(text);
		root.appendChild(button);

		actionText = "(" + name + ") Category(" + cat + ") Channel(" + chan + ")";
		root.setAttribute("title", actionText);      

		GuideStore guide = GuideStore.getInstance();
		HashMap<String, Vector<GuideItem>> results = new HashMap<String, Vector<GuideItem>>();
		guide.simpleEpgSearch(name, 3, cat, chan, results);  

		String[] keys = (String[])results.keySet().toArray(new String[0]);

		for (int y = 0; y < keys.length; y++)
		{
			Vector<?> result = (Vector<?>) results.get(keys[y]);

			if (result != null && result.size() > 0)
			{         
				for (int x = 0; x < result.size(); x++)
				{
					GuideItem item = (GuideItem) result.get(x);

					button = doc.createElement("button");
					button.setAttribute("name" , item.getName());
					elm = doc.createElement("url");

					String actionURL = "/servlet/KBEpgItemsRes?action=02&name=" + 
					Formatter.encodeURL(item.getName(), "UTF-8");

					text = doc.createTextNode(actionURL);
					elm.appendChild(text);
					button.appendChild(elm);  
					root.appendChild(button);
				}
			}
		}

		// send back data
		XSL transformer = new XSL(doc, "kb-list.xsl", m_headers, m_urlData);
		return transformer.transform();      
	}

	private byte[] showNames() throws Exception
	{
		String letter = m_urlData.getParameter("letter");

		String start = m_urlData.getParameter("start");
		if(start == null)
			start = "0";

		String show = m_urlData.getParameter("show");
		if(show == null)
			show = "10";

		GuideStore guide = GuideStore.getInstance();
		String[] progs = guide.getNamesStartingWith(letter);

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "buttons", null); 

		Element root = doc.getDocumentElement();

		root.setAttribute("start" , start);
		root.setAttribute("show" , show);      
		root.setAttribute("back" , "/servlet/KBEpgItemsRes");

		Element button = null;
		Element elm = null;
		Text text = null;  

		button = doc.createElement("mainurl");
		text = doc.createTextNode("/servlet/KBEpgItemsRes?action=01&letter=" + letter + "&");
		button.appendChild(text);
		root.appendChild(button);

		for(int x = 0; x < progs.length; x++)
		{
			button = doc.createElement("button");         
			button.setAttribute("name" , progs[x]);
			elm = doc.createElement("url");

			String actionURL = "/servlet/KBEpgItemsRes?action=02&name=" + Formatter.encodeURL(progs[x], "UTF-8");         

			text = doc.createTextNode(actionURL);
			elm.appendChild(text);
			button.appendChild(elm);  
			root.appendChild(button);        
		}

		// send back data
		XSL transformer = new XSL(doc, "kb-list.xsl", m_headers, m_urlData);
		return transformer.transform();
	}

	private byte[] showFirstLetter() throws Exception
	{
		GuideStore guide = GuideStore.getInstance();
		String[] firsts = guide.getFirstLetters();

		String start = m_urlData.getParameter("start");
		if(start == null)
			start = "0";

		String show = m_urlData.getParameter("show");
		if(show == null)
			show = "10";

		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "buttons", null); 

		Element root = doc.getDocumentElement();

		root.setAttribute("start" , start);
		root.setAttribute("show" , show);

		root.setAttribute("back" , m_store.getTargetUrl("epg-sub", "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons", m_headers.get("User-Agent")));

		Element button = null;
		Element elm = null;
		Text text = null;  

		button = doc.createElement("mainurl");
		text = doc.createTextNode("/servlet/KBEpgItemsRes?");
		button.appendChild(text);
		root.appendChild(button);

		if(firsts.length == 0)
		{
			button = doc.createElement("button");         
			button.setAttribute("name" , "None");
			elm = doc.createElement("url");
			text = doc.createTextNode("/servlet/KBEpgItemsRes");
			elm.appendChild(text);
			button.appendChild(elm);  
			root.appendChild(button);        
		}

		for(int x = 0; x < firsts.length; x++)
		{
			button = doc.createElement("button");         
			button.setAttribute("name" , firsts[x]);
			elm = doc.createElement("url");
			text = doc.createTextNode("/servlet/KBEpgItemsRes?action=01&letter=" + firsts[x]);
			elm.appendChild(text);
			button.appendChild(elm);  
			root.appendChild(button);      
		}               

		// send back data
		XSL transformer = new XSL(doc, "kb-list.xsl", m_headers, m_urlData);
		return transformer.transform();
	}
}