/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;
import java.text.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;

class KBFileManagerRes extends HTTPResponse
{

	public KBFileManagerRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if("01".equals(action))
		{
			m_outStream.write(showFileActions());
			return;
		}
		if("02".equals(action))
		{
			m_outStream.write(deleteFile());
			return;
		}      
		if("03".equals(action))
		{
			m_outStream.write(runCommand());
			return;
		}        
		else
		{
			m_outStream.write(showFiles());
			return;
		}
	}

	private byte[] runCommand() throws Exception
	{
		String commandID = m_urlData.getParameter("command");

		File thisFile = new File(m_urlData.getParameter("file"));
		String requestedFilePath = thisFile.getCanonicalPath();

		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = m_store.getServeFilePaths();
		if (paths[0].isEmpty())
			paths[0] = ".";
		File baseDir = new File(paths[0]);

		// check requested file is still within the capture dir
		boolean bFound = false;
		for(int x = paths.length-1; x >= 0; x--)
		{
			if (paths[x].isEmpty())
				paths[x] = ".";

			baseDir = new File(paths[x]);
			String rootFilePath = baseDir.getCanonicalPath();

			if(requestedFilePath.indexOf(rootFilePath) > -1)
			{
				bFound = true;
				break;
			}      

		}

		if (!bFound)
		{
			throw new Exception("File out of bounds!");
		}

		Map<String, TaskCommand> tasks = m_store.getTaskList();
		TaskCommand taskCommand = tasks.get(commandID);

		boolean autoRem = taskCommand.getAutoRemove();
		String command = taskCommand.getCommand();

		command = command.replace("$filename", thisFile.getCanonicalPath());
		
		if ( taskCommand.isConditional() )
		{
			TaskItem taskItem = new TaskItem(taskCommand, command, null);
			ThreadLock.getInstance().getLock();
			try
			{
				DVScheduler.log("Queuing : " + command);
				m_store.m_waitingTaskList.add(taskItem);
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
		}
		else
		{
			ThreadLock.getInstance().getLock();
			try
			{
				DVScheduler.log("Running : " + command);

				CommandWaitThread comThread = new CommandWaitThread(command, taskCommand.getDelay());
				Thread comWait = new Thread(Thread.currentThread().getThreadGroup(), comThread, comThread.getClass().getName());
				comWait.start();
				String key = new Long(new Date().getTime()).toString() + "-" + new Long(m_store.m_rand.nextLong()).toString();
				TaskItemThread taskItem = new TaskItemThread(key, comThread, command, autoRem);
				taskItem.m_allowSuspend = taskCommand.isAllowSuspend();
				Thread taskThread = new Thread(Thread.currentThread().getThreadGroup(), taskItem, taskItem.getClass().getName());
				taskThread.start();
				m_store.m_runningTaskList.put(key, taskItem);  
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
		}

		String start = m_urlData.getParameter("start");
		if(start == null)
			start = "0";
		start = start.trim();

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "?path=" + Formatter.encodeURL(thisFile.getParent(), "UTF-8") + "&start=" + start + "\n\n");
		return out.toString().getBytes();
	}

	private byte[] deleteFile() throws Exception
	{
		File thisFile = new File(m_urlData.getParameter("file"));
		String requestedFilePath = thisFile.getCanonicalPath();

		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = m_store.getServeFilePaths();
		if (paths[0].isEmpty())
			paths[0] = ".";
		File baseDir = new File(paths[0]);

		// check requested file is still within the capture dir
		boolean bFound = false;
		for(int x = paths.length-1; x >= 0; x--)
		{
			if (paths[x].isEmpty())
				paths[x] = ".";

			baseDir = new File(paths[x]);
			String rootFilePath = baseDir.getCanonicalPath();

			if(requestedFilePath.indexOf(rootFilePath) > -1)
			{
				bFound = true;
				break;
			}      

		}

		if (!bFound)
		{
			throw new Exception("File out of bounds!");
		}

		if(thisFile != null && thisFile.exists())
		{
			DVScheduler.log( (thisFile.delete() ? "File deleted: " : "Cannot delete: ") + thisFile.getAbsolutePath() );
		}

		String start = m_urlData.getParameter("start");
		if(start == null)
			start = "0";
		start = start.trim();

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "?path=" + Formatter.encodeURL(thisFile.getParent(), "UTF-8") + "&start=" + start + "\n\n");
		return out.toString().getBytes();     
	}

	private byte[] showFileActions() throws Exception
	{
		File file = new File(m_urlData.getParameter("file"));

		if(file == null || !file.exists())
		{
			StringBuffer out = new StringBuffer();
			out.append("HTTP/1.0 302 Moved Temporarily\n");
			out.append("Location: /servlet/KBFileManagerRes\n\n");

			return out.toString().getBytes();
		}   

		String start = m_urlData.getParameter("start");
		if(start == null)
			start = "0";
		start = start.trim();

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();      

		Document doc = di.createDocument("", "buttons", null);
		Element root = doc.getDocumentElement(); 

		root.setAttribute("title" , file.getName());

		Element button = null;
		Element elm = null;
		Text text = null;   

		String action = "";

		button = doc.createElement("button");         
		button.setAttribute("name" , "Back to Files");
		elm = doc.createElement("url");
		action = "/servlet/" + m_urlData.getServletClass() + "?path=" + Formatter.encodeURL(file.getParent(), "UTF-8") + "&start=" + start;
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);  

		root.appendChild(button);          

		button = doc.createElement("button");         
		button.setAttribute("name" , "Delete File");
		elm = doc.createElement("url");
		action = "/servlet/" + m_urlData.getServletClass() + "?action=02&file=" + Formatter.encodeURL(file.getPath(), "UTF-8") + "&start=" + start;
		text = doc.createTextNode(action);
		elm.appendChild(text);
		button.appendChild(elm);

		elm = doc.createElement("confirm");
		text = doc.createTextNode("true");
		elm.appendChild(text);
		button.appendChild(elm); 

		root.appendChild(button);       

		addCommands(doc, file, start);         

		//
		// Send output back 
		//
		XSL transformer = new XSL(doc, "kb-buttons.xsl", m_headers, m_urlData);
		return transformer.transform();          
	}

	private void addCommands(Document doc, File file, String start) throws Exception
	{
		HashMap<String, TaskCommand> tasks = m_store.getTaskList();
		String[] keys = (String[])tasks.keySet().toArray(new String[0]);

		Arrays.sort(keys);

		Element button = null;
		Element elm = null;
		Text text = null;   
		Element root = doc.getDocumentElement();

		for(int x = 0; x < keys.length; x++)
		{
			TaskCommand command = (TaskCommand)tasks.get(keys[x]);

			if(command != null && command.getEnabled())
			{
				String action = "/servlet/" + m_urlData.getServletClass() + "?action=03&file=" + Formatter.encodeURL(file.getPath(), "UTF-8") + "&command=" + Formatter.encodeURL(keys[x], "UTF-8") + "&start=" + start;

				button = doc.createElement("button");
				button.setAttribute("name" , keys[x]);
				elm = doc.createElement("url");
				text = doc.createTextNode(action);
				elm.appendChild(text);
				button.appendChild(elm);

				elm = doc.createElement("confirm");
				text = doc.createTextNode("true");
				elm.appendChild(text);
				button.appendChild(elm); 

				root.appendChild(button);
			}
		}
	}

	private byte[] showFiles() throws Exception
	{
		boolean atRoot = true;

		String pathString = m_urlData.getParameter("path");

		//String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		String[] paths = m_store.getServeFilePaths();
		if (paths[0].isEmpty())
			paths[0] = ".";
		File baseDir = new File(paths[0]);
		int pathSelected = 0;

		if(pathString != null && pathString.length() > 0)
		{
			File path = new File(m_urlData.getParameter("path"));
			String fullPath = path.getCanonicalPath();

			// check requested file is still within the capture dir
			for(int x = paths.length-1; x >= 0; x--)
			{
				if (paths[x].isEmpty())
					paths[x] = ".";

				baseDir = new File(paths[x]);
				if (!baseDir.exists())
					baseDir.mkdirs();
				String rootFilePath = baseDir.getCanonicalPath();

				if(fullPath.equals(rootFilePath))
				{
					atRoot = true;
					pathSelected = x;
					break;
				}
				else if(fullPath.indexOf(rootFilePath) > -1)
				{
					atRoot = false;
					baseDir = path;
					pathSelected = x;
					break;
				} 
			}
		}

		NumberFormat nf = NumberFormat.getInstance();

		int count = 0;

		int start = 0;
		int show = 10;

		try
		{
			show = Integer.parseInt(m_urlData.getParameter("show"));
		}
		catch(Exception e){}

		try
		{
			start = Integer.parseInt(m_urlData.getParameter("start"));
		}
		catch(Exception e){}      

		if(start < 0) start = 0;

		String fileMasks = DataStore.getInstance().getProperty("filebrowser.masks", ".log,.mpg,.mpeg,.bin,.mpv,.mpa,.ts,.tp,.dvr-ms,.rec,.pva,.avi,.mp4,.mkv,.xml,.srt");

		File[] files = baseDir.listFiles(new FileTypeFilter(fileMasks, 2));
		if(files == null)
			files = new File[0];

		boolean dirsAtTop = "1".equals(m_store.getProperty("filebrowser.DirsAtTop", "1"));
		Arrays.sort(files, new CompareFiles(dirsAtTop));  

		//
		// Creation of an XML document
		//
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();

		Document doc = di.createDocument("", "files", null);
		Element root = doc.getDocumentElement();     

		root.setAttribute("back" , "/servlet/ApplyTransformRes?xml=root&xsl=kb-buttons");

		long freeSpace = baseDir.getFreeSpace();
		root.setAttribute("FreeSpace", nf.format((freeSpace / (1024 * 1024))));

		Element fileitem = null;
		Element elm = null;
		Text text = null;  

		// Show capture paths.
		for(int x = 0; x < paths.length; x++)
		{
			count++;

			fileitem = doc.createElement("file");     

			elm = doc.createElement("name");
			if (x == pathSelected)
				text = doc.createTextNode(">" + paths[x] + "<");
			else
				text = doc.createTextNode(paths[x]);
			elm.appendChild(text);
			fileitem.appendChild(elm);     

			elm = doc.createElement("size");
			elm.setAttribute("units" , "");
			text = doc.createTextNode("");
			elm.appendChild(text);
			fileitem.appendChild(elm);      

			String action = "/servlet/" + m_urlData.getServletClass() +
			"?path=" + Formatter.encodeURL_UTF8E(paths[x]);

			elm = doc.createElement("action");
			text = doc.createTextNode(action);
			elm.appendChild(text);
			fileitem.appendChild(elm); 

			root.appendChild(fileitem);
		}

		int fileIndex = start;
		if(baseDir.exists() && baseDir.isDirectory())
		{
			if(!atRoot)
			{
				count++;

				fileitem = doc.createElement("file");     

				elm = doc.createElement("name");
				text = doc.createTextNode("<parent>");
				elm.appendChild(text);
				fileitem.appendChild(elm);     

				elm = doc.createElement("size");
				elm.setAttribute("units" , "");
				text = doc.createTextNode("");
				elm.appendChild(text);
				fileitem.appendChild(elm);      

				String action = "/servlet/" + m_urlData.getServletClass() +
				"?path=" + Formatter.encodeURL(baseDir.getParentFile().getCanonicalPath(), "UTF-8");

				elm = doc.createElement("action");
				text = doc.createTextNode(action);
				elm.appendChild(text);
				fileitem.appendChild(elm); 

				root.appendChild(fileitem);               
			}

			while(fileIndex < files.length && (count) < show)
			{
				if(files[fileIndex].isHidden() == false)
				{            
					count++;

					if(files[fileIndex].isDirectory())
					{
						fileitem = doc.createElement("file");     

						elm = doc.createElement("name");
						text = doc.createTextNode("<" + files[fileIndex].getName() + ">");
						elm.appendChild(text);
						fileitem.appendChild(elm);     

						elm = doc.createElement("size");
						elm.setAttribute("units" , "");
						text = doc.createTextNode("");
						elm.appendChild(text);
						fileitem.appendChild(elm);      

						String action = "/servlet/" + m_urlData.getServletClass() +
						"?path=" + Formatter.encodeURL(files[fileIndex].getCanonicalPath(), "UTF-8");

						elm = doc.createElement("action");
						text = doc.createTextNode(action);
						elm.appendChild(text);
						fileitem.appendChild(elm); 

						root.appendChild(fileitem);     

					}
					else
					{
						fileitem = doc.createElement("file");     

						elm = doc.createElement("name");
						text = doc.createTextNode(files[fileIndex].getName());
						elm.appendChild(text);
						fileitem.appendChild(elm);     

						elm = doc.createElement("size");
						elm.setAttribute("units" , "KB");
						text = doc.createTextNode(nf.format(files[fileIndex].length()/1024));
						elm.appendChild(text);
						fileitem.appendChild(elm);      

						String action = "/servlet/" + m_urlData.getServletClass() + "?action=01" + 
						"&file=" + Formatter.encodeURL(files[fileIndex].getPath(), "UTF-8") + "&start=" + start;

						elm = doc.createElement("action");
						text = doc.createTextNode(action);
						elm.appendChild(text);
						fileitem.appendChild(elm); 

						root.appendChild(fileitem);  
					}
				}

				fileIndex++;            
			}
		} 

		root.setAttribute("start" , new Integer(start).toString());
		root.setAttribute("end" , new Integer(fileIndex).toString());
		root.setAttribute("show", new Integer(show).toString());
		root.setAttribute("total", new Integer(files.length).toString());
		root.setAttribute("path", Formatter.encodeURL(baseDir.getCanonicalPath(), "UTF-8"));

		//
		// Send output back 
		//
		XSL transformer = new XSL(doc, "kb-showfiles.xsl", m_headers, m_urlData);
		return transformer.transform();
	}     
}
