/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.text.*;
import java.util.*;

import org.w3c.dom.*;
import javax.xml.parsers.*;

class KBSysInfoRes extends HTTPResponse
{

	public KBSysInfoRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");
		if("01".equals(action))
		{
			return;
		}
		else
		{
			m_outStream.write(showSysInfo());
			return;
		}
	}

	private byte[] showSysInfo() throws Exception
	{
		// Creation of an XML document
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		DOMImplementation di = db.getDOMImplementation();
		Document doc = di.createDocument("", "sys-info", null); 

		Element root = doc.getDocumentElement();
		Element item = null;

		// Server Version
		item = doc.createElement("info");
		item.setAttribute("name" , "Server Version");
		item.setAttribute("units" , "");
		item.setAttribute("value" , m_store.getVersion());
		root.appendChild(item);  

		// Num Sch
		item = doc.createElement("info");
		item.setAttribute("name" , "Number of Schedules");
		item.setAttribute("units" , "");

		int numSch = m_store.getScheduleList().size();

		item.setAttribute("value" , new Integer(numSch).toString());
		root.appendChild(item);         

		// Num Running
		CaptureDeviceList devList = CaptureDeviceList.getInstance();
		item = doc.createElement("info");
		item.setAttribute("name" , "Devices in use");
		item.setAttribute("units" , "");
		item.setAttribute("value" , new Integer(devList.getRunningDeviceCount()).toString());
		root.appendChild(item);    

		// To Next
		Date now = new Date();
		int timeToNext = -1;
		ScheduleItem next = m_store.getNextSchedule();
		if(next != null)
		{
			timeToNext = (int)((next.getStart().getTime() - now.getTime()) / (1000 * 60));
		}

		item = doc.createElement("info");
		item.setAttribute("name" , "Time to next");
		item.setAttribute("units" , "");
		item.setAttribute("value" , new Integer(timeToNext).toString());
		root.appendChild(item);        

		// Free Space
		String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
		NumberFormat nf = NumberFormat.getNumberInstance();
		File cap = null;
		for(int x = 0; x < paths.length; x++)
		{
			if (paths[x].isEmpty())
				paths[x] = ".";

			cap = new File(paths[x]);
			if (!cap.exists())
				cap.mkdirs();

			long freeSpace = cap.getFreeSpace();
			item = doc.createElement("info");
			item.setAttribute("name" , "Free Space");
			item.setAttribute("units" , "MB");
			item.setAttribute("value" , nf.format((freeSpace / (1024 * 1024))));   
			root.appendChild(item);
		}
		
		// Do transform and return data
		XSL transformer = new XSL(doc, "kb-sys-info.xsl", m_headers, m_urlData);
		return transformer.transform();      
	}
}