/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;
import java.io.*;
import java.net.*;
import java.text.*;

import sun.net.www.protocol.http.AuthCacheImpl;
import sun.net.www.protocol.http.AuthCacheValue;

class LoaderTread implements Runnable
{
	private URL webPage = null;
	private ByteArrayOutputStream m_dataBytes = new ByteArrayOutputStream();
	private boolean finished = false;
	private Date lastRead = new Date();
	private long urlLength = 0;
	private int returnCode = -1;

	public LoaderTread(URL pageUrl, String username, String password)
	{

		webPage = pageUrl;

		try
		{
			DataStore store = DataStore.getInstance();

			// Set the HTTP proxy
			String proxy = store.getProperty("proxy.server", "");
			proxy = proxy.trim();
			if(proxy.length() > 0)
			{
				DVScheduler.log("Using Proxy : " + proxy + " : " + store.getProperty("proxy.port", "8080").trim());
				System.getProperties().put("proxySet", "true");

				System.getProperties().put("proxyHost", proxy);
				System.getProperties().put("proxyPort", store.getProperty("proxy.port", "8080").trim());
			}
			else
			{
				System.getProperties().put("proxySet", "false");
				System.getProperties().put("proxyHost", "");
				System.getProperties().put("proxyPort", "");
			}

			AuthCacheValue.setAuthCache(new AuthCacheImpl());	// Reset.
			Authenticator.setDefault(new HTTPAuthenticator(username, password));
		}
		catch (Exception e)
		{
			DVScheduler.log("Error setting HTTP proxy");
			e.printStackTrace();
		}
	}

	public void run()
	{

		try
		{
			NumberFormat nf = NumberFormat.getNumberInstance();
			lastRead = new Date();
			HttpURLConnection con = null;
			InputStream is = null;

			try
			{
				String verString = DataStore.getInstance().getVersion();
				con = (HttpURLConnection)webPage.openConnection();
				con.setRequestProperty("User-Agent", "DVScheduler(" + verString + ")");
				is = con.getInputStream();
			}
			catch (Exception e)
			{
				DVScheduler.log("ERROR: Url Exception (" + e.toString() + ")");
				//e.printStackTrace();

				finished = true;
				return;
			}

			returnCode = con.getResponseCode();

			int colCount = 0;

			lastRead = new Date();
			byte[] buffer = new byte[256];

			int read = is.read(buffer);

			if (read < 0)
			{
				finished = true;
				return;
			}

			int checkLength = read - 10;
			int nFound = -1;
			checker:
				for (int i=0; i<checkLength; i++)
				{
					switch (buffer[i])
					{
					case 0:
						break checker;
					case '<':
						if ( (buffer[i+1] == '!') && (buffer[i+2] == 'D') && (buffer[i+3] == 'O') && (buffer[i+4] == 'C') && (buffer[i+5] == 'T') && (buffer[i+6] == 'Y') && (buffer[i+7] == 'P') && (buffer[i+8] == 'E') && ((buffer[i+9] == ' ') || (buffer[i+9] == '\t') || (buffer[i+9] == '\r') || (buffer[i+9] == '\n')) )
							break checker;
						else if ( (buffer[i+1] == 't') && (buffer[i+2] == 'v') && ((buffer[i+3] == ' ') || (buffer[i+3] == '\t') || (buffer[i+3] == '>') || (buffer[i+3] == '\r') || (buffer[i+3] == '\n')) )
						{
							nFound = i;
							break checker;
						}
						break;
					}
				}

			if (nFound >= 0)
			{
				m_dataBytes.write(buffer, 0, nFound);
				m_dataBytes.write("<!DOCTYPE tv [<!ENTITY % xhtml-lat1 SYSTEM \"xhtml-lat1.ent\"> %xhtml-lat1; <!ENTITY % xhtml-special SYSTEM \"xhtml-special.ent\"> %xhtml-special;  <!ENTITY % xhtml-symbol SYSTEM \"xhtml-symbol.ent\"> %xhtml-symbol;]>".getBytes());
				m_dataBytes.write(buffer, nFound, read-nFound);
			}
			else
				m_dataBytes.write(buffer, 0, read);

			read = is.read(buffer);
			while (read > -1)
			{
				if(colCount == 80)
				{
					System.out.println(" " + nf.format(m_dataBytes.size()));
					colCount = 0;
				}
				colCount++;

				System.out.print(".");
				lastRead = new Date();
				m_dataBytes.write(buffer, 0, read);
				read = is.read(buffer);
			}
			System.out.println(" " + nf.format(m_dataBytes.size()));
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finished = true;
	}

	public void kill()
	{
		returnCode = -1;
	}

	public int getResponceCode()
	{
		return returnCode;
	}

	public boolean isFinished()
	{

		return finished;
	}

	/*
	public String getDataString()
	{
		return m_dataBytes.toString();
	}
	 */

	public byte[] getDataBytes()
	{

		return m_dataBytes.toByteArray();
	}

	public boolean isTimedOut(int sec)
	{

		Date now = new Date();
		long timeOut = sec * 1000;
		long lifeTime = now.getTime() - lastRead.getTime();

		if(lifeTime > timeOut)
			return true;
		else
			return false;
	}

	public long getLength()
	{

		return urlLength;
	}
}
