/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */


import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.*;

public class ProcessInputReader implements Runnable
{
	Process process = null;
	Vector<String> commandQue = new Vector<String>();
	BufferedReader in = null;
	Integer exitCode = null;
	CaptureEngine m_captureEngine;
	private DataStore m_store = null;

	public ProcessInputReader(Process proc, CaptureEngine captureEngine)
	{
		process = proc;
		in = new BufferedReader(new InputStreamReader(process.getInputStream()));
		m_captureEngine = captureEngine;
		m_store = DataStore.getInstance();
	}

	protected void finalize() throws Throwable 
	{
		DVScheduler.log("Finalize called on ProcessInputReader object");         
		super.finalize();
	}      

	public void run()
	{
		try
		{
			String line = in.readLine();
			while(line != null)
			{
				if(line.length() > 0)
				{
					commandQue.add(line);
					if ( m_captureEngine.isRecordingStarted(line) || m_captureEngine.isRecordingStopped(line) )
						notifyResponse();
					else if ( (m_captureEngine.isRecordingError(line)) || (m_captureEngine.isPidChange(line)) )
						m_store.setToDoWake();
					DVScheduler.log(line);
				}
				line = in.readLine();
			}

			int code = process.waitFor();
			exitCode = new Integer(code);
			notifyResponse();
			DVScheduler.log("External app exited with code : " + exitCode);
		}
		catch(Exception e)
		{
			DVScheduler.log("Error in ProcessInputReader!");
			e.printStackTrace();            
		}
		DVScheduler.log("ProcessInputReader Thread Exiting");
	}

	public Integer getExitCode()
	{
		return exitCode;
	}

	public synchronized void notifyResponse()
	{
		notify();
	}

	public synchronized void waitResponse()
	{
		try {
			wait(100);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public synchronized boolean isResponse()
	{
		return !commandQue.isEmpty();
	}

	public synchronized String getResponse()
	{
		return commandQue.remove(0);
	}
}