/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.net.*;
import java.util.*;

class RequestObject implements Runnable
{
	private Socket m_server = null;
	private String m_line = "";
	private String m_request = "";
	private HashMap<String, String> m_headers = new HashMap<String, String>();
	private String m_postData = null;

	RequestObject(Socket server)
	{
		this.m_server=server;
		try {
			m_server.setSendBufferSize(65536);
		} catch (SocketException e) {
			e.printStackTrace();
		}
	}

	public void run ()
	{
		try
		{
			// Get input from the client
			InputStream inStream = m_server.getInputStream();
			BufferedReader in = new BufferedReader(new InputStreamReader(inStream));
			OutputStream out = m_server.getOutputStream();

			m_line = in.readLine();
			m_request = m_line;

			while( (m_line != null) && !(m_line.equals("")) )
			{
				m_line = in.readLine();
				int index = m_line.indexOf(": ");
				if(index > 0)
				{
					m_headers.put(m_line.substring(0, index), m_line.substring(index + 2, m_line.length()));
				}
			}

			m_headers.put("RemoteAddress", m_server.getInetAddress().getHostAddress());
			m_headers.put("HostAddress", m_server.getLocalAddress().getHostAddress());
			m_headers.put("HostName", m_server.getLocalAddress().getHostName());

			int dataLength = 0;
			String contLength = (String)m_headers.get("Content-Length");
			if(contLength != null)
			{
				try
				{
					dataLength = Integer.parseInt(contLength);
				}
				catch(Exception e){}
			}

			if(dataLength > 0)
			{
				m_postData = "";
				int totaleRead = 0;
				char dataRead = 0;

				while(totaleRead < dataLength)
				{
					dataRead = (char)in.read();
					if(dataRead != -1)
					{
						m_postData += dataRead;
						totaleRead++;
					}
					else
					{
						break;
					}
				}
				//DVScheduler.log("Post Data:" + postData);
			}

			if (m_request == null)
				DVScheduler.log("Worker RequestObject: null HTTP request");
			else
			{
				Thread.currentThread().setName(m_request == null ? "???" : m_request);

				DataStore store = DataStore.getInstance();
				boolean bUnsecureDownload = store.isAllowUnsecureDownload() && (store.getWebGuiProtocol() != DataStore.WEBGUI_HTTPS);
				HTTPRequest httpRequest = new HTTPRequest(m_request, m_headers, m_postData, out);
				httpRequest.sendResponseData(bUnsecureDownload);
			}
		}
		catch (Exception e)
		{
			DVScheduler.log("Worker RequestObject Exception: " + m_request);
			e.printStackTrace();
		}
		finally
		{
			try
			{
				m_server.close();
			}
			catch(Exception e){}
		}

		//DVScheduler.log("Request Thread Exiting");
	}
}
