/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

class ScanResult
{
	private StringBuffer m_xmlData = null;
	private Vector<Channel> m_scanResult = new Vector<Channel>();
	private int m_frequency = 0;
	private int m_bandwidth = 0;
	private String m_log = "";
	private String m_log2 = "";
	private CaptureEngine m_captureEngine = null;

	public ScanResult(Channel channel, CaptureEngine captureEngine)
	{
		m_frequency = channel.getFrequency();
		m_bandwidth = channel.getBandWidth();
		m_captureEngine = captureEngine;
	}

	public String getLog()
	{
		return m_log;
	}

	public String getLog2()
	{
		return m_log2;
	}

	public int readInput(InputStream inStream) throws Exception
	{
		BufferedReader reader = new BufferedReader(new InputStreamReader(inStream, "UTF-8"));
		m_xmlData = new StringBuffer();

		boolean bStart = false;
		String line = reader.readLine();
		while(line != null)
		{
			if (line.length() > 0)
			{
				if ( !bStart && line.contains("<programs"))
					bStart = true;

				if (bStart)
					m_xmlData.append(line + "\n");

				DVScheduler.log(line);

				if (line.contains("</programs"))
					break;
			}
			line = reader.readLine();
		}

		// Read the rest.
		line = reader.readLine();
		while(line != null)
		{
			if ( m_captureEngine.isCaptureEngineExiting(line))
				break;

			if (line.length() > 0)
			{
				m_log2 += line + "\n";
			}
			line = reader.readLine();
		}

		return m_xmlData.length();
	}   

	public int parseXML() throws Exception
	{
		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();

			ByteArrayInputStream reader = new ByteArrayInputStream(m_xmlData.toString().getBytes("UTF-8"));
			Document doc = docBuilder.parse(reader);   

			NodeList programs = doc.getElementsByTagName("program");

			for(int x = 0; x < programs.getLength(); x++)
			{
				Node program = programs.item(x);

				NodeList nl = program.getChildNodes();   

				String provider = "";
				String name = "";
				String id = "";
				String access = "";
				String description = "";
				int originalNetworkId = -2;
				int transportStreamId = -2;
				int sourceId = -2;

				for(int y = 0; y < nl.getLength(); y++)
				{
					Node item = nl.item(y);

					if(item.getNodeName().equals("provider"))
					{            
						provider = item.getTextContent();
					}
					else if(item.getNodeName().equals("name"))
					{            
						name = item.getTextContent();
					}
					else if(item.getNodeName().equals("id"))
					{            
						id = item.getTextContent();
					}
					else if(item.getNodeName().equals("originalNetworkId"))
					{            
						originalNetworkId = Integer.parseInt(item.getTextContent());
					}
					else if(item.getNodeName().equals("transportStreamId"))
					{            
						transportStreamId = Integer.parseInt(item.getTextContent());
					}
					else if(item.getNodeName().equals("sourceId"))
					{            
						sourceId = Integer.parseInt(item.getTextContent());
					}
					else if(item.getNodeName().equals("access"))
					{            
						access = item.getTextContent();
					}
					else if(item.getNodeName().equals("description"))
					{            
						description = item.getTextContent();
					}
				}

				DVScheduler.log(description);
				if (!m_log.isEmpty())
					m_log += "\n";
				m_log += "Program Name: " + name + ", ID: " + id + "\n";
				if (!provider.isEmpty())
					m_log += "Provider: " + provider + "\n";
				if (originalNetworkId > -2)
					m_log += "Original network ID: " + originalNetworkId + "\n";
				if (transportStreamId > -2)
					m_log += "Transport stream ID: " + transportStreamId + "\n";
				if (sourceId > -2)
					m_log += "Source ID: " + sourceId + "\n";

				Channel chan = null;
				//if(name.length() > 0 && id.length() > 0)
				if (id.length() > 0)
				{
					int idInt = Integer.parseInt(id);
					int accessInt = access.equalsIgnoreCase("FTA") ? Channel.ACCESS_FTA : Channel.ACCESS_CA;
					chan = new Channel(name, m_frequency, m_bandwidth, idInt);
					chan.setDemodulatorName(provider);
					chan.setAccess(accessInt);
					chan.setOriginalNetworkId(originalNetworkId);
					chan.setTransportStreamId(transportStreamId);
					chan.setSourceId(sourceId);
				}

				if(chan != null)
				{
					for(int y = 0; y < nl.getLength(); y++)
					{
						Node item = nl.item(y);

						if ( (item.getNodeName().equals("stream")) || (item.getNodeName().equals("packet")) )
						{ 
							NodeList dataList = item.getChildNodes();

							int streamID = -1;
							int streamType = -1;
							int teletextSubtitlePage = -1;

							for(int z = 0; z < dataList.getLength(); z++)
							{
								Node streamData = dataList.item(z);

								if(streamData.getNodeName().equals("id"))
								{
									streamID = Integer.parseInt(streamData.getTextContent());
								}
								else if(streamData.getNodeName().equals("type"))
								{
									String tp = streamData.getTextContent();
									if ( tp.equalsIgnoreCase("Video MPG") )
										streamType = Channel.TYPE_VIDEO_MPG;
									else if ( tp.equalsIgnoreCase("Video H264") )
										streamType = Channel.TYPE_VIDEO_H264;
									else if ( tp.equalsIgnoreCase("Video") )
										streamType = Channel.TYPE_VIDEO_MPG;
									else if ( tp.equalsIgnoreCase("Audio MPG") )
										streamType = Channel.TYPE_AUDIO_MPG;
									else if ( tp.equalsIgnoreCase("Audio AC3") )
										streamType = Channel.TYPE_AUDIO_AC3;
									else if ( tp.equalsIgnoreCase("Audio AAC") )
										streamType = Channel.TYPE_AUDIO_AAC;
									else if ( tp.equalsIgnoreCase("Teletext") )
										streamType = Channel.TYPE_TELETEXT;									
									else if ( tp.equalsIgnoreCase("PCR") )
										streamType = Channel.TYPE_PCR;									
									else if ( tp.equalsIgnoreCase("PMT") )
										streamType = Channel.TYPE_PMT;									
									else if ( tp.equalsIgnoreCase("Subtitle") )
										streamType = Channel.TYPE_SUBTITLE;									
									else if ( tp.equalsIgnoreCase("Unknown") )
										streamType = Channel.TYPE_UNKNOWN;									
									else
									{
										try
										{
											streamType = Integer.parseInt(tp);
										}
										catch (Exception e)
										{
										}
									}
								}
								else if ( (streamData.getNodeName().equals("pageSubtitle") || streamData.getNodeName().equals("pageSubtitleHearingImpaired")) )
								{
									try
									{
										teletextSubtitlePage = Integer.parseInt(streamData.getTextContent());
									}
									catch (Exception e)
									{
									}
								}
							}

							if (streamID > -1 && streamType == Channel.TYPE_TELETEXT && teletextSubtitlePage > -1)
							{
								// First teletext with subtitle.
								if (chan.getTeletextPid() < 0)
								{
									chan.setTeletextPid(streamID);
									chan.setTeletextSubtitlePage(teletextSubtitlePage);
								}

								int[] streamData = new int[3];
								streamData[0] = streamID;
								streamData[1] = streamType;
								streamData[2] = teletextSubtitlePage;
								chan.addStream(streamData);

								m_log += "Packet ID: " + streamID + ", Type: " + Channel.getPacketType(streamType) + ", Subtitle Page: " + teletextSubtitlePage + "\n";
							}
							else if (streamID > -1 && streamType > -1)
							{
								int[] streamData = new int[2];
								streamData[0] = streamID;
								streamData[1] = streamType;
								chan.addStream(streamData);

								m_log += "Packet ID: " + streamID + ", Type: " + Channel.getPacketType(streamType) + "\n";
							}
						}          
					}

					m_scanResult.add(chan);
				}

				if ( !description.isEmpty() )
					m_log += "Long Description:\n" + description + "\n";
			}
		}
		catch(Exception e)
		{
			m_scanResult = new Vector<Channel>();
			DVScheduler.log("Error parsing channel scan XML.");
			DVScheduler.log(m_xmlData);
			return 0;
		}

		return m_scanResult.size();
	}

	public void addStream(int progID, int streamID, int type)
	{
		int[] streamData = new int[2];
		streamData[0] = streamID;
		streamData[1] = type;

		for(int x = 0; x < m_scanResult.size(); x++)
		{
			Channel chan = (Channel)m_scanResult.get(x);
			if(chan.getProgramID() == progID)
			{
				chan.addStream(streamData);
			}
		}
	}

	public Vector<Channel> getResult()
	{
		return m_scanResult;
	}

}
