/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;
import java.io.*;


public class ScheduleItem implements Serializable, Comparable<ScheduleItem>
{
	private static final long serialVersionUID = -1438453044440839810L;
	private StringBuffer log = new StringBuffer();
	private Date startTime = null;
	private Date stopTime = null;
	private boolean bUserModified = false;
	private int duration = 0;
	private String channel = "";
	private String m_channelRunning = "";
	private int state = 0;
	private String status = "";
	private boolean abort = false;
	private String fileName = "";
	private int type = 0;
	private int m_customDays = 0;
	private String m_captureType = "Capture.deftype";
	long id = 0;
	private boolean autoDeleteable = false;
	private String m_capturePath = "*";
	private String fileNamePattern = "";
	private int keepFor = 30;
	private String postTask = "";
	private GuideItem createdFrom = null;
	private Vector<ScheduleItem> children = new Vector<ScheduleItem>();
	int indexCapture = 0;	// Used when start time is changed in the middle of recording.
	private int m_type2 = 0;
	private HashSet<Integer> captureDevicesUsed = new HashSet<Integer>();
	private String m_clientName = "";
	private String m_hostName = "";
	private long m_tryTime = 0;
	private boolean m_bTrackEpg = false;
	private int m_errorCount = 0;
	private String m_epgAutoAddName = "";

	// Item States
	public static int WAITING = 0;
	public static int SKIPPED = 1;
	public static int RUNNING = 2;
	public static int FINISHED = 3;
	public static int ABORTED = 4;
	public static int ERROR = 5;
	public static int RESTART = 6;
	public static int ABORTED_DELETEFILES = 7;

	// item types
	public static int ONCE = 0;
	public static int DAILY = 1;
	public static int WEEKLY = 2;
	public static int MONTHLY = 3;
	public static int WEEKDAY = 4;
	public static int EPG = 5;
	public static int EPG_SCAN = 6;

	// Item type2
	protected static int ITEMTYPE2_PREVIEWLOCAL = 0x01;
	protected static int ITEMTYPE2_SURFLOCAL = 0x02;
	protected static int ITEMTYPE2_PREVIEWREMOTE = 0x04;
	protected static int ITEMTYPE2_SURFREMOTE = 0x08;
	protected static int ITEMTYPE2_PLAYLIST = 0x10;
	protected static int ITEMTYPE2_NOWAKESYSTEM = 0x20;

	// Custom days
	protected static int CUSTOM_SUN = 0x01;
	protected static int CUSTOM_MON = 0x02;
	protected static int CUSTOM_TUE = 0x04;
	protected static int CUSTOM_WED = 0x08;
	protected static int CUSTOM_THU = 0x10;
	protected static int CUSTOM_FRI = 0x20;
	protected static int CUSTOM_SAT = 0x40;
	protected static int CUSTOM_MAX = 0x80;

	public ScheduleItem(GuideItem item, String chan, String captureType, 
			long itemID, boolean autoDel) throws Exception
			{
		id = itemID;
		this.setCreatedFrom(item);

		startTime = item.getStart();
		stopTime = item.getStop();
		duration = item.getDuration();
		channel = chan;
		state = WAITING;
		type = EPG;
		m_captureType = ((captureType == null) || captureType.isEmpty()) ? "Capture.deftype" : captureType;
		status = "Waiting";
		autoDeleteable = autoDel;

		fileName = checkFileName(item.getName());
			}

	public ScheduleItem(long itemID)
	{
		id = itemID;// new Date().getTime();
	}

	public ScheduleItem()
	{
		id = DataStore.getInstance().m_rand.nextLong();
	}

	public String getPostTask()
	{
		return postTask;
	}

	public void setPostTask(String task) throws Exception
	{
		postTask = ((task == null) || task.equals("*UNDEFINED*")) ? "" : task;
	}

	public String getCapType()
	{
		return m_captureType;
	}

	public String getCapTypeActual()
	{
		if (m_captureType == null || m_captureType.isEmpty() || m_captureType.equals("Capture.deftype"))
		{
			DataStore store = DataStore.getInstance();
			Channel ch = store.getChannel(getRunningChannel());
			String actual = ch == null ? "Capture.deftype" : ch.getCapType();

			if (actual == null || actual.isEmpty() || actual.equals("Capture.deftype"))
				return store.getProperty("Capture.deftype", "TS");

			return actual;
		}

		return m_captureType;
	}

	public String getCaptureCommandFile()
	{
		String extension = ".ini";

		if (isPreviewLocal())
			extension = ".pre";
		else if (isPreviewRemote())
			extension = ".prr";
		else if (isSurfLocal())
			extension = ".suf";
		else if (isSurfRemote())
			extension = ".sur";
		else if (isPlaylist())
			extension = ".plt";

		return getCapTypeActual() + extension;
	}

	public void setCapType(String captureType) throws Exception
	{
		m_captureType = ((captureType == null) || captureType.isEmpty() || captureType.equals("*UNDEFINED*")) ? "Capture.deftype" : captureType;
	}

	public boolean equals(Object item)
	{
		ScheduleItem compTo = (ScheduleItem)item;

		if(compTo.getStart().getTime() == this.getStart().getTime() &&
				compTo.getDuration() == this.getDuration() && 
				compTo.getChannel().equals(this.getChannel()) && 
				compTo.getCapType().equals(this.getCapType()))
		{
			return true;
		}

		return false;
	}

	public boolean equalsEPG(ScheduleItem compTo)
	{
		if (compTo.getStartEPG() == null || compTo.getStopEPG() == null || 
				getStartEPG() == null || getStopEPG() == null)
			return false;

		if(compTo.getStartEPG().getTime() == this.getStartEPG().getTime() &&
				compTo.getStopEPG().getTime() == this.getStopEPG().getTime() && 
				compTo.getChannel().equals(this.getChannel()))
		{
			// Less stringent check.
			boolean overlap = false;

			if(compTo.getStart().getTime() >= getStart().getTime() && compTo.getStart().getTime() < getStop().getTime())
				overlap = true;

			else if(compTo.getStop().getTime() > getStart().getTime() && compTo.getStop().getTime() <= getStop().getTime())
				overlap = true;

			else if(compTo.getStart().getTime() <= getStart().getTime() && compTo.getStop().getTime() >= getStop().getTime())
				overlap = true;

			return overlap;

			// More stringent check.
			//return (compTo.getStart().getTime() <= getStart().getTime() && compTo.getStop().getTime() >= getStop().getTime());
		}

		return false;
	}

	public int compareTo(ScheduleItem comTo)
	{
		if(comTo == null)
			return -1;

		long comStart = comTo.getStart().getTime();
		long thisStart = getStart().getTime();

		int result = 0;
		if(comStart > thisStart)
			result = -1;
		else if(comStart < thisStart)
			result = +1;

		// DVScheduler.log("Comp Result : " + result);
		return result;
	}

	public void setType(int ty) throws Exception
	{
		type = ty;
	}

	public int getType()
	{
		return type;
	}

	public void setCustomDays(int customDays) throws Exception
	{
		m_customDays = customDays;
	}

	public int getCustomDays()
	{
		return m_customDays;
	}

	public String getTypeStr()
	{
		if(type == ONCE)
			return "Once";
		else if(type == DAILY)
			return "Daily";
		else if(type == MONTHLY)
			return "Monthly";
		else if(type == WEEKDAY)
			return "Week Day";
		else if(type == EPG)
			return "EPG";
		else if(type == WEEKLY)
		{
			if ( (m_customDays <= 0) || (m_customDays >= CUSTOM_MAX) )
				return "Weekly";

			String text = "";

			if ( (m_customDays & CUSTOM_SUN) > 0 )
				text += "Sun ";
			if ( (m_customDays & CUSTOM_MON) > 0 )
				text += "Mon ";
			if ( (m_customDays & CUSTOM_TUE) > 0 )
				text += "Tue ";
			if ( (m_customDays & CUSTOM_WED) > 0 )
				text += "Wed ";
			if ( (m_customDays & CUSTOM_THU) > 0 )
				text += "Thu ";
			if ( (m_customDays & CUSTOM_FRI) > 0 )
				text += "Fri ";
			if ( (m_customDays & CUSTOM_SAT) > 0 )
				text += "Sat ";

			return text.trim();
		}
		else if(type == EPG_SCAN)
			return "EPG Scan";
		else
			return "?" + type + "?";
	}

	public boolean skipToNext() throws Exception
	{
		if(this.getState() != ScheduleItem.WAITING)
			return false;

		Calendar start = Calendar.getInstance();
		start.setTime(this.getStart());

		// Daily, Just add one day
		if ( (type == DAILY) || (type == EPG_SCAN) )
		{
			start.add(Calendar.DATE, 1);
		}
		// Monthly, Just add one month
		else if(type == MONTHLY)
		{
			start.add(Calendar.MONTH, 1);
		}
		// Week Day, increment to the next week day
		else if(type == WEEKDAY)
		{
			start.add(Calendar.DATE, 1);

			int dayOfWeek = start.get(Calendar.DAY_OF_WEEK);
			while (dayOfWeek < Calendar.MONDAY || dayOfWeek > Calendar.FRIDAY)
			{
				start.add(Calendar.DATE, 1);
				dayOfWeek = start.get(Calendar.DAY_OF_WEEK);
			}
		}
		else if(type == WEEKLY)
		{
			if ( (m_customDays <= 0) || (m_customDays >= CUSTOM_MAX) )
			{
				start.add(Calendar.DATE, 7);
			}
			else
			{
				while (true)
				{
					start.add(Calendar.DATE, 1);
					int dayOfWeek = start.get(Calendar.DAY_OF_WEEK);

					if ( (dayOfWeek == Calendar.SUNDAY) && ((m_customDays & CUSTOM_SUN) > 0) )
						break;
					if ( (dayOfWeek == Calendar.MONDAY) && ((m_customDays & CUSTOM_MON) > 0) )
						break;
					if ( (dayOfWeek == Calendar.TUESDAY) && ((m_customDays & CUSTOM_TUE) > 0) )
						break;
					if ( (dayOfWeek == Calendar.WEDNESDAY) && ((m_customDays & CUSTOM_WED) > 0) )
						break;
					if ( (dayOfWeek == Calendar.THURSDAY) && ((m_customDays & CUSTOM_THU) > 0) )
						break;
					if ( (dayOfWeek == Calendar.FRIDAY) && ((m_customDays & CUSTOM_FRI) > 0) )
						break;
					if ( (dayOfWeek == Calendar.SATURDAY) && ((m_customDays & CUSTOM_SAT) > 0) )
						break;
				}
			}
		}

		this.setStart(start);
		resetCaptureDevicesUsed();
		resetErrorCount();
		this.setState(ScheduleItem.WAITING);
		this.setStatus("Waiting");
		this.resetAbort();
		this.log("Skipping schedule item forward to " + start.getTime().toString());

		return true;
	}

	public ScheduleItem createNextInstance(Calendar now, Random rand) throws Exception
	{
		Calendar start = Calendar.getInstance();
		start.setTime(this.getStart());

		// Daily, Just change to todays date and leave time along
		if ( (type == DAILY) || (type == EPG_SCAN) )
		{
			while (start.before(now))
			{
				start.add(Calendar.DATE, 1);
			}
		}

		// Monthly, if it has already occured set to next month
		else if(type == MONTHLY)
		{
			while (start.before(now))
			{
				start.add(Calendar.MONTH, 1);
			}
		}

		// Week Day, if it has already occured set to next week day
		else if(type == WEEKDAY)
		{
			int dayOfWeek = start.get(Calendar.DAY_OF_WEEK);
			while (start.before(now) || 
					dayOfWeek < Calendar.MONDAY || dayOfWeek > Calendar.FRIDAY)
			{
				start.add(Calendar.DATE, 1);
				dayOfWeek = start.get(Calendar.DAY_OF_WEEK);
			}
		}

		// Weekly, if it has alredy occured then set to next date.
		else if(type == WEEKLY)
		{
			if ( (m_customDays <= 0) || (m_customDays >= CUSTOM_MAX) )
			{
				while (start.before(now))
				{
					start.add(Calendar.DATE, 7);
				}
			}
			else
			{
				while (true)
				{
					if (start.before(now))
					{
						start.add(Calendar.DATE, 1);
						continue;
					}

					int dayOfWeek = start.get(Calendar.DAY_OF_WEEK);

					if ( (dayOfWeek == Calendar.SUNDAY) && ((m_customDays & CUSTOM_SUN) > 0) )
						break;
					if ( (dayOfWeek == Calendar.MONDAY) && ((m_customDays & CUSTOM_MON) > 0) )
						break;
					if ( (dayOfWeek == Calendar.TUESDAY) && ((m_customDays & CUSTOM_TUE) > 0) )
						break;
					if ( (dayOfWeek == Calendar.WEDNESDAY) && ((m_customDays & CUSTOM_WED) > 0) )
						break;
					if ( (dayOfWeek == Calendar.THURSDAY) && ((m_customDays & CUSTOM_THU) > 0) )
						break;
					if ( (dayOfWeek == Calendar.FRIDAY) && ((m_customDays & CUSTOM_FRI) > 0) )
						break;
					if ( (dayOfWeek == Calendar.SATURDAY) && ((m_customDays & CUSTOM_SAT) > 0) )
						break;

					start.add(Calendar.DATE, 1);
				}
			}
		}		

		ScheduleItem newInstance = new ScheduleItem(rand.nextLong());
		this.copyToSchedule(newInstance);

		newInstance.setStart(start);
		newInstance.setState(ScheduleItem.WAITING);
		newInstance.setStatus("Waiting");
		newInstance.resetAbort();

		newInstance.resetLog();
		newInstance.log("Repeating schedule item re-created (" + newInstance.getType() + ") : "
				+ start.getTime().toString());

		this.setType(ScheduleItem.ONCE);

		return newInstance;
	}

	public void setName(String name) throws Exception
	{
		//fileName = checkFileName(m_name);
		fileName = Formatter.formatDirectory(name);
	}

	public void setCreatedFrom(GuideItem item) throws Exception
	{
		if(item != null)
			createdFrom = item.clone();
	}

	public GuideItem getCreatedFrom()
	{
		return createdFrom;
	}

	public String getName()
	{
		return fileName;
	}

	public CaptureFile getFileName()
	{
		CaptureFile cap = new CaptureFile();

		Calendar cal = Calendar.getInstance();
		cal.setTime(this.getStart());

		String fileName = this.getFilePattern();

		fileName = fileName.replace("%y", addZero(cal.get(Calendar.YEAR)));
		fileName = fileName.replace("%m", addZero((cal.get(Calendar.MONTH) + 1)));
		fileName = fileName.replace("%d", addZero(cal.get(Calendar.DATE)));
		fileName = fileName.replace("%h", addZero(cal.get(Calendar.HOUR_OF_DAY)));
		fileName = fileName.replace("%M", addZero(cal.get(Calendar.MINUTE)));
		fileName = fileName.replace("%a", m_epgAutoAddName);

		String dayOfWeek = "";
		try
		{
			dayOfWeek = (String) DataStore.getInstance().dayName.get(new Integer(
					cal.get(Calendar.DAY_OF_WEEK)));
		}
		catch (Exception e){}
		fileName = fileName.replace("%D", dayOfWeek);

		// item name
		if(this.getName().length() > 0)
		{
			//fileName = fileName.replace("%n", checkFileName(this.getName()));
			fileName = fileName.replace("%n", Formatter.formatDirectory(getName()));
		}
		else
		{
			fileName = fileName.replace("%n", "");
		}

		// if this item has a Created From and %s then add that
		if(createdFrom != null)
		{
			fileName = fileName.replace("%s", checkFileName(createdFrom.getSubName()));
			fileName = fileName.replace("%t", checkFileName(createdFrom.getFirstCategory()));	// First category if has multiple categories.
		}
		else
		{
			fileName = fileName.replace("%s", "");
			fileName = fileName.replace("%t", "");
		}

		fileName = fileName.replace("%c", checkFileName(this.getChannel()));

		//DVScheduler.log("|" + fileName.trim() + "|");

		//remove double spaces
		//fileName = fileName.replaceAll("(\\ )+", " ");

		fileName = Formatter.formatDirectory(fileName);
		String finalName = fileName.trim();
		//DVScheduler.log("|" + fileName.trim() + "|");
		if (finalName.isEmpty())
			finalName = "Recording";
		cap.setName(finalName);

		return cap;
	}

	public void abort()
	{
		abort = true;
	}

	public boolean isAborted()
	{
		return abort;
	}

	public void resetAbort()
	{
		abort = false;
	}

	public void setDuration(int dur) throws Exception
	{
		duration = dur;

		if(startTime != null)
		{
			Calendar stop = Calendar.getInstance();
			stop.setTime(startTime);
			stop.add(Calendar.MINUTE, duration);
			stopTime = stop.getTime();
		}
	}

	public int getDuration()
	{
		return duration;
	}

	public Date getStop()
	{
		return stopTime;
	}

	public int getStopHour()
	{
		Calendar cal = Calendar.getInstance();
		cal.setTime(getStop());
		return cal.get(Calendar.HOUR_OF_DAY);
	}

	public int getStopMinute()
	{
		Calendar cal = Calendar.getInstance();
		cal.setTime(getStop());
		return cal.get(Calendar.MINUTE);
	}

	public Date getStartEPG()
	{
		return getCreatedFrom() == null ? null : getCreatedFrom().getStart();
	}

	public Date getStopEPG()
	{
		return getCreatedFrom() == null ? null : getCreatedFrom().getStop();
	}

	public boolean isUserModified()
	{
		return bUserModified;
	}

	public void setUserModified()
	{
		bUserModified = true;
	}

	public void setChannel(String ch) throws Exception
	{
		channel = ch;
	}

	public String getChannel()
	{
		return channel;
	}

	public void setState(int st)
	{
		state = st;
	}

	public int getState()
	{
		return state;
	}

	public void setStatus(String s)
	{
		status = s;
	}

	public String getStatus()
	{
		return status;
	}

	public void setStart(Calendar cal) throws Exception
	{
		startTime = cal.getTime();

		if(duration > 0)
		{
			Calendar stop = Calendar.getInstance();
			stop.setTime(startTime);
			stop.add(Calendar.MINUTE, duration);
			stopTime = stop.getTime();
		}
	}

	public void setStart(long time) throws Exception
	{
		if (startTime == null)
			startTime = new Date();

		startTime.setTime(time);

		if(duration > 0)
		{
			Calendar stop = Calendar.getInstance();
			stop.setTime(startTime);
			stop.add(Calendar.MINUTE, duration);
			stopTime = stop.getTime();
		}
	}

	public Date getStart()
	{
		return startTime;
	}

	public int getStartHour()
	{
		Calendar cal = Calendar.getInstance();
		cal.setTime(getStart());
		return cal.get(Calendar.HOUR_OF_DAY);
	}

	public int getStartMinute()
	{
		Calendar cal = Calendar.getInstance();
		cal.setTime(getStart());
		return cal.get(Calendar.MINUTE);
	}

	public String toString()
	{
		return Long.toString(id);
	}

	public long getID()
	{
		return id;
	}

	public void log(String mes)
	{
		Date date = new Date();
		log.append(date.toString() + " : " + mes + "\n");
		DVScheduler.log("SID " + getID() + ": " + mes, date);
	}

	public String getLog()
	{
		return log.toString();
	}

	public void resetLog()
	{
		log = new StringBuffer();
	}

	private String addZero(int input)
	{
		if(input < 10)
			return (String) ("0" + input);
		else
			return (new Integer(input)).toString();
	}

	private String checkFileName(String name)
	{
		return Formatter.formatFilename(name);
		//return Formatter.formatDirectory(name);
		/*
		name = name.trim();

		char[] notAllowed = {'\\','/',':','*','?','\"','<','>','|'};
		char[] replaceWith = {'-', '-','-','#','#','\'','-','-','!'};

		for(int x = 0; x < notAllowed.length; x++)
		{
			name = name.replace(notAllowed[x], replaceWith[x]);
		}

		StringBuffer finalName = new StringBuffer(256);
		for (int x = 0; x < name.length(); x++)
		{
			char charAt = name.charAt(x);

			if (charAt >= ' ' && charAt != 127 && charAt <= 255)
				finalName.append(charAt);
			else
				finalName.append('-');
		}

		return finalName.toString();
		 */
	}

	public boolean isOverlapping(ScheduleItem checkItem)
	{
		// Does it start in our active time period.
		// it is allowed to start in the same min the other one ends
		if(checkItem.getStart().getTime() >= this.getStart().getTime()
				&& checkItem.getStart().getTime() < this.getStop().getTime())
			return true;

		// Does it end in our active time period.
		// it is allowed to end in the same min the other one starts
		if(checkItem.getStop().getTime() > this.getStart().getTime()
				&& checkItem.getStop().getTime() <= this.getStop().getTime())
			return true;

		// Does it start before and end after (completly enveloping) our active
		// time period.
		if(checkItem.getStart().getTime() <= this.getStart().getTime()
				&& checkItem.getStop().getTime() >= this.getStop().getTime())
			return true;

		return false;
	}

	public boolean isAutoDeletable()
	{
		return autoDeleteable;
	}

	public void setAutoDeletable(boolean del) throws Exception
	{
		autoDeleteable = del;
	}

	public String getFilePattern()
	{
		return fileNamePattern;
	}

	public String getCapturePath()
	{
		return m_capturePath;
	}

	public String getCapturePathActual()
	{
		if (m_capturePath == null || m_capturePath.equals("*"))
		{
			DataStore store = DataStore.getInstance();
			Channel ch = store.getChannel(getRunningChannel());
			String actual = ch == null ? "*" : ch.getCapturePath();

			if (actual == null || actual.equals("*"))
			{
				return "*";
			}

			return actual;
		}

		return m_capturePath;
	}

	public void setFilePattern(String pattern) throws Exception
	{
		fileNamePattern = ( (pattern == null) || pattern.isEmpty() || pattern.equals("*UNDEFINED*") ) ? "%n" : pattern;
	}

	public void setCapturePath(String capturePath) throws Exception
	{
		m_capturePath = ( (capturePath == null) || capturePath.equals("*UNDEFINED*") ) ? "*" : capturePath;
	}

	public void setKeepFor(int keep) throws Exception
	{
		keepFor = keep;
	}

	public int getKeepFor()
	{
		return keepFor;
	}

	public Vector<ScheduleItem> getChildren()
	{
		return children;
	}

	public void addChildren(Vector<ScheduleItem> childs)
	{
		children.addAll(childs);
	}

	public void addChild(ScheduleItem child)
	{
		children.add(child);
	}

	public void copyToSchedule(ScheduleItem to) throws Exception
	{
		Calendar start = Calendar.getInstance();
		start.setTime(this.getStart());
		to.setStart(start);

		to.setDuration(this.getDuration());
		to.setCapType(this.getCapType());
		to.setType(this.getType());
		to.setCustomDays(this.getCustomDays());
		to.setPostTask(this.getPostTask());
		to.setChannel(this.getChannel());  
		to.setFilePattern(this.getFilePattern());
		to.setName(this.getName());
		to.setCreatedFrom(this.getCreatedFrom());
		to.setKeepFor(this.getKeepFor());
		to.setAutoDeletable(this.isAutoDeletable());
		to.setState(this.getState());
		to.setStatus(this.getStatus());
		to.m_type2 = m_type2;
		to.setCapturePath(this.getCapturePath());
		to.setTrackEpg(this.isTrackEpg());
		to.setEpgAutoAddName(getEpgAutoAddName());

		if(this.isAborted())
			to.abort();
	}

	public void setType2(int type2)
	{
		m_type2 = type2;
	}

	public int getType2()
	{
		return m_type2;
	}

	public void setPreviewLocal()
	{
		m_type2 |= ITEMTYPE2_PREVIEWLOCAL;
	}

	public boolean isPreviewLocal()
	{
		return (m_type2 & ITEMTYPE2_PREVIEWLOCAL) > 0;
	}

	public void setPreviewRemote()
	{
		m_type2 |= ITEMTYPE2_PREVIEWREMOTE;
	}

	public boolean isPreviewRemote()
	{
		return (m_type2 & ITEMTYPE2_PREVIEWREMOTE) > 0;
	}

	public boolean isPreview()
	{
		return ( isPreviewLocal() || isPreviewRemote() );
	}

	public void setPlaylist()
	{
		m_type2 |= ITEMTYPE2_PLAYLIST;
	}

	public boolean isPlaylist()
	{
		return (m_type2 & ITEMTYPE2_PLAYLIST) > 0;
	}

	public void setSurfLocal()
	{
		m_type2 |= ITEMTYPE2_SURFLOCAL;
	}

	public boolean isSurfLocal()
	{
		return (m_type2 & ITEMTYPE2_SURFLOCAL) > 0;
	}

	public void setSurfRemote()
	{
		m_type2 |= ITEMTYPE2_SURFREMOTE;
	}

	public boolean isSurfRemote()
	{
		return (m_type2 & ITEMTYPE2_SURFREMOTE) > 0;
	}

	public boolean isSurf()
	{
		return ( isSurfLocal() || isSurfRemote() );
	}

	public boolean isLive()
	{
		return ( (isPreview() || isSurf() || isPlaylist()) && isAutoDeletable() );
	}

	public void setWakeSystem(boolean bWakeSystem)
	{
		if (!bWakeSystem)
			m_type2 |= ITEMTYPE2_NOWAKESYSTEM;
		else
			m_type2 &= ~ITEMTYPE2_NOWAKESYSTEM;
	}

	public boolean isWakeSystem()
	{
		return (m_type2 & ITEMTYPE2_NOWAKESYSTEM) == 0;
	}

	public void addCaptureDeviceUsed(int deviceID)
	{
		captureDevicesUsed.add(deviceID);
	}

	public boolean isCaptureDeviceUsed(int deviceID)
	{
		return captureDevicesUsed.contains(deviceID);
	}

	public void resetCaptureDevicesUsed()
	{
		captureDevicesUsed.clear();
	}

	public void incrementErrorCount()
	{
		m_errorCount++;
	}

	public int getErrorCount()
	{
		return m_errorCount;
	}

	public void resetErrorCount()
	{
		m_errorCount = 0;
	}

	public void setClientName(String clientName)
	{
		m_clientName = clientName;
	}

	public String getClientName()
	{
		return m_clientName;
	}

	// Should be full name including port (e.g. 127.0.0.1:8007).
	public void setHostName(String hostName)
	{
		m_hostName = hostName;
	}

	public String getHostName()
	{
		int sep = m_hostName.lastIndexOf(":");
		return (sep > -1) ? m_hostName.substring(0, sep) : m_hostName;
	}

	public void setNames(Map<String, String> headers)
	{
		String remoteAddress = headers.get("RemoteAddress");
		if ( remoteAddress.equals("127.0.0.1") || remoteAddress.equals("0:0:0:0:0:0:0:1") )
			remoteAddress = "localhost";
		setClientName(remoteAddress);
		setHostName(headers.get("Host"));
	}

	public String getRunningChannel()
	{
		// null or isEmpty() should not happen - means there's a bug somewhere.
		return ( (m_channelRunning == null) || m_channelRunning.isEmpty() ) ? getChannel() : m_channelRunning;
	}

	public void setRunningChannel(String ch)
	{
		m_channelRunning = ch;
	}

	public void setRunningStatus(Date now)
	{
		if (getState() != ScheduleItem.RUNNING)
			return;

		Date start = getStart();
		Date stop = getStop();
		long totalTime = stop.getTime() - start.getTime();
		long perDone = (long) (((double) (now.getTime() - start.getTime()) / (double) totalTime) * 100.00);
		int leftToGo = (int)((stop.getTime() - now.getTime()) / (1000 * 60));
		setStatus("Running " + perDone + "% (" + leftToGo +"min)");
	}

	public synchronized void notifyResponse()
	{
		notify();
	}

	public long getTryTime()
	{
		return m_tryTime;
	}

	public void setTryTime(long tryTime)
	{
		m_tryTime = tryTime;
	}

	public synchronized void waitResponse()
	{
		try {
			wait(100);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	public static long getSerialVersionUid()
	{
		return serialVersionUID;
	}

	public boolean isTrackEpg()
	{
		return m_bTrackEpg;
	}

	public void setTrackEpg(boolean bTrackEpg)
	{
		m_bTrackEpg = bTrackEpg;
	}

	public void setEpgAutoAddName(String name)
	{
		m_epgAutoAddName = name;
	}

	public String getEpgAutoAddName()
	{
		return m_epgAutoAddName;
	}

	public static int getOverlap(long t1start, long t1stop, long t2start, long t2stop)
	{
		long start = t2start > t1start ? t2start : t1start;
		long stop = t2stop < t1stop ? t2stop : t1stop;

		return (int)(stop - start);
	}

	private void readObject(java.io.ObjectInputStream in)
			throws IOException, ClassNotFoundException
			{
		in.defaultReadObject();

		if (log == null)
			log = new StringBuffer();
		if (startTime == null)
			startTime = new Date();
		if (stopTime == null)
			startTime = new Date();
		if (channel == null)
			channel = "";
		if (m_channelRunning == null)
			m_channelRunning = "";
		if (status == null)
			status = "";
		if (fileName == null)
			fileName = "";
		if (m_captureType == null)
			m_captureType = "Capture.deftype";
		if ( autoDeleteable && (keepFor <= 0) )
			keepFor = 30;
		if (m_capturePath == null)
			m_capturePath = "*";
		if (fileNamePattern == null)
			fileNamePattern = "";
		if (postTask == null)
			postTask = "";
		if (children == null)
			children = new Vector<ScheduleItem>();
		if (captureDevicesUsed == null)
			captureDevicesUsed = new HashSet<Integer>();
		if (m_clientName == null)
			m_clientName = "";
		if (m_hostName == null)
			m_hostName = "";
		if (m_epgAutoAddName == null)
			m_epgAutoAddName = "";
			}

	public String replaceKeywords(String text)
	{
		Calendar cal = Calendar.getInstance();
		cal.setTime(getStart());
		text = text.replace("$year", Integer.toString(cal.get(Calendar.YEAR)));
		String month = Integer.toString(cal.get(Calendar.MONTH) + 1);
		text = text.replace("$month2", (month.length() < 2 ? "0" + month : month));
		text = text.replace("$month", month);
		String day = Integer.toString(cal.get(Calendar.DAY_OF_MONTH));
		text = text.replace("$day2", (day.length() < 2 ? "0" + day : day));
		text = text.replace("$day", day);
		String calValue = Integer.toString(cal.get(Calendar.HOUR_OF_DAY));
		text = text.replace("$hour2", (calValue.length() < 2 ? "0" + calValue : calValue));
		text = text.replace("$hour", calValue);
		calValue = Integer.toString(cal.get(Calendar.MINUTE));
		text = text.replace("$minute2", (calValue.length() < 2 ? "0" + calValue : calValue));
		text = text.replace("$minute", calValue);

		text = text.replace("$rid", toString());

		String text2 = getName();
		text = text.replace("$schedulename", text2);
		try
		{
			int number = Integer.parseInt(text2);
			if ( (number > 0) && (number <= 65535) )
				text = text.replace("$scheduleport", Integer.toString(number));
		}
		catch (Exception e) {}

		text = text.replace("$clientname", m_clientName);
		
		text = text.replace("$serverpath", m_hostName);
		DataStore store = DataStore.getInstance();
		int sep = m_hostName.lastIndexOf(":");
		String serverName = (sep > -1) ? m_hostName.substring(0, sep) : m_hostName;
		text = text.replace("$servername", serverName);
		text = text.replace("$serverurl", (store.getWebGuiProtocol() == DataStore.WEBGUI_HTTPS) ? "https://" + serverName + ":" + store.getHttpsPort() : "http://" + serverName + ":" + DVScheduler.getHttpPort());
		
		text = text.replace("$totalminute", new Integer(getDuration()).toString());
		text = text.replace("$totalsecond", new Integer(getDuration()*60).toString());
		String capTypeActual = getCapTypeActual();	// May not work on channel group (e.g. keyword on pre-capture task, as actual channel hasn't been determined yet).
		text = text.replace("$capturetype", capTypeActual); 
		return text.replace("$capType", capTypeActual);
	}
}