/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.Date;

public class TaskItem
{
	private boolean m_autoRemove = false;
	private int m_delay = 0;

	private boolean m_bWhenNotCapturing = false;
	private int m_maxConcurrentTask = 0;
	private int m_minIdleTime = 0;
	private String m_formattedCommand = "";
	private ScheduleItem m_scheduleItem = null;
	private Date m_created = null;
	private boolean m_allowSuspend = false;

	public TaskItem(boolean autoRemove, int delay, boolean bWhenNotCapturing, int maxConcurrentTask, int minIdleTime, String formattedCommand, ScheduleItem scheduleItem)
	{
		m_autoRemove = autoRemove;
		m_delay = delay;
		m_bWhenNotCapturing = bWhenNotCapturing;
		m_maxConcurrentTask = maxConcurrentTask;
		m_minIdleTime = minIdleTime;
		m_formattedCommand = formattedCommand;
		m_scheduleItem = scheduleItem;
		m_created = new Date();
	}
	
	public TaskItem(TaskCommand taskCommand, String formattedCommand, ScheduleItem scheduleItem)
	{
		m_autoRemove = taskCommand.getAutoRemove();
		m_delay = taskCommand.getDelay();
		m_bWhenNotCapturing = taskCommand.isWhenNotCapturing();
		m_maxConcurrentTask = taskCommand.getMaxConcurrentTask();
		m_minIdleTime = taskCommand.getMinIdleTime();
		m_formattedCommand = formattedCommand;
		m_scheduleItem = scheduleItem;
		m_created = new Date();
		m_allowSuspend = taskCommand.isAllowSuspend();
	}

	public boolean getAutoRemove()
	{
		return m_autoRemove;
	}

	public int getDelay()
	{
		return m_delay;
	}

	public boolean isWhenNotCapturing()
	{
		return m_bWhenNotCapturing;
	}

	public int getMaxConcurrentTask()
	{
		return m_maxConcurrentTask;
	}

	public int getMinIdleTime()
	{
		return m_minIdleTime;
	}

	public boolean isConditional()
	{
		return ( isWhenNotCapturing() || (getMaxConcurrentTask() > 0) || (getMinIdleTime() > 0) );
	}

	public ScheduleItem getScheduleItem()
	{
		return m_scheduleItem;
	}

	public String getFormattedCommand()
	{
		return m_formattedCommand;
	}
	
	public Date getCreationDate()
	{
		return m_created;
	}
	
	public boolean isAllowSuspend()
	{
		return m_allowSuspend;
	}
}