/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;

public class TaskItemThread implements Runnable
{
	public CommandWaitThread command = null;
	StringBuffer control = new StringBuffer();
	String commandLine = "";
	boolean autoRemove = false;
	String itemKey = null;
	Date m_created = null;
	Date m_started = null;
	Date m_finished = null;
	public int m_streamPort = -1;
	public boolean m_allowSuspend = false;

	public TaskItemThread(String key, CommandWaitThread task, String cmd, boolean autoRem, Date created)
	{
		command = task;
		commandLine = cmd;
		autoRemove = autoRem;
		itemKey = key;

		m_created = created;
		m_started = new Date();

		control.append("Running Command:\n(" + commandLine + ") AR=" + autoRemove + "<br><br>");
		//DVScheduler.log("Running Command:\n(" + commandLine + ") AR=" + autoRemove + "\n\n");
	}

	public TaskItemThread(String key, CommandWaitThread task, String cmd, boolean autoRem)
	{
		command = task;
		commandLine = cmd;
		autoRemove = autoRem;
		itemKey = key;

		m_created = new Date();
		m_started = new Date();

		control.append("Running Command:\n(" + commandLine + ") AR=" + autoRemove + "<br><br>");
		//DVScheduler.log("Running Command:\n(" + commandLine + ") AR=" + autoRemove + "\n\n");
	}

	public Date getCreationDate()
	{
		return m_created;
	}

	public Date getStartedDate()
	{
		return m_started;
	}

	public Date getFinishedDate()
	{
		return m_finished;
	}

	public void killNow()
	{
		control.append("\nKilling Command\n");
		DVScheduler.log("Killing Command");
		command.stop();
	}

	public String getOutput()
	{
		return command.getInputStream();
	}

	public String getError()
	{
		return command.getErrorStream();
	}   

	public String getControl()
	{
		return control.toString();
	}    

	public int getDelayLeft()
	{
		return command.getDelay();
	}

	public void run()
	{

		try
		{  
			if(command.isDelaying())
				control.append("Command is delaying, it will run soon<br>");

			while(command.isDelaying())
			{
				Thread.sleep(1000);
			}

			control.append("Delay over, command now running<br>");

			int reads = 0;
			while(true)
			{
				if(command.isFinished() && reads++ > 0)
				{
					break;
				}

				Thread.sleep(2000);
			}
		}
		catch(Exception e)
		{
			DVScheduler.log("Exception, killing command");
			e.printStackTrace();
			command.stop();
		}      

		try
		{
			ThreadLock.getInstance().getLock();
			try
			{
				if (autoRemove)
					DVScheduler.log("Auto Removing task from task list.");
				DataStore store = DataStore.getInstance();
				store.m_runningTaskList.remove(itemKey);
				if (autoRemove == false)
				{
					int maxLine = store.getPropertyInt("log.task.maxline", 100);
					if (maxLine < 1)
						maxLine = 1;
					if ( store.m_finishedTaskList.size() >= maxLine )
						store.m_finishedTaskList.remove(0);
					store.m_finishedTaskList.add(this);
				}
			}
			finally
			{
				ThreadLock.getInstance().releaseLock();
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

		m_finished = new Date();
		control.append("External app exited with : " + command.getExitCode());
		control.append("\nCommand finished");
		DVScheduler.log("External app exited with : " + command.getExitCode());
	}
}
