/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.*;
import java.util.*;

class TaskManagementDataRes extends HTTPResponse
{
	public TaskManagementDataRes(Map<String, String> headers, HTTPurl urlData, OutputStream outStream) throws Exception
	{
		super(headers, urlData, outStream);
	}

	public void getResponse() throws Exception
	{
		String action = m_urlData.getParameter("action");

		if("01".equals(action))
		{
			m_outStream.write(showTaskOutput());
			return;
		}
		else if("02".equals(action))
		{
			m_outStream.write(killTask());
			return;
		}
		else if("03".equals(action))
		{
			m_outStream.write(removeTask());
			return;
		}
		else if("04".equals(action))
		{
			m_outStream.write(removeFinished());
			return;
		}      
		else if("05".equals(action))
		{
			m_outStream.write(removeQueuedTask());
			return;
		}      
		else if("06".equals(action))
		{
			m_outStream.write(showFinishedTaskOutput());
			return;
		}      
		else if("07".equals(action))
		{
			m_outStream.write(removeFinishedTask());
			return;
		}      

		m_outStream.write(showTaskList());
	}
	
	private byte[] removeFinished() throws Exception
	{
		ThreadLock.getInstance().getLock();
		try
		{
			m_store.m_finishedTaskList.clear();
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return out.toString().getBytes();      
	}

	private byte[] killTask() throws Exception
	{
		String dateID = m_urlData.getParameter("id");

		ThreadLock.getInstance().getLock();
		try
		{
			TaskItemThread item = (TaskItemThread)m_store.m_runningTaskList.get(dateID);

			if(item != null)
			{
				item.killNow();
			}
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return out.toString().getBytes();
	}

	private byte[] removeTask() throws Exception
	{
		String id = m_urlData.getParameter("id");

		ThreadLock.getInstance().getLock();
		try
		{
			m_store.m_runningTaskList.remove(id);
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return out.toString().getBytes();
	}

	private byte[] showTaskOutput() throws Exception
	{
		PageTemplate template = new PageTemplate(m_headers, m_urlData, "TaskOutPut.html");

		String id = m_urlData.getParameter("id");
		ThreadLock.getInstance().getLock();
		try
		{
			TaskItemThread item = (TaskItemThread)m_store.m_runningTaskList.get(id);

			if(item != null)
			{
				template.replace("$taskControl", item.getControl());
				template.replace("$taskOutput", item.getOutput());
				template.replace("$taskError", item.getError());
			}
			else
			{
				StringBuffer out = new StringBuffer(256);
				out.append("HTTP/1.0 302 Moved Temporarily\n");
				out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");
				return out.toString().getBytes();
			}
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		return template.getPageBytes();
	}
	
	private byte[] showFinishedTaskOutput() throws Exception
	{
		PageTemplate template = new PageTemplate(m_headers, m_urlData, "TaskOutPut.html");

		int id = Integer.parseInt(m_urlData.getParameter("id"));
		ThreadLock.getInstance().getLock();
		try
		{
			TaskItemThread item = (TaskItemThread)m_store.m_finishedTaskList.get(id);

			if(item != null)
			{
				template.replace("$taskControl", item.getControl());
				template.replace("$taskOutput", item.getOutput());
				template.replace("$taskError", item.getError());
			}
			else
			{
				StringBuffer out = new StringBuffer(256);
				out.append("HTTP/1.0 302 Moved Temporarily\n");
				out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");
				return out.toString().getBytes();
			}
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		return template.getPageBytes();
	}

	private byte[] removeQueuedTask() throws Exception
	{
		int id = Integer.parseInt(m_urlData.getParameter("id"));

		ThreadLock.getInstance().getLock();
		try
		{
			m_store.m_waitingTaskList.remove(id);
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return out.toString().getBytes();
	}
	
	private byte[] removeFinishedTask() throws Exception
	{
		int id = Integer.parseInt(m_urlData.getParameter("id"));

		ThreadLock.getInstance().getLock();
		try
		{
			m_store.m_finishedTaskList.remove(id);
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		StringBuffer out = new StringBuffer(256);
		out.append("HTTP/1.0 302 Moved Temporarily\n");
		out.append("Location: /servlet/" + m_urlData.getServletClass() + "\n\n");

		return out.toString().getBytes();
	}
	
	private byte[] showTaskList() throws Exception
	{
		StringBuffer buff = new StringBuffer(2048);
		StringBuffer buff2 = new StringBuffer(2048);
		StringBuffer buff3 = new StringBuffer(2048);
		PageTemplate template = new PageTemplate(m_headers, m_urlData, "RunningTaskList.html", true);

		ThreadLock.getInstance().getLock();
		try
		{
			String[] keys = (String[])m_store.m_runningTaskList.keySet().toArray(new String[0]);

			Arrays.sort(keys);

			for(int x = 0; x  < keys.length; x++)
			{
				TaskItemThread item = (TaskItemThread)m_store.m_runningTaskList.get(keys[x]);

				buff.append("<tr><td>" + item.getCreationDate().toString() + "</td><td>" + item.getStartedDate().toString() + "</td><td align='center'>" + item.commandLine + "</td><td align='center'>" + item.getDelayLeft() + "</td>");

				buff.append("<td><a class='noUnder' href='/servlet/" + m_urlData.getServletClass() + "?action=01&id=" + Formatter.encodeURL(keys[x], "UTF-8") + "'><img align='absmiddle' src='/images/log.png' border='0' alt='Show Output' title='Show Output' width='24' height='24'></a> ");

				buff.append(" <a class='noUnder' onClick='return confirmAction(\"Kill\");' href='/servlet/" + m_urlData.getServletClass() + "?action=02&id=" + Formatter.encodeURL(keys[x], "UTF-8") + "'><img align='absmiddle' src='/images/stop.png' border='0' alt='Kill' title='Kill' width='24' height='24'></a> ");

				buff.append("</td></tr>\n");
			}
			
			for (int i=0; i<m_store.m_waitingTaskList.size(); i++)
			{
				TaskItem item = m_store.m_waitingTaskList.get(i);
				
				buff2.append("<tr><td>" + item.getCreationDate().toString() + "</td><td align='center'>" + item.getFormattedCommand() + "</td>");

				buff2.append("<td><a class='noUnder' onClick='return confirmAction(\"Delete\");' href='/servlet/" + m_urlData.getServletClass() + "?action=05&id=" + i + "'><img align='absmiddle' src='/images/delete.png' border='0' alt='Delete' title='Delete' width='24' height='24'></a>");

				buff2.append("</td></tr>\n");
			}
			
			for (int i=0; i<m_store.m_finishedTaskList.size(); i++)
			{
				TaskItemThread item = m_store.m_finishedTaskList.get(i);
				
				buff3.append("<tr><td>" + item.getCreationDate().toString() + "</td><td>" + item.getStartedDate().toString() + "</td><td>" + item.getFinishedDate().toString() + "</td><td align='center'>" + item.commandLine + "</td>");

				buff3.append("<td><a class='noUnder' href='/servlet/" + m_urlData.getServletClass() + "?action=06&id=" + i + "'><img align='absmiddle' src='/images/log.png' border='0' alt='Show Output' title='Show Output' width='24' height='24'></a> ");

				buff3.append(" <a class='noUnder' onClick='return confirmAction(\"Delete\");' href='/servlet/" + m_urlData.getServletClass() + "?action=07&id=" + i + "'><img align='absmiddle' src='/images/delete.png' border='0' alt='Delete' title='Delete' width='24' height='24'></a>");

				buff3.append("</td></tr>\n");
			}
		}
		finally
		{
			ThreadLock.getInstance().releaseLock();
		}

		template.replace("$taskList", buff.toString());
		template.replace("$taskQueue", buff2.toString());
		template.replace("$taskFinished", buff3.toString());
		return template.getPageBytes();
	}
}
