/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.util.*;
import java.io.*;
import java.text.*;

public class TimerThread implements Runnable
{
	private boolean lowSpaceWarningSent = false;
	private DataStore m_store = null;
	private Calendar now = Calendar.getInstance();
	private Calendar start = Calendar.getInstance();
	private Calendar stop = Calendar.getInstance();
	private HashMap<String, ScheduleItem> times = null;
	private long autoDelSchedTime = 0;

	private String scheduleOptions = "";
	private String[] schOptsArray = new String[0];

	File m_fileCaptureInfo = null;
	String m_nowRunningXmlSaveTo = "nowRunning.xml";
	boolean bSoonPrev = false;
	long wakeUpTimePrev = Long.MAX_VALUE;
	boolean m_bNoNextPrev = true;
	int m_nRecordingErrorCountPrev = -1;
	long m_nanoTimePrev = 0;
	int m_indicateAlive = 0;
	int m_pid = 0;
	int m_taskRunningPrev = 0;

	public TimerThread() throws Exception
	{
		DVScheduler.log("Timer Thread: Created");
		m_fileCaptureInfo = new File(DataStore.m_saveFolder + "nowRunning.txt");
		m_nowRunningXmlSaveTo = DataStore.m_saveFolder + "nowRunning.xml";
		m_store = DataStore.getInstance();
		DllWrapper capEng = new DllWrapper();
		m_pid = capEng.getCurrentProcessId();
		writeCaptureInfoFull();
	}

	public void run()
	{
		DVScheduler.log("Timer Thread: Started");
		ThreadLock locker = ThreadLock.getInstance();
		CaptureDeviceList deviceList = CaptureDeviceList.getInstance();
		GuideStore guideStore = GuideStore.getInstance();

		try
		{
			int adLoops = 30;
			long inUseCounter = 0;
			int timer = 0;
			int timerDivisor = 10;
			long lastPreviewOrSurf = 0;

			while (true)
			{
				m_store.timerStatus = 1;
				m_store.m_bSoonCapturing = false;

				locker.getLock();
				try
				{
					// Auto Delete any old files marked for auto deletion
					// do this Every 30 seconds
					// See if we need to import any schedule info .
					// Also get the Schedule Item Autodelete time.

					adLoops += timer / timerDivisor;
					if(adLoops >= 30)
					{
						//
						// Nuke of AutoDelete files
						//
						try
						{
							nukeOldFiles();
						}
						catch (Exception e)
						{
							e.printStackTrace();
						}

						//
						// Calculate the Autodelete Schedule Interval.
						//
						try
						{
							autoDelSchedTime = 3600000 * Integer.parseInt(m_store.getProperty("sch.autodel", "180"));
						}
						catch (Exception e)
						{
						}
					}

					try
					{
						if (!guideStore.checkReloadInternalEpg())
							loadInternalEpg();
					}
					catch (Exception e)
					{
					}

					//////////////////////////////////////////////////////////////////////////////////////////
					// Look at all the running captures
					//
					m_store.timerStatus = 2;
					String[] keys = (String[]) m_store.m_captureTasks.keySet().toArray(new String[0]);

					for (int index = 0; index < keys.length; index++)
					{
						CaptureTask task = (CaptureTask) m_store.m_captureTasks.get(keys[index]);

						boolean fin = task.isFinished();

						if(fin)
						{
							m_store.timerStatus = 3;
							task.stopCapture();
							try
							{
								m_store.addCaptureTime(task.getCaptureTime());
							}
							catch (Exception e)
							{
								e.printStackTrace();
							}
							m_store.timerStatus = 4;

							//Now Send update message to Listening App
							writeCaptureInfoFull();

							// set the in use timer to zero as
							// we now have a free card
							inUseCounter = 0;

							// Force a Garbage Collection when we stop a capture
							System.gc();
						}
					}

					//
					//////////////////////////////////////////////////////////////////////////////////////////

					// 
					// Now look at all the schedules to see if any need action
					//     
					now.setTime(new Date());

					if ( (now.getTimeInMillis() >= m_store.m_wakeUpTime) && (now.getTimeInMillis() < m_store.m_wakeUpEventTime) )
					{
						DllWrapper capEng = new DllWrapper();
						m_store.m_wakeUpTime = m_store.m_wakeUpEventTime;
						Calendar wakeAt = Calendar.getInstance();
						wakeAt.setTimeInMillis(m_store.m_wakeUpEventTime);

						int sec = wakeAt.get(Calendar.SECOND);
						int min = wakeAt.get(Calendar.MINUTE);
						int hour = wakeAt.get(Calendar.HOUR_OF_DAY);
						int day = wakeAt.get(Calendar.DATE);
						int month = wakeAt.get(Calendar.MONTH) + 1;
						int year = wakeAt.get(Calendar.YEAR);

						DVScheduler.log("About to set secondary wake-up time " + day + "/" + month + "/" + year + " " + hour + ":" + min + ":" + sec + ":");

						int result = capEng.setWakeUpTime(year, month, day, hour, min, sec);

						if(result != 0)
							DVScheduler.log("SetWaitableTimer FAILED! (" + result + ")");
						else
							DVScheduler.log("SetWaitableTimer Succeeded (" + result + ")");
					}

					times = m_store.getScheduleList();
					keys = (String[]) times.keySet().toArray(new String[0]);
					Arrays.sort(keys);

					for (int x = 0; x < keys.length; x++)
					{
						ScheduleItem item = (ScheduleItem) times.get(keys[x]);

						if(item == null)
						{
							DVScheduler.log("ERROR: for some reason one of your schedule items in the MAP is null : " + keys[x]);
							break;
						}

						// only look at schedules that are not in the running list.
						if(m_store.m_captureTasks.keySet().contains(item.toString()) == false)
						{
							start.setTime(item.getStart());
							stop.setTime(item.getStop());

							//
							// Mark items as skipped if in the past and still waiting
							//
							if(item.getState() == ScheduleItem.WAITING && now.after(stop))
							{
								item.setState(ScheduleItem.SKIPPED);
								item.setStatus("Skipped");
								item.log("Marked as skipped");
								m_store.saveSchedules(item);
							}

							//
							// for repeating Schedule items create a new
							// one for the next trigger time
							//
							else if((item.getType() == ScheduleItem.DAILY || item.getType() == ScheduleItem.WEEKLY || item.getType() == ScheduleItem.WEEKDAY || item.getType() == ScheduleItem.MONTHLY || item.getType() == ScheduleItem.EPG_SCAN)
									&&
									(item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.SKIPPED))
							{
								ScheduleItem nextInstance = item.createNextInstance(now, m_store.m_rand);
								m_store.addScheduleItemAndSave(nextInstance);
							}                     

							//
							// Check to see if any Schedule items are to be
							// autodeleted
							//
							else if(autoDelSchedTime > 0 &&
									(item.getState() == ScheduleItem.FINISHED || item.getState() == ScheduleItem.SKIPPED) && 
									(now.getTimeInMillis() - stop.getTimeInMillis()) > autoDelSchedTime)
							{
								m_store.autoDelLogAdd("Auto-Deleting Past Item: (" + item.getName() + 
										") (" + item.getStart() + ") (" + item.getDuration() + ")");
								DVScheduler.log("Deleting Finished Item: " + keys[x]);
								m_store.removeScheduleItem(keys[x]);
								m_store.saveSchedules(item);
							}

							//
							// Here a schedule should not be in RUNNING state so set it
							// to waiting
							// if it is past it will be set to skipped, if it is still
							// current it will be restarted
							//
							else if(item.getState() == ScheduleItem.RUNNING)
							{
								item.setState(ScheduleItem.WAITING);
								item.setStatus("Reset");
								item.log("Running Item marked as Waiting, this should not happen.");
								m_store.saveSchedules(item);
							}

							//
							// an item should also not be able to get to this point and
							// still be in the
							// aborted state so set it to FINISHED.
							//
							else if ( (item.getState() == ScheduleItem.ABORTED) || (item.getState() == ScheduleItem.ABORTED_DELETEFILES) )
							{
								item.setState(ScheduleItem.FINISHED);
								item.setStatus("Reset");
								item.log("Aborted Item marked as Finished, this should not happen.");
								m_store.saveSchedules(item);
							}

							//
							// We should never see RESTART items here as they should be reset by
							// the stop capture call above before getting to this point so if an
							// item state is RESTART then restart it here.
							// This usually means WS crashed!
							//
							else if(item.getState() == ScheduleItem.RESTART)
							{
								item.setState(ScheduleItem.WAITING);
								item.setStatus("Restarted");
								item.log("Item Restarted and Set to WAITING by Timer Thread, this should not happen.");
								m_store.saveSchedules(item);
							}

							//
							// try to start a capture if it needs starting
							//
							else if(item.getState() == ScheduleItem.WAITING && (now.compareTo(start) >= 0) && now.before(stop))
							{
								m_store.m_bSoonCapturing = true;

								try
								{
									long currentNano = System.nanoTime();
									Channel channel = m_store.getChannel(item.getChannel());
									if (channel == null)
									{
										item.incrementErrorCount();
										item.setState(ScheduleItem.ERROR);
										item.setStatus("Error channel not found");
										item.log("Channel object was not found in Channel List");
										m_store.m_nRecordingErrorCount++;
										m_store.saveSchedules(item);
									}
									else if ( ((inUseCounter <= 0) || (item.getTryTime() < currentNano)) && 
											(channel.isGroup() || deviceList.isAvailableDevice(channel.getNetworkType(), channel.getHardSetting(), channel.getSetting())) )
									{
										// Force a Garbage Collection before we start
										System.gc();

										m_store.refreshWakeupTime();

										DVScheduler.log("Item:" + item.getName() + 
												", Channel:" + item.getChannel() + 
												", Number of devices available (" + 
												deviceList.getDeviceCount(channel.getNetworkType()) + 
												") number in use (" + 
												deviceList.getActiveDeviceCount(channel.getNetworkType()) + ")");

										CaptureTask capTask = new CaptureTask(item);
										//store.captureTasks.put(item.toString(), capTask);
										//item.setState(ScheduleItem.RUNNING);
										//item.setStatus("Starting");

										m_store.timerStatus = 7;
										int startCode = capTask.startCapture();
										m_store.timerStatus = 8;

										if ( (startCode == -4) && channel.isGroup() )
										{
											if (item.isLive())
											{
												m_store.removeScheduleItem(item.toString());
											}
											else
											{
												inUseCounter = 60;
												item.setTryTime(System.nanoTime() + inUseCounter * 1000000000L);

												DVScheduler.log("Item:" + item.getName() + 
														", Channel:" + item.getChannel() + 
														", No device available, will try again in " + inUseCounter + " seconds.");
											}

											m_store.saveSchedules(item);
										}
										else if (startCode < 0)
										{
											//store.captureTasks.remove(item.toString());
											item.notifyResponse();
											if (item.isLive())
											{
												m_store.removeScheduleItem(item.toString());
											}
											else if (item.getState() == ScheduleItem.ERROR)
											{
												DVScheduler.log("Item:" + item.getName() + 
														", Channel:" + item.getChannel() + 
														", Fatal error with capture : (" + startCode + ")");
											}
											else
											{
												inUseCounter = 60;
												try
												{
													inUseCounter = Integer.parseInt(m_store.getProperty("Capture.retryWait", "60"));
												}
												catch (Exception e) {}
												if (inUseCounter < 5)
													inUseCounter = 5;
												else if (inUseCounter > 1800)
													inUseCounter = 1800;

												item.setTryTime(System.nanoTime() + inUseCounter * 1000000000L);

												DVScheduler.log("Item:" + item.getName() + 
														", Channel:" + item.getChannel() + 
														", Error starting capture : (" + startCode + ") " + "Will retry in " + (inUseCounter) + " seconds.");
												//DVScheduler.log("See the DirectShow.log file for more details.");
											}

											m_store.saveSchedules(item);
										}
										else
										{
											item.notifyResponse();

											// Now Send update message to Listening App
											writeCaptureInfoFull();

											m_store.saveSchedules(item);
											m_store.setToDo(true);
											break;	// Give other thread time.
										}
									}
									else if ( (inUseCounter <= 0) || (item.getTryTime() < currentNano) )
									{
										if (item.isLive())
										{
											item.abort();
											item.setStatus("No device available");
											item.setState(ScheduleItem.ERROR);
											m_store.removeScheduleItem(item.toString());
										}
										else
										{
											inUseCounter = 60;
											item.setTryTime(System.nanoTime() + inUseCounter * 1000000000L);

											DVScheduler.log("Item:" + item.getName() + 
													", Channel:" + item.getChannel() + 
													", No device available " + 
													"(" + deviceList.getDeviceCount(channel.getNetworkType()) + 
													", " + deviceList.getActiveDeviceCount(channel.getNetworkType()) + 
													"), will try again in " + inUseCounter + " seconds.");
										}
									}

									//if(inUseCounter > 0)
									//   DVScheduler.log("In Use Counter = " + inUseCounter); 
								}
								catch (Exception e)
								{
									DVScheduler.log("Error running the action task: " + e);
									e.printStackTrace();
								}

								//break;
							}
						}
					} // end for

					if (deviceList.isPreviewOrSurfOrEpgScan() == false)
					{
						if ( (deviceList.getRunningDeviceCount() > 0) && ((System.nanoTime() - lastPreviewOrSurf) > 10000000000L) )
							deviceList.stopIdleRunningDevices();
					}
					else
						lastPreviewOrSurf = System.nanoTime();

					now.setTime(new Date());

					if(adLoops >= 30)
					{
						//
						// Import any guide info if required
						//
						scheduleOptions = m_store.getProperty("guide.source.schedule", "0:0:0:1:10:1:24");
						schOptsArray = scheduleOptions.split(":");

						// if the first part is 1 meaning schedule data load is enabled
						// then look at the time
						if (schOptsArray.length >= 3 && "1".equals(schOptsArray[0]) && m_store.m_nextEpgLoadTime != Long.MAX_VALUE)
						{
							boolean bWhenNotCapturing = false;
							int minIdleTime = 0;

							try
							{
								if ( (schOptsArray.length >= 4) && schOptsArray[3].equals("1") )
									bWhenNotCapturing = true;

								if (schOptsArray.length >= 5)
									minIdleTime = Integer.parseInt(schOptsArray[4]);
							}
							catch (Exception e)
							{
							}

							int maxConcurrentTask = 0;
							boolean bRunPreTask = false;
							String preActionRun = m_store.getProperty("guide.preActionRun", "");
							if (preActionRun.equals("All"))
								bRunPreTask = true;
							else if ( preActionRun.equals("Scheduled Data Load") )
								bRunPreTask = true;

							if (bRunPreTask)
							{
								String preTaskName = m_store.getProperty("Tasks.PreEPGloadTask", "");
								HashMap<String, TaskCommand> tasks = m_store.getTaskList();
								TaskCommand taskCommand = (TaskCommand) tasks.get(preTaskName);

								if (taskCommand != null)
								{
									if (!bWhenNotCapturing)
										bWhenNotCapturing = taskCommand.isWhenNotCapturing();

									maxConcurrentTask = taskCommand.getMaxConcurrentTask();

									if (taskCommand.getMinIdleTime() > minIdleTime)
										minIdleTime = taskCommand.getMinIdleTime();									
								}
							}

							boolean bRunTask = true;
							if ( (m_store.m_bSoonCapturing || !m_store.m_captureTasks.isEmpty() || (deviceList.getRunningDeviceCount() > 0)) && (bWhenNotCapturing || (minIdleTime > 0)) )
								bRunTask = false;

							if ( bRunTask && (maxConcurrentTask > 0) && (m_store.m_runningTaskList.size() >= maxConcurrentTask) )
								bRunTask = false;

							if ( bRunTask && (minIdleTime > 0) && ((m_store.m_nextScheduleTime - now.getTimeInMillis()) <= (minIdleTime*60*1000)) )
								bRunTask = false;

							String source = m_store.getProperty("guide.source.type", "Internal");
							if ( bRunTask && (source.isEmpty() || (!source.contains("File") && !source.contains("Http"))) )
								bRunTask = false;

							// if autoLoad time is in past, i.e it is time to do our
							// thing then do it and add the specified number of hours to the next run time.
							if (bRunTask && (m_store.m_nextEpgLoadTime < now.getTimeInMillis()))
							{
								DVScheduler.log("Auto Loading Guide Data");

								// add a time period until next run is in the future
								int epgLoadPeriod = 24;	// default is every 24 hours.
								if (schOptsArray.length >= 7)
									epgLoadPeriod = Integer.parseInt(schOptsArray[6]);
								if (epgLoadPeriod <= 0)
									epgLoadPeriod = 24;
								while (m_store.m_nextEpgLoadTime < now.getTimeInMillis())
								{
									m_store.m_nextEpgLoadTime += epgLoadPeriod * 3600 * 1000;
								}

								m_store.refreshWakeupTime();

								if ( m_store.isEpgLoading() )
								{
									DVScheduler.log("Aborted loading guide data, as one is already running");
								}
								else
								{
									// Use the Sch Guide Loader Thread to do the actual
									// loading
									SchGuideLoaderTread sch = new SchGuideLoaderTread(false, null);
									m_store.m_epgLoaderThread = new Thread(Thread.currentThread().getThreadGroup(), sch, sch.getClass().getName());
									m_store.m_epgLoaderThread.start();
								}
							}
							else if (m_store.m_nextEpgLoadWakeUpTime < now.getTimeInMillis())
								m_store.refreshWakeupTime();
						}

						if ( !m_store.isEpgLoading() )
							scheduleInternalEpg();

						// reset adLoops
						adLoops = 0;
					}

					writeCaptureInfo(m_store.m_bSoonCapturing, m_store.m_wakeUpTime, false);

					if (!m_store.m_waitingTaskList.isEmpty())
					{
						TaskItem task = m_store.m_waitingTaskList.get(0);
						boolean bRunTask = true;
						if ( (m_store.m_bSoonCapturing || !m_store.m_captureTasks.isEmpty()) && (task.isWhenNotCapturing() || (task.getMinIdleTime() > 0)) )
							bRunTask = false;

						if ( bRunTask && (task.getMaxConcurrentTask() > 0) && (m_store.m_runningTaskList.size() >= task.getMaxConcurrentTask()) )
							bRunTask = false;

						if ( bRunTask && (task.getMinIdleTime() > 0) && ((m_store.m_nextScheduleTime - now.getTimeInMillis()) <= (task.getMinIdleTime()*60*1000)) )
							bRunTask = false;

						if (bRunTask)
						{
							try
							{
								ScheduleItem item = task.getScheduleItem();
								if (item != null)
									item.log("Running task (" + task.getFormattedCommand() + ")");
								else
									DVScheduler.log("Running task (" + task.getFormattedCommand() + ")");

								CommandWaitThread comThread = new CommandWaitThread(task.getFormattedCommand(), task.getDelay());
								Thread comWait = new Thread(Thread.currentThread().getThreadGroup(), comThread, comThread.getClass().getName());
								comWait.start();
								String key = new Long(task.getCreationDate().getTime()).toString() + "-" + new Long(m_store.m_rand.nextLong()).toString();
								TaskItemThread taskItem = new TaskItemThread(key, comThread, task.getFormattedCommand(), task.getAutoRemove(), task.getCreationDate());
								taskItem.m_allowSuspend = task.isAllowSuspend();
								Thread taskThread = new Thread(Thread.currentThread().getThreadGroup(), taskItem, taskItem.getClass().getName());
								taskThread.start();
								m_store.m_runningTaskList.put(key, taskItem);
								m_store.m_waitingTaskList.remove(0);
							}
							catch (Exception e)
							{
								DVScheduler.log("Error running queued task: " + e);
								e.printStackTrace();
							}
						}
					}
				} // lock
				finally
				{
					locker.releaseLock();
				}

				m_store.timerStatus = 0;
				for (timer=0; timer<50; timer++)
				{
					if (m_store.isToDo())
					{
						m_store.setToDo(false);
						break;
					}

					waitResponse();
					//Thread.sleep(100);
				}

				// decrement the in use counter this gives a time out 
				// before tyring to start a capture again when 
				// no cards are available so we dont hammer the
				// system to death
				inUseCounter -= timer / timerDivisor;
			} // end while
		}
		catch (Exception e)
		{
			DVScheduler.log("The main Timer Thread has crashed!");
			e.printStackTrace();
			DVScheduler.log("This is really bad!!!!!");
			m_store.timerStatus = -1;

			ByteArrayOutputStream ba = new ByteArrayOutputStream();
			PrintWriter err = new PrintWriter(ba);
			e.printStackTrace(err);
			err.flush();
			m_store.timerThreadErrorStack = ba.toString();

			//DVScheduler.log("System Exiting!");
			//System.exit(-10);
		}
	}

	public synchronized void notifyResponse()
	{
		notify();
	}

	public synchronized void waitResponse()
	{
		try {
			wait(100);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	protected long getMax(long val1, long val2)
	{
		return val1 > val2 ? val1 : val2;
	}

	private void nukeOldFiles()
	{
		try
		{
			long spaceLimit = m_store.getPropertyLong("AutoDel.FreeSpaceLimit", -1) * 1024 * 1024;
			long lowSpaceWarning = m_store.getPropertyLong("email.LowSpaceWarning", -1) * 1024 * 1024;

			boolean bUpdated = false;

			//DVScheduler.log("Free Space Limit : " + spaceLimit);
			//DVScheduler.log("Nuking AutoDel Files");

			HashMap<String, KeepForDetails> adList = m_store.getAutoDelList();
			String[] key = (String[]) adList.keySet().toArray(new String[0]);
			Arrays.sort(key);

			Calendar limit = Calendar.getInstance();
			Calendar itemDate = Calendar.getInstance();
			boolean bAutoDelOverride = m_store.isAutoDelOverride();
			int keepForMinutes = m_store.getPropertyInt("AutoDel.OverrideMinutes", 0);
			for (int x = 0; x < key.length; x++)
			{
				KeepForDetails item = (KeepForDetails)adList.get(key[x]);

				File delFile = new File(item.getFileName());

				limit.setTime(new Date());
				if (!bAutoDelOverride)
					keepForMinutes = item.getKeepForMinutes();
				limit.add(Calendar.MINUTE, (-1 * keepForMinutes));

				itemDate.setTime(item.getCreated());

				if(delFile.exists() == false)
				{
					m_store.autoDelLogAdd("AutoDelete : " + item.getCreated().toString() + " - " + item.getFileName() + " : No longer exists, removing");
					DVScheduler.log("AutoDelete : " + item.getCreated().toString() + " - " + item.getFileName() + " : No longer exists, removing");
					adList.remove(key[x]);
					bUpdated = true;
				}
				else if(itemDate.before(limit))
				{
					if (delFile.delete())
					{
						m_store.autoDelLogAdd("AutoDelete : " + item.getCreated().toString() + " - " + item.getFileName() + " : Too old (" + keepForMinutes / 24 / 60 + " days, " + keepForMinutes % (24 * 60) + " mins) DELETED");
						DVScheduler.log("AutoDelete : " + item.getCreated().toString() + " - " + item.getFileName() + " : Too old (" + keepForMinutes / 24 / 60 + " days, " + keepForMinutes % (24 * 60) + " mins) DELETED");
						adList.remove(key[x]);
						bUpdated = true;
					}
				}
			}

			if (bUpdated)
				m_store.saveAutoDelList();

			// Get current free space
			String[] paths = DVScheduler.getArray(m_store.getProperty("Capture.path", "capture"));
			HashMap<Long, String> pathFreeSpaces = new HashMap<Long, String>();
			boolean bLow = true;
			boolean bDelete = false;
			for(int x = 0; x < paths.length; x++)
			{
				if (paths[x].isEmpty())
					paths[x] = ".";

				File cap = new File(paths[x]);
				if (!cap.exists())
					cap.mkdirs();
				if (!cap.exists())
					continue;
				String fullPath = cap.getAbsolutePath();
				long freeSpace = cap.getFreeSpace();
				pathFreeSpaces.put(freeSpace, fullPath);

				if (freeSpace > lowSpaceWarning)
				{
					bLow = false;
				}

				if (freeSpace <= spaceLimit)
				{
					bDelete = true;
				}
			}
			Long[] keyPathFreeSpaces = (Long[])pathFreeSpaces.keySet().toArray(new Long[0]);
			Arrays.sort(keyPathFreeSpaces);

			//DVScheduler.log("Free space = " + freeSpace + " Warning Level = "
			// + lowSpaceWarning);
			// Send warning Email is required
			if(bLow && !lowSpaceWarningSent)
			{
				DVScheduler.log("Sending Low Disk Space Warning Email");

				NumberFormat nf = NumberFormat.getInstance();

				EmailSender sender = new EmailSender();

				sender.setSubject("DV Scheduler Low Disk Space Warning");

				String text = "Disk space on capture drives are low:";
				for (int x = 0; x < keyPathFreeSpaces.length; x++)
				{
					text += "\n\"" + (String)pathFreeSpaces.get(keyPathFreeSpaces[x]) + "\" has " + nf.format(keyPathFreeSpaces[x]) + " bytes left.";
				}
				sender.setBody(text);

				Thread mailThread = new Thread(Thread.currentThread().getThreadGroup(), sender, sender.getClass().getName());
				mailThread.start();
				lowSpaceWarningSent = true;
			}
			else if(!bLow)
			{
				lowSpaceWarningSent = false;
			}

			// Check for free space and delete auto deletable files if needed
			if(!bDelete)
				return;

			bUpdated = false;
			adList = m_store.getAutoDelList();
			key = (String[]) adList.keySet().toArray(new String[0]);

			Arrays.sort(key);

			for (int x = 0; x < key.length; x++)
			{
				KeepForDetails item = (KeepForDetails) adList.get(key[x]);
				String fileName = item.getFileName();

				File delFile = new File(fileName);

				if(delFile.exists() == false)
				{
					m_store.autoDelLogAdd("AutoDelete : " + key[x].toString() + " - " + fileName + " : No longer exists, removing");
					DVScheduler.log("AutoDelete : " + key[x].toString() + " - " + fileName + " : No longer exists, removing");
					adList.remove(key[x]);
					bUpdated = true;
				}
				else
				{
					long freeSpace = getMax(delFile.getFreeSpace(), new File(delFile.getParent()).getFreeSpace());

					if (freeSpace < spaceLimit)
					{
						if (delFile.delete())
						{
							m_store.autoDelLogAdd("AutoDelete : " + key[x].toString() + " - " + fileName + " : DELETED to free space");
							DVScheduler.log("AutoDelete : " + key[x].toString() + " - " + fileName + " : DELETED to free space");
							adList.remove(key[x]);
							bUpdated = true;
							//return;
						}
						else
						{
							m_store.autoDelLogAdd("AutoDelete : " + key[x].toString() + " - " + fileName + " : cannot be DELETED");
							DVScheduler.log("AutoDelete : " + key[x].toString() + " - " + fileName + " : cannot be DELETED");							
						}
					}
				}
			}

			if (bUpdated)
				m_store.saveAutoDelList();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private void loadInternalEpg()
	{
		try
		{
			if (!m_store.getProperty("guide.source.type", "Internal").contains("Internal"))
				return;

			File dir = new File(DataStore.m_saveFolder + "xmltvInternal");

			File[] files = dir.listFiles();
			if(files == null)
				return;

			//GuideStore guide = GuideStore.getInstance();
			//long time = new Date().getTime() - 20000;
			for (int i=0; i<files.length; i++)
			{
				try
				{
					if (files[i].length() == 0)
					{
						files[i].delete();
					}
				}
				catch (Exception e) {}

				//if (!files[i].isFile() || (files[i].length() < 10) || (files[i].lastModified() > time))
				if (!files[i].isFile() || (files[i].length() < 10))
					continue;

				if (FileUploader.isFileOpen(files[i]))
					continue;

				//guide.loadInternalXmltv(files[i]);

				if (m_store.isEpgLoading())
					return;
				SchGuideLoaderTread sch = new SchGuideLoaderTread(true, files[i]);
				m_store.m_epgLoaderThread = new Thread(Thread.currentThread().getThreadGroup(), sch, sch.getClass().getName());
				m_store.m_epgLoaderThread.start();
				break;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private void scheduleInternalEpg()
	{
		try
		{
			if (!m_store.getProperty("guide.source.type", "Internal").contains("Internal"))
				return;
			GuideStore guide = GuideStore.getInstance();
			guide.scheduleInternalEpgScan();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	private void writeCaptureInfoFull()
	{
		writeCaptureInfo(m_store.m_bSoonCapturing, m_store.m_wakeUpTime, true);
		NowRunningInfo update = new NowRunningInfo(m_store.m_captureTasks, m_nowRunningXmlSaveTo);
		update.writeNowRunning();
	}

	private void writeCaptureInfo(boolean bSoon, long wakeUpTime, boolean bWrite)
	{
		boolean bNoNext = wakeUpTime == Long.MAX_VALUE;

		/*
		int allow = 0;
		try
		{
			allow = Integer.parseInt(store.getProperty("Schedule.wake.system", "45").trim());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}

		if (allow == 0)
			bNoNext = true;

		wakeUpTime.setTime( wakeUpTime.getTime() - allow * 1000 );
		 */

		int taskRunning = m_store.m_runningTaskList.size();
		Iterator<TaskItemThread> iter = m_store.m_runningTaskList.values().iterator();
		while (iter.hasNext())
		{
			TaskItemThread item = iter.next();
			// Don't count task that allow system to sleep.
			if (item.m_allowSuspend)
				taskRunning--;
		}

		bSoon |= (wakeUpTime - now.getTimeInMillis()) < (2*60*1000);

		boolean bIndicateAlive = System.nanoTime() > (m_nanoTimePrev + 30 * 1000000000L);

		if ( bWrite || (bSoon != bSoonPrev) || (bNoNext != m_bNoNextPrev) ||
				(wakeUpTime != wakeUpTimePrev) || (m_store.m_nRecordingErrorCount != m_nRecordingErrorCountPrev) || 
				(taskRunning != m_taskRunningPrev) || bIndicateAlive || (m_fileCaptureInfo.length() < 2) )
		{
			try
			{
				if (bIndicateAlive)
				{
					if (++m_indicateAlive > 9)
						m_indicateAlive = 0;
				}

				FileWriter writerCapCount = new FileWriter(m_fileCaptureInfo);

				writerCapCount.write((bSoon ? "1" : "0") + 

						(bNoNext ? String.format("%1$016x", 0x0L) : 
							String.format("%1$016x", wakeUpTime)) + 

							String.format("%1$08x", m_store.m_captureTasks.size()) + 
							String.format("%1$08x", m_store.m_nRecordingErrorCount - m_store.m_nLastRecordingErrorCount) + 
							m_indicateAlive + 
							String.format("%1$08x", m_pid) + 
							String.format("%1$08x", taskRunning));

				writerCapCount.close();
				bSoonPrev = bSoon;
				wakeUpTimePrev = wakeUpTime;
				m_bNoNextPrev = bNoNext;
				m_nRecordingErrorCountPrev = m_store.m_nRecordingErrorCount;
				m_taskRunningPrev = taskRunning;
				m_nanoTimePrev = System.nanoTime();
			}
			catch (Exception e)
			{
				DVScheduler.log("Error when writing nowRunning.txt : " + e.toString());
				e.printStackTrace();
			}
		}
	}
}
