/*
   Copyright (C) 2007-2010 contributors

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 */

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

class TunerScanResult
{
	private Vector<CaptureDevice> scanResult = new Vector<CaptureDevice>();
	StringBuffer xmlData = null;

	public TunerScanResult()
	{

	}

	public int readInput(InputStream inStream) throws Exception
	{
		BufferedReader reader = new BufferedReader(new InputStreamReader(inStream, "UTF-8"));
		xmlData = new StringBuffer();

		boolean bStart = false;
		String line = reader.readLine();
		while(line != null)
		{
			if(line.length() > 0)
			{
				if ( !bStart && line.contains("<devices"))
					bStart = true;

				if (bStart)
					xmlData.append(line + "\n");

				DVScheduler.log(line);

				if (line.contains("</devices"))
					break;
			}
			line = reader.readLine();
		}      

		return xmlData.length();
	}

	public int parseXML(int indexBase) throws Exception
	{
		try
		{
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();

			ByteArrayInputStream reader = new ByteArrayInputStream(xmlData.toString().getBytes("UTF-8"));
			Document doc = docBuilder.parse(reader);   

			NodeList devices = doc.getElementsByTagName("device");

			for(int x = 0; x < devices.getLength(); x++)
			{
				Node device = devices.item(x);

				NodeList nl = device.getChildNodes();   

				String name = "";
				String id = "";
				String uniqueName = "";
				SortedSet<String> networkTypes = new TreeSet<String>();

				for(int y = 0; y < nl.getLength(); y++)
				{
					Node item = nl.item(y);

					if(item.getNodeName().equals("name"))
					{            
						name = item.getTextContent();
					}
					else if(item.getNodeName().equals("id"))
					{            
						id = item.getTextContent();
					}
					else if ( (item.getNodeName().equals("displayname")) || 
							(item.getNodeName().equals("uniquename")) )
					{            
						uniqueName = item.getTextContent();
					}            
					else if (item.getNodeName().equals("type"))
					{
						String type = item.getTextContent().trim();
						if (!type.isEmpty())
							networkTypes.add(type);
					}
				}

				if (networkTypes.isEmpty())
					networkTypes.add("DVB-T");

				if(name.length() > 0 && id.length() > 0)
				{
					int idInt = Integer.parseInt(id) - indexBase;
					CaptureDevice dev = new CaptureDevice(name, idInt, uniqueName, networkTypes);
					scanResult.add(dev);
				}
			}

			return scanResult.size();
		}
		catch(Exception e)
		{
			DVScheduler.log("Error parsing device XML.");
			DVScheduler.log(xmlData);
			//e.printStackTrace();
			scanResult = new Vector<CaptureDevice>();
			return 0;
		}
	}

	public Vector<CaptureDevice> getResult()
	{
		return scanResult;
	}

}
