

//#include <tchar.h>

#include "graph.h"
#include "log.h"
//#include <stdio.h>

//#include <string>
#include <sstream>



int _tmain(int argc, _TCHAR* argv[])
{
	if(argc < 4)
		return -1;

	int freq = -1;
	std::string freqS((char*)(argv[1]));
	std::istringstream f(freqS);
	if (!(f >> freq))
		freq = -1;

	int band = -1;
	std::string bandS((char*)(argv[2]));
	std::istringstream b(bandS);
	if (!(b >> band))
		band = -1;

	int deviceID = -1;
	std::string deviceS((char*)(argv[3]));
	std::istringstream d(deviceS);
	if (!(d >> deviceID))
		deviceID = -1;

	if(freq == -1 || band == -1 || deviceID == -1)
		return -2;

	openLogFile("ChannelScan-");

	CoInitialize(NULL);

	CBDAFilterGraph *p_BDAgraph = new CBDAFilterGraph();

	p_BDAgraph->SetFrequency(freq);
	p_BDAgraph->SetBandwidth(band);

	HRESULT hr = p_BDAgraph->BuildGraph(DVB_T, deviceID);
	if(FAILED(hr))
	{
		log("BuildGraph failed with : 0x%x\n", hr);
		delete p_BDAgraph;
		closeLogFile();
		return -3;
	}

	hr = p_BDAgraph->LoadPsiFilter();
	if(FAILED(hr))
	{
		log("LoadPsiFilter failed with : 0x%x\n", hr);
		delete p_BDAgraph;
		closeLogFile();
		return -4;
	}

	hr = p_BDAgraph->ChangeChannel();
	if(FAILED(hr))
	{
		log("ChangeChannel failed with : 0x%x\n", hr);
		delete p_BDAgraph;
		closeLogFile();
		return -5;
	}

	hr = p_BDAgraph->RunGraph(NULL);
	if(FAILED(hr))
	{
		log("RunGraph failed with : 0x%x\n", hr);
		p_BDAgraph->StopGraph();
		delete p_BDAgraph;
		closeLogFile();
		return -6;
	}

	std::vector<CProgramInfo> ppinfo;
	hr = p_BDAgraph->getProgramInfo(&ppinfo);
	if(FAILED(hr))
	{
		log("getProgramInfo failed with : 0x%x\n", hr);
		p_BDAgraph->StopGraph();
		delete p_BDAgraph;
		closeLogFile();
		return -7;
	}

	hr = p_BDAgraph->StopGraph();

	delete p_BDAgraph;

	CoUninitialize();

	log("Scan Finished now sending XML\n");

	std::string xmlOut = "";

	xmlOut.append("<programs>\n");
	char buff[256];

	for(std::vector<CProgramInfo>::size_type x = 0; x < ppinfo.size(); x++)
	{
		CProgramInfo prog = ppinfo.at(x);
		xmlOut.append("<program>\n");
		
		StringCchPrintf(buff, 256, "<name>%s</name>\n", prog.programName);
		xmlOut.append(buff);
		StringCchPrintf(buff, 256, "<id>%d</id>\n", prog.programID);
		xmlOut.append(buff);

		for(std::vector<CStreamInfo>::size_type y = 0; y < prog.streams.size(); y++)
		{
			CStreamInfo si = prog.streams.at(y);
			xmlOut.append("<stream>\n");
			StringCchPrintf(buff, 256, "<id>%d</id>\n", si.getID());
			xmlOut.append(buff);
			StringCchPrintf(buff, 256, "<type>%d</type>\n", si.getType());
			xmlOut.append(buff);
			xmlOut.append("</stream>\n");
		}

		xmlOut.append("</program>\n");
	}

	xmlOut.append("</programs>\n");

	printf(xmlOut.c_str());
	fflush(stdout);

	log("\n%s", xmlOut.c_str());

	closeLogFile();
	ExitProcess(0);

	return 0;
}

